Package: mpactr
Title: Correction of Preprocessed MS Data
Version: 0.3.3
Authors@R: c(
    person("Allison", "Mason", , "masonar@umich.edu", role = c("aut"),
            comment = c(ORCID = "0000-0003-1339-1592")),
    person("Gregory", "Johnson", , "grejoh@umich.edu", role = c("aut"),
            comment = c(ORCID = "0009-0008-3890-0297")),
    person("Patrick", "Schloss", , "pschloss@umich.edu", role = c("aut", "cre", "cph"),
            comment = c(ORCID = "0000-0002-6935-4275"))
    )
Description: An 'R' implementation of the 'python' program Metabolomics Peak Analysis Computational Tool ('MPACT') (Robert M. Samples, Sara P. Puckett, and Marcy J. Balunas (2023) <doi:10.1021/acs.analchem.2c04632>). Filters in the package serve to address common errors in tandem mass spectrometry preprocessing, including: (1) isotopic patterns that are incorrectly split during preprocessing, (2) features present in solvent blanks due to carryover between samples, (3) features whose abundance is greater than user-defined abundance threshold in a specific group of samples, for example media blanks, (4) ions that are inconsistent between technical replicates, and (5) in-source fragment ions created during ionization before fragmentation in the tandem mass spectrometry workflow.
Encoding: UTF-8
Suggests: knitr, rmarkdown, testthat (>= 3.0.0), tidyverse, plotly,
        Hmisc, corrplot, ggdendro, ggtext
Config/testthat/edition: 3
Imports: cli, data.table, ggplot2, R6, Rcpp, stats, treemapify, viridis
LazyData: true
URL: https://www.mums2.org/mpactr/, https://github.com/mums2/mpactr
BugReports: https://github.com/mums2/mpactr/issues
VignetteBuilder: knitr
LinkingTo: Rcpp
License: GPL (>= 3)
Config/roxygen2/version: 8.0.0
NeedsCompilation: yes
Packaged: 2026-05-20 15:45:21 UTC; grejoh
Author: Allison Mason [aut] (ORCID: <https://orcid.org/0000-0003-1339-1592>),
  Gregory Johnson [aut] (ORCID: <https://orcid.org/0009-0008-3890-0297>),
  Patrick Schloss [aut, cre, cph] (ORCID:
    <https://orcid.org/0000-0002-6935-4275>)
Maintainer: Patrick Schloss <pschloss@umich.edu>
Depends: R (>= 3.5.0)
Repository: CRAN
Date/Publication: 2026-05-21 17:20:10 UTC
Built: R 4.5.2; x86_64-apple-darwin20; 2026-05-25 01:34:41 UTC; unix
Archs: mpactr.so.dSYM

Package: mpactr
Title: Correction of Preprocessed MS Data
Version: 0.3.3
Authors@R: c(
    person("Allison", "Mason", , "masonar@umich.edu", role = c("aut"),
            comment = c(ORCID = "0000-0003-1339-1592")),
    person("Gregory", "Johnson", , "grejoh@umich.edu", role = c("aut"),
            comment = c(ORCID = "0009-0008-3890-0297")),
    person("Patrick", "Schloss", , "pschloss@umich.edu", role = c("aut", "cre", "cph"),
            comment = c(ORCID = "0000-0002-6935-4275"))
    )
Description: An 'R' implementation of the 'python' program Metabolomics Peak Analysis Computational Tool ('MPACT') (Robert M. Samples, Sara P. Puckett, and Marcy J. Balunas (2023) <doi:10.1021/acs.analchem.2c04632>). Filters in the package serve to address common errors in tandem mass spectrometry preprocessing, including: (1) isotopic patterns that are incorrectly split during preprocessing, (2) features present in solvent blanks due to carryover between samples, (3) features whose abundance is greater than user-defined abundance threshold in a specific group of samples, for example media blanks, (4) ions that are inconsistent between technical replicates, and (5) in-source fragment ions created during ionization before fragmentation in the tandem mass spectrometry workflow.
Encoding: UTF-8
Suggests: knitr, rmarkdown, testthat (>= 3.0.0), tidyverse, plotly,
        Hmisc, corrplot, ggdendro, ggtext
Config/testthat/edition: 3
Imports: cli, data.table, ggplot2, R6, Rcpp, stats, treemapify, viridis
LazyData: true
URL: https://www.mums2.org/mpactr/, https://github.com/mums2/mpactr
BugReports: https://github.com/mums2/mpactr/issues
VignetteBuilder: knitr
LinkingTo: Rcpp
License: GPL (>= 3)
Config/roxygen2/version: 8.0.0
NeedsCompilation: yes
Packaged: 2026-05-20 15:45:21 UTC; grejoh
Author: Allison Mason [aut] (ORCID: <https://orcid.org/0000-0003-1339-1592>),
  Gregory Johnson [aut] (ORCID: <https://orcid.org/0009-0008-3890-0297>),
  Patrick Schloss [aut, cre, cph] (ORCID:
    <https://orcid.org/0000-0002-6935-4275>)
Maintainer: Patrick Schloss <pschloss@umich.edu>
Depends: R (>= 3.5.0)
Repository: CRAN
Date/Publication: 2026-05-21 17:20:10 UTC
Built: R 4.6.0; x86_64-apple-darwin20; 2026-05-25 03:37:48 UTC; unix
Archs: mpactr.so.dSYM

# Generated by roxygen2: do not edit by hand

export(example_path)
export(filter_cv)
export(filter_group)
export(filter_insource_ions)
export(filter_mispicked_ions)
export(filter_summary)
export(get_cv_data)
export(get_group_averages)
export(get_metadata)
export(get_peak_table)
export(get_raw_data)
export(get_similar_ions)
export(import_data)
export(limit_cores)
export(plot_qc_tree)
export(qc_summary)
importFrom(R6,R6Class)
importFrom(Rcpp,sourceCpp)
importFrom(cli,cli_abort)
importFrom(cli,cli_alert_info)
importFrom(cli,cli_alert_success)
importFrom(data.table,address)
importFrom(data.table,as.data.table)
importFrom(data.table,copy)
importFrom(data.table,data.table)
importFrom(data.table,dcast)
importFrom(data.table,fread)
importFrom(data.table,melt)
importFrom(data.table,setDTthreads)
importFrom(data.table,setnames)
importFrom(data.table,setorder)
importFrom(ggplot2,aes)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,is_ggplot)
importFrom(ggplot2,theme)
importFrom(stats,cor)
importFrom(stats,cutree)
importFrom(stats,dist)
importFrom(stats,hclust)
importFrom(stats,sd)
importFrom(treemapify,geom_treemap)
importFrom(treemapify,geom_treemap_text)
importFrom(viridis,scale_fill_viridis)
useDynLib(mpactr, .registration = TRUE)

# mpactr <a href="https://www.mums2.org/mpactr/"><img src="man/figures/logo.png" align="right" height="138" alt="mpactr website" /></a>

<!-- README.md is generated from README.Rmd. Please edit that file -->

<!-- badges: start -->

[![R-CMD-check](https://github.com/mums2/mpactr/actions/workflows/r.yml/badge.svg)](https://github.com/mums2/mpactr/actions/workflows/r.yml)
[![test-coverage](https://github.com/mums2/mpactr/actions/workflows/test-coverage.yml/badge.svg)](https://github.com/mums2/mpactr/actions/workflows/test-coverage.yml)
[![lint](https://github.com/mums2/mpactr/actions/workflows/lintr.yml/badge.svg)](https://github.com/mums2/mpactr/actions/workflows/lintr.yml)
[![pkgdown](https://github.com/mums2/mpactr/actions/workflows/pkgdown.yaml/badge.svg)](https://github.com/mums2/mpactr/actions/workflows/pkgdown.yaml)
<!-- badges: end -->

## Overview

mpactr is a collection of filters for the purpose of identifying high
quality MS1 features by correcting peak selection errors introduced
during the pre-processing of tandem mass spectrometry data.

Filters in this package address the following issues:

- `filter_mispicked_ions()`: removal of mispicked peaks, or those
  isotopic patterns that are incorrectly split during preprocessing.
- `filter_group()`: removal of features overrepresented in a specific
  group of samples; for example removal of features present in solvent
  blanks due to carryover between samples.
- `filter_cv()`: removal of non-reproducible features, or those that are
  inconsistent between technical replicates.
- `filter_insource_ions()`: removal of fragment ions created during the
  first ionization in the tandem MS/MS workflow.

All filters are independent, meaning they can be used to create a
project-specific workflow, or you can learn more in [the Getting Started
page](https://www.mums2.org/mpactr/articles/mpactr.html).

## Installation

You can install the CRAN version with:

``` r
install.packages("mpactr")
```

You can install the development version of mpactr from
[GitHub](https://github.com/mums2/mpactr) with:

``` r
# install.packages("devtools")
devtools::install_github("mums2/mpactr")
```

## Get started

See the [Getting
Started](https://www.mums2.org/mpactr/articles/mpactr.html) page to get
started.

## Getting help

If you encounter an issue, please file an issue on
[GitHub](https://github.com/mums2/mpactr/issues). Please include a
minimal reproducible example with your issue.

## Contributing

Is there a feature you’d like to see included, please let us know! Pull
requests are welcome on [GitHub](https://github.com/mums2/mpactr/pulls).

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

## ----setup, message=FALSE-----------------------------------------------------
library(mpactr)
library(viridis)
library(plotly)
library(data.table)
library(tidyverse)
library(Hmisc)
library(corrplot)
library(ggdendro)
library(ggtext)

## ----include = FALSE----------------------------------------------------------
limit_cores()

## -----------------------------------------------------------------------------
data <- import_data(example_path("cultures_peak_table.csv"),
  example_path("cultures_metadata.csv"),
  format = "Progenesis"
)

## -----------------------------------------------------------------------------
data_filtered <- data |>
  filter_mispicked_ions(merge_peaks = TRUE, merge_method = "sum") |>
  filter_group(group_to_remove = "Solvent_Blank") |>
  filter_group(group_to_remove = "Media") |>
  filter_cv(cv_threshold = 0.2)

## -----------------------------------------------------------------------------
plot_qc_tree(data_filtered)

## -----------------------------------------------------------------------------
get_raw_data(data_filtered) %>%
  select(compound, mz, rt) %>%
  head()

## -----------------------------------------------------------------------------
qc_summary(data_filtered) %>%
  head()

## -----------------------------------------------------------------------------
get_raw_data(data_filtered) %>%
  mutate(compound = as.character(compound)) %>%
  select(compound, mz, rt) %>%
  left_join(qc_summary(data_filtered),
    by = join_by("compound" == "compounds")
  ) %>%
  head()

## -----------------------------------------------------------------------------
get_raw_data(data_filtered) %>%
  mutate(compound = as.character(compound)) %>%
  select(compound, mz, rt) %>%
  left_join(qc_summary(data_filtered),
    by = join_by("compound" == "compounds")
  ) %>%
  ggplot() +
  aes(x = rt, y = mz, color = status) +
  geom_point() +
  viridis::scale_color_viridis(discrete = TRUE) +
  labs(
    x = "Retention time (min)",
    y = "m/z",
    color = "Ion Status"
  ) +
  theme_bw() +
  theme(legend.position = "top")

## -----------------------------------------------------------------------------
feature_plot <- get_raw_data(data_filtered) %>%
  mutate(compound = as.character(compound)) %>%
  select(compound, mz, rt) %>%
  left_join(qc_summary(data_filtered),
    by = join_by("compound" == "compounds")
  ) %>%
  ggplot() +
  aes(
    x = rt, y = mz, color = status,
    text = paste0("compound: ", compound)
  ) +
  geom_point() +
  viridis::scale_color_viridis(discrete = TRUE) +
  labs(
    x = "Retention time (min)",
    y = "m/z",
    color = "Ion Status"
  ) +
  theme_bw() +
  theme(legend.position = "top")

ggplotly(feature_plot)

## -----------------------------------------------------------------------------
ft <- get_peak_table(data_filtered)

ft[1:5, 1:7]

## -----------------------------------------------------------------------------
counts <- ft %>%
  select(compound, all_of(get_metadata(data_filtered)$injection)) %>%
  column_to_rownames(var = "compound") %>%
  select(where(~ sum(.x) != 0))

counts[1:5, 1:2]

## -----------------------------------------------------------------------------
counts_cor <- rcorr(as.matrix(counts), type = "spearman")

## -----------------------------------------------------------------------------
counts_cor$r[, 1]

## ----warning=FALSE------------------------------------------------------------
corrplot(counts_cor$r,
  type = "lower",
  method = "square",
  order = "hclust",
  col = COL2("BrBG", 10),
  tl.col = "black",
  tl.cex = .5
)

## -----------------------------------------------------------------------------
meta <- get_metadata(data_filtered)

counts %>%
  rownames_to_column(var = "compound") %>%
  pivot_longer(
    cols = starts_with("102623"),
    names_to = "injection",
    values_to = "intensity"
  ) %>%
  head()

## -----------------------------------------------------------------------------
counts %>%
  rownames_to_column(var = "compound") %>%
  pivot_longer(
    cols = starts_with("102623"),
    names_to = "injection",
    values_to = "intensity"
  ) %>%
  left_join(meta, by = "injection") %>%
  head()

## -----------------------------------------------------------------------------
counts %>%
  rownames_to_column(var = "compound") %>%
  pivot_longer(
    cols = starts_with("102623"),
    names_to = "injection",
    values_to = "intensity"
  ) %>%
  left_join(meta, by = "injection") %>%
  summarise(
    mean_intensity = mean(intensity),
    .by = c(compound, sample_code)
  ) %>%
  head()

## ----warning=FALSE------------------------------------------------------------
sample_counts <- counts %>%
  rownames_to_column(var = "compound") %>%
  pivot_longer(
    cols = starts_with("102623"),
    names_to = "injection",
    values_to = "intensity"
  ) %>%
  left_join(meta, by = "injection") %>%
  summarise(
    mean_intensity = mean(intensity),
    .by = c(compound, sample_code)
  ) %>%
  pivot_wider(
    names_from = sample_code,
    values_from = mean_intensity
  ) %>%
  column_to_rownames(var = "compound")

sample_counts[1:5, 1:5]

## -----------------------------------------------------------------------------
sample_counts_cor <- rcorr(as.matrix(sample_counts), type = "spearman")

## -----------------------------------------------------------------------------
corrplot(sample_counts_cor$r,
  type = "lower",
  method = "square",
  order = "alphabet",
  col = COL2("BrBG", 10),
  tl.col = "black"
)

## ----warning=FALSE------------------------------------------------------------
group_counts <- counts %>%
  rownames_to_column(var = "compound") %>%
  pivot_longer(
    cols = starts_with("102623"),
    names_to = "injection",
    values_to = "intensity"
  ) %>%
  left_join(meta, by = "injection") %>%
  summarise(
    mean_intensity = mean(intensity),
    .by = c(compound, biological_group)
  ) %>%
  pivot_wider(
    names_from = biological_group,
    values_from = mean_intensity
  ) %>%
  column_to_rownames(var = "compound")

## -----------------------------------------------------------------------------
group_counts_cor <- rcorr(as.matrix(group_counts), type = "spearman")

## -----------------------------------------------------------------------------
corrplot(group_counts_cor$r,
  type = "lower",
  method = "square",
  order = "alphabet",
  col = COL2("BrBG", 10),
  tl.col = "black"
)

## -----------------------------------------------------------------------------
dist <- stats::dist(t(counts), method = "euclidian")
cluster <- stats::hclust(dist, method = "complete")

## -----------------------------------------------------------------------------
dendro <- as.dendrogram(cluster)

## -----------------------------------------------------------------------------
den_data <- dendro_data(dendro, type = "rectangle")

## -----------------------------------------------------------------------------
ggplot(segment(den_data)) +
  geom_segment(aes(x = x, y = y, xend = xend, yend = yend)) +
  geom_text(
    data = den_data$labels,
    aes(x = x, y = y, label = label),
    size = 3,
    hjust = "outward"
  ) +
  coord_cartesian(ylim = c((min(segment(den_data)$y) + 10),
                           (max(segment(den_data)$y)))) +
  coord_flip() +
  scale_y_reverse(expand = c(.5, 0)) +
  theme_dendro()

## -----------------------------------------------------------------------------
get_group_averages(data_filtered) %>%
  filter(biological_group == "Coculture" |
           biological_group == "ANG18") %>%
  select(compound, biological_group, average) %>%
  pivot_wider(names_from = biological_group, values_from = average) %>%
  mutate(fc = Coculture / ANG18) %>%
  head()

## -----------------------------------------------------------------------------
get_group_averages(data_filtered) %>%
  filter(biological_group == "Coculture" |
           biological_group == "ANG18") %>%
  select(compound, biological_group, average) %>%
  pivot_wider(names_from = biological_group, values_from = average) %>%
  mutate(nonzero_compound = if_else(Coculture == 0 & ANG18 == 0,
                                    FALSE,
                                    TRUE)) %>%
  filter(nonzero_compound == TRUE) %>%
  mutate(fc = Coculture / ANG18) %>%
  head()

## -----------------------------------------------------------------------------
get_group_averages(data_filtered) %>%
  filter(biological_group == "Coculture" |
           biological_group == "ANG18") %>%
  select(compound, biological_group, average) %>%
  head()

## -----------------------------------------------------------------------------
get_group_averages(data_filtered) %>%
  filter(biological_group == "Coculture" |
           biological_group == "ANG18") %>%
  select(compound, biological_group, average) %>%
  head()

## -----------------------------------------------------------------------------
get_group_averages(data_filtered) %>%
  filter(biological_group == "Coculture" |
           biological_group == "ANG18") %>%
  select(compound, biological_group, average) %>%
  mutate(average = average + 0.001) %>%
  pivot_wider(names_from = biological_group, values_from = average) %>%
  head()

## -----------------------------------------------------------------------------
get_group_averages(data_filtered) %>%
  filter(biological_group == "Coculture" |
           biological_group == "ANG18") %>%
  select(compound, biological_group, average) %>%
  mutate(average = average + 0.001) %>%
  pivot_wider(names_from = biological_group, values_from = average) %>%
  mutate(nonzero_compound = if_else(Coculture == 0.001 & ANG18 == 0.001,
                                    FALSE,
                                    TRUE)) %>%
  filter(nonzero_compound == TRUE) %>%
  head()

## -----------------------------------------------------------------------------
get_group_averages(data_filtered) %>%
  filter(biological_group == "Coculture" |
           biological_group == "ANG18") %>%
  select(compound, biological_group, average) %>%
  mutate(average = average + 0.001) %>%
  pivot_wider(names_from = biological_group, values_from = average) %>%
  mutate(nonzero_compound = if_else(Coculture == 0.001 & ANG18 == 0.001,
                                    FALSE,
                                    TRUE)) %>%
  filter(nonzero_compound == TRUE) %>%
  mutate(fc = Coculture / ANG18) %>%
  head()

## -----------------------------------------------------------------------------
foldchanges <- get_group_averages(data_filtered) %>%
  filter(biological_group == "Coculture" |
           biological_group == "ANG18") %>%
  select(compound, biological_group, average) %>%
  mutate(average = average + 0.001) %>%
  pivot_wider(names_from = biological_group, values_from = average) %>%
  mutate(nonzero_compound = if_else(Coculture == 0.001 & ANG18 == 0.001,
                                    FALSE,
                                    TRUE)) %>%
  filter(nonzero_compound == TRUE) %>%
  mutate(fc = Coculture / ANG18,
         logfc = log2(fc))

head(foldchanges)

## -----------------------------------------------------------------------------
fc_plotting <- foldchanges %>%
  left_join(select(ft, compound, mz, rt), by = "compound")

plot_ly(fc_plotting,
  x = ~logfc, y = ~rt, z = ~mz,
  marker = list(color = ~logfc, showscale = TRUE)
) %>%
  add_markers() %>%
  layout(
    scene = list(
      xaxis = list(title = "log2 Fold Change"),
      yaxis = list(title = "Retention Time (min)"),
      zaxis = list(title = "m/z")
    ),
    annotations = list(
      x = 1.13,
      y = 1.05,
      text = "log2 Fold Change",
      xref = "paper",
      yref = "paper",
      showarrow = FALSE
    )
  )

## -----------------------------------------------------------------------------
# Satterwaite
calc_samplesize_ws <- function(sd1, n1, sd2, n2) {
  s1 <- sd1 / (n1^0.5)
  s2 <- sd2 / (n2^0.5)

  n <- (s1^2 / n1 + s2^2 / n2)^2
  d1 <- s1^4 / ((n1^2) * (n1 - 1))
  d2 <- s2^4 / ((n2^2) * (n2 - 1))

  d1[which(!is.finite(d1))] <- 0
  d2[which(!is.finite(d2))] <- 0

  d <- d1 + d2

  return(n / d)
}

my_comp <- c("Coculture", "ANG18")

stats <- get_group_averages(data_filtered) %>%
  mutate(
    combRSD = sqrt(techRSD^2 + BiolRSD^2),
    combASD = combRSD * average,
    combASD = if_else(is.na(combASD), 0, combASD),
    BiolRSD = if_else(is.na(BiolRSD), 0, BiolRSD),
    techRSD = if_else(is.na(techRSD), 0, techRSD),
    neff = calc_samplesize_ws(techRSD, techn, BiolRSD, Bioln) + 1
  ) %>%
  filter(biological_group %in% my_comp)

head(stats)

## -----------------------------------------------------------------------------
denom <- stats %>%
  summarise(den = combASD^2 / (neff),
            .by = c("compound", "biological_group")) %>%
  mutate(den = if_else(!is.finite(den), 0, den)) %>%
  summarise(denom = sqrt(sum(den)), .by = c("compound"))

t_test <- stats %>%
  select(compound, biological_group, average) %>%
  pivot_wider(names_from = biological_group, values_from = average) %>%
  mutate(numerator = (Coculture - ANG18)) %>% # experimental - control
  left_join(denom, by = "compound") %>%
  mutate(t = abs(numerator / denom))

head(t_test)

## -----------------------------------------------------------------------------
df <- stats %>%
  select(compound, biological_group, neff) %>%
  mutate(neff = if_else(!is.finite(neff), 0, neff)) %>%
  pivot_wider(names_from = biological_group, values_from = neff) %>%
  mutate(deg = Coculture + ANG18 - 2) %>%
  select(compound, deg)

head(df)

## -----------------------------------------------------------------------------
t <- t_test %>%
  left_join(df, by = "compound") %>%
  mutate(
    p = (1 - pt(t, deg)) * 2,
    logp = log10(p),
    neg_logp = -logp
  ) %>%
  select(compound, t, deg, p, logp, neg_logp)

head(t)

## -----------------------------------------------------------------------------
num_ions <- t %>%
  filter(p <= 1) %>%
  count() %>%
  pull()

fc <- foldchanges %>%
  left_join(t, by = "compound") %>%
  arrange(p) %>%
  mutate(
    qval = seq_len(length(p)),
    qval = p * num_ions / qval
  ) %>%
  arrange(desc(p))

min_qval <- 1

for (i in seq_len(nrow(fc))) {
  if (!is.finite(fc$qval[i])) {
    next
  }

  if (fc$qval[i] < min_qval) {
    min_qval <- fc$qval[i]
  } else {
    fc$qval[i] <- min_qval
  }
}

fc2 <- fc %>%
  mutate(neg_logq = -log10(qval))

## -----------------------------------------------------------------------------
fc2 %>%
  ggplot() +
  aes(x = logfc, y = neg_logp) +
  geom_point() +
  labs(x = "log2 Fold Change",
       y = "-Log~10~ *P*",
       color = "Differential Abundance") +
  theme_bw()

## -----------------------------------------------------------------------------
fc2 %>%
  ggplot() +
  aes(x = logfc, y = neg_logp) +
  geom_point() +
  geom_hline(yintercept = -log10(0.05), linetype = "dashed") +
  labs(x = "log2 Fold Change",
       y = "-Log~10~ *P*",
       color = "Differential Abundance") +
  theme_bw()

## -----------------------------------------------------------------------------
fc2 %>%
  ggplot() +
  aes(x = logfc, y = neg_logp) +
  geom_point() +
  geom_hline(yintercept = -log10(0.05), linetype = "dashed") +
  labs(x = "log2 Fold Change",
       y = "-Log~10~ *P*",
       color = "Differential Abundance") +
  geom_vline(xintercept = log2(1.5), linetype = "dashed") +
  geom_vline(xintercept = -log2(1.5), linetype = "dashed") +
  theme_bw()

## -----------------------------------------------------------------------------
fc2 %>%
  mutate(
    sig = case_when(
      p > 0.05 ~ "not_sig",
      p <= 0.05 & logfc > log2(1.5) ~ "Increased",
      p <= 0.05 & logfc < -log2(1.5) ~ "Decreased",
      TRUE ~ "Inconclusive"
    ),
    sig = factor(sig,
      levels = c("Increased", "Decreased", "Inconclusive", "not_sig"),
      labels = c("Increased", "Decreased", "Inconclusive", "Not significant")
    )
  ) %>%
  select(compound, ANG18, Coculture, fc, logfc, p, sig) %>%
  head()

## -----------------------------------------------------------------------------
fc2 %>%
  mutate(
    sig = case_when(
      p > 0.05 ~ "not_sig",
      p <= 0.05 & logfc > log2(1.5) ~ "Increased",
      p <= 0.05 & logfc < -log2(1.5) ~ "Decreased",
      TRUE ~ "Inconclusive"
    ),
    sig = factor(sig,
      levels = c("Increased", "Decreased", "Inconclusive", "not_sig"),
      labels = c("Increased", "Decreased", "Inconclusive", "Not significant")
    )
  ) %>%
  ggplot() +
  aes(x = logfc, y = neg_logp, color = sig) +
  geom_point() +
  geom_hline(yintercept = -log10(0.05), linetype = "dashed") +
  geom_vline(xintercept = log2(1.5), linetype = "dashed") +
  geom_vline(xintercept = -log2(1.5), linetype = "dashed") +
  scale_color_manual(values = c("red", "blue", "grey", "black")) +
  labs(
    x = "log2 Fold Change",
    y = "-Log~10~ *P*",
    color = "Differential Abundance"
  ) +
  theme_bw()

## -----------------------------------------------------------------------------
fc2 %>%
  mutate(
    sig = case_when(
      p > 0.05 ~ "not_sig",
      p <= 0.05 & logfc > log2(1.5) ~ "Increased",
      p <= 0.05 & logfc < -log2(1.5) ~ "Decreased",
      TRUE ~ "Inconclusive"
    ),
    sig = factor(sig,
      levels = c("Increased", "Decreased", "Inconclusive", "not_sig"),
      labels = c("Increased", "Decreased", "Inconclusive", "Not significant")
    )
  ) %>%
  ggplot() +
  aes(x = logfc, y = neg_logp, color = sig) +
  geom_point() +
  geom_hline(yintercept = -log10(0.05), linetype = "dashed") +
  geom_vline(xintercept = log2(1.5), linetype = "dashed") +
  geom_vline(xintercept = -log2(1.5), linetype = "dashed") +
  scale_color_manual(values = c("red", "blue", "grey", "black")) +
  labs(
    x = "log2 Fold Change",
    y = "-Log~10~ *P*",
    color = "Differential Abundance"
  ) +
  theme_bw() +
  theme(
    axis.title = element_markdown(size = 20),
    axis.text = element_text(size = 15)
  )

## -----------------------------------------------------------------------------
volcano <- fc2 %>%
  mutate(
    sig = case_when(
      p > 0.05 ~ "not_sig",
      p <= 0.05 & logfc > log2(1.5) ~ "Increased",
      p <= 0.05 & logfc < -log2(1.5) ~ "Decreased",
      TRUE ~ "Inconclusive"
    ),
    sig = factor(sig,
      levels = c("Increased", "Decreased", "Inconclusive", "not_sig"),
      labels = c("Increased", "Decreased", "Inconclusive", "Not significant")
    )
  ) %>%
  ggplot() +
  aes(
    x = logfc, y = neg_logp, color = sig,
    text = paste0("compound: ", compound)
  ) +
  geom_point() +
  geom_hline(yintercept = -log10(0.05), linetype = "dashed") +
  geom_vline(xintercept = log2(1.5), linetype = "dashed") +
  geom_vline(xintercept = -log2(1.5), linetype = "dashed") +
  scale_color_manual(values = c("red", "blue", "grey", "black")) +
  labs(
    x = "log2 Fold Change",
    y = "-Log~10~ *P*",
    color = "Differential Abundance"
  ) +
  theme_bw() +
  theme(
    axis.title = element_markdown(size = 20),
    axis.text = element_text(size = 15)
  )

ggplotly(volcano)

## -----------------------------------------------------------------------------
fc2 %>%
  mutate(
    sig = case_when(
      qval > 0.05 ~ "not_sig",
      qval <= 0.05 & logfc > log2(1.5) ~ "Increased",
      qval <= 0.05 & logfc < -log2(1.5) ~ "Decreased",
      TRUE ~ "Inconclusive"
    ),
    sig = factor(sig,
      levels = c("Increased", "Decreased", "Inconclusive", "not_sig"),
      labels = c("Increased", "Decreased", "Inconclusive", "Not significant")
    )
  ) %>%
  ggplot() +
  aes(x = logfc, y = neg_logq, color = sig) +
  geom_point() +
  geom_hline(yintercept = -log10(0.05), linetype = "dashed") +
  geom_vline(xintercept = log2(1.5), linetype = "dashed") +
  geom_vline(xintercept = -log2(1.5), linetype = "dashed") +
  scale_color_manual(values = c("red", "blue", "grey", "black")) +
  labs(
    x = "log2 Fold Change",
    y = "-Log~10~ q-value",
    color = "Differential Abundance"
  ) +
  theme_bw() +
  theme(
    axis.title = element_markdown(size = 20),
    axis.text = element_text(size = 15)
  )

#' Clones the mpactr object
#' @description
#' This function allows us to clone the mpactr object to give
#' our users a more r-based flow.
#'
#' @param mpactr_object the object that we are cloning.
#' @noRd

clone <- function(mpactr_object) {
  new_object <- mpactr_object$clone(deep = TRUE)
  new_object$mpactr_data <- mpactr_object$mpactr_data$clone(deep = TRUE)
  new_object$mpactr_data$set_peak_table(data.table::copy(
    new_object$mpactr_data$get_peak_table()
  ))
  new_object$logger <- as.environment(as.list(mpactr_object$logger,
    all.names = TRUE
  ))
  return(new_object)
}

summary <- R6::R6Class("summary",
  public = list(
    initialize = function(filter, failed_ions, passed_ions) {
      stopifnot(any(class(filter) == "character"))

      if (is.null(failed_ions)) {
        private$filter <- filter
        private$failed_ions <- c()
        private$passed_ions <- passed_ions
      } else {
        stopifnot(any(class(failed_ions) == c("numeric", "character")))
        stopifnot(any(class(passed_ions) == c("numeric", "character")))

        private$filter <- filter
        private$failed_ions <- failed_ions
        private$passed_ions <- passed_ions
      }
    },
    summarize = function(x) {
      l <- length(private$failed_ions)
      f <- private$filter
      r <- length(private$passed_ions)

      cli::cli_alert_success(c("{l} ions failed the {f} filter, ",
                               "{r} ions remain."))
    },
    get_failed_ions = function() {
      return(private$failed_ions)
    },
    get_passed_ions = function() {
      return(private$passed_ions)
    },
    get_filter = function() {
      return(private$filter)
    }
  ),
  private = list(
    filter = NA,
    failed_ions = NA,
    passed_ions = NA
  )
)

####  filter 1: mismatched peaks    ###
test_that("test that check_mismatched_peaks works 
properly with filter_pactr-class data", {
            limit_cores()
            directory <- "exttestdata"
            peak_table_name <- "102623_peaktable_coculture_simple.csv"
            meta_data_name <- "102623_metadata_correct.csv"
            meta <- fread(test_path(directory, meta_data_name))
            pt_list <- progenesis_formatter(test_path(directory,
                                                      peak_table_name))

            mpactr_class <- mpactr$new(
              pt_list,
              meta
            )
            mpactr_class$setup()
            filter_class <- filter_pactr$new(mpactr_class)
            filter_class$check_mismatched_peaks(
              ringwin = 0.5,
              isowin = 0.01,
              trwin = 0.005,
              max_iso_shift = 3,
              merge_peaks = TRUE,
              merge_method = "sum"
            )

            expected_cut_ions <- fread(test_path(directory,
                                                 "cut_ions.csv"),
                                       col.names = c("V1", "V2"))
            expected_cut_ions <- as.character(expected_cut_ions$V1)
            logger_index_name <- "check_mismatched_peaks"
            expect_equal(filter_class$logger[[logger_index_name]][["cut_ions"]],
                         expected_cut_ions)
            expect_equal(filter_class$mpactr_data$get_peak_table()[
              Compound == "153",
              "102623_UM1850B_ANGDT_71_1_5007"
            ][[1]], 2158.4)
            expect_equal(nrow(filter_class$mpactr_data$get_peak_table()), 1233)
            expect_equal(address(mpactr_class),
                         address(filter_class$mpactr_data))
            expect_equal(mpactr_class$get_peak_table(),
                         filter_class$mpactr_data$get_peak_table())
            expect_false(is.null(filter_class$logger$
                                   list_of_summaries$mispicked))
            expect_equal(class(filter_class$logger$list_of_summaries$mispicked),
                         c("summary", "R6"))
          })

test_that("test that check_mismatched_peaks returns 
an error when no merge method is supplied", {
            limit_cores()
            directory <- "exttestdata"
            peak_table_name <- "102623_peaktable_coculture_simple.csv"
            meta_data_name <- "102623_metadata_correct.csv"
            meta <- fread(test_path(directory, meta_data_name))
            pt_list <- progenesis_formatter(test_path(directory,
                                                      peak_table_name))


            mpactr_class <- mpactr$new(
              pt_list,
              meta
            )
            mpactr_class$setup()
            filter_class <- filter_pactr$new(mpactr_class)
            expect_error(filter_class$check_mismatched_peaks(
              ringwin = 0.5,
              isowin = 0.01,
              trwin = 0.005,
              max_iso_shift = 3,
              merge_peaks = TRUE,
            ))
          })

####  filter 2: group filter    ###
test_that("blank filter works correctly", {
  limit_cores()
  directory <- "exttestdata"
  peak_table_name <- "102623_peaktable_coculture_simple.csv"
  meta_data_name <- "102623_metadata_correct.csv"
  meta <- fread(test_path(directory, meta_data_name))
  pt_list <- progenesis_formatter(test_path(directory,
                                            peak_table_name))


  mpactr_class <- mpactr$new(
    pt_list,
    meta
  )
  mpactr_class$setup()
  filter_class <- filter_pactr$new(mpactr_class)
  filter_class$check_mismatched_peaks(
    ringwin = 0.5,
    isowin = 0.01,
    trwin = 0.005,
    max_iso_shift = 3,
    merge_peaks = TRUE,
    merge_method = "sum"
  )
  filter_class$filter_blank()

  grp_avg <- "102623_peaktable_coculture_simple_groupaverages.csv"
  test_path(directory, grp_avg)
  error_prop <- fread(test_path(directory, grp_avg),
    skip = 1,
    col.names = c("Compound", "mz", "rt", "biologicalGroup", "average")
  )[, Compound := as.character(Compound)]
  setorder(error_prop, Compound)
  logger_index_name <- "group_filter-group_stats"
  expect_true(all(filter_class$logger[[logger_index_name]]$Biological_Group %in%
                    error_prop$biologicalGroup))

  log_grp_avg <- "group_filter-group_stats"
  expect_true(all(round(filter_class$logger[[log_grp_avg]]$average,
                        digits = 5) == round(error_prop$average, digits = 5)))
})

test_that("parse_ions_by_group flags the correct ions", {
  limit_cores()
  directory <- "exttestdata"
  peak_table_name <- "102623_peaktable_coculture_simple.csv"
  meta_data_name <- "102623_metadata_correct.csv"
  meta <- fread(test_path(directory, meta_data_name))
  pt_list <- progenesis_formatter(test_path(directory, peak_table_name))

  mpactr_class <- mpactr$new(
    pt_list,
    meta
  )
  mpactr_class$setup()
  filter_class <- filter_pactr$new(mpactr_class)
  filter_class$check_mismatched_peaks(
    ringwin = 0.5,
    isowin = 0.01,
    trwin = 0.005,
    max_iso_shift = 3,
    merge_peaks = TRUE,
    merge_method = "sum"
  )
  filter_class$filter_blank()
  filter_class$parse_ions_by_group(group_threshold = 0.01)

  ang_18 <- fread(test_path(directory, "output_ANG18_monoculture.csv"),
    col.names = c("V1"),
  )
  angdt <- fread(test_path(directory, "output_ANGDT_monoculture"),
    col.names = c("V1"),
  )
  blanks <- fread(test_path(directory, "output_Blanks"),
                  col.names = c("V1"))
  coculture <- fread(test_path(directory, "output_Coculture"),
                     col.names = c("V1"))
  jc1 <- fread(test_path(directory, "output_JC1_monoculture"),
               col.names = c("V1"))
  jc28 <- fread(test_path(directory, "output_JC28_monoculture"),
                col.names = c("V1"))
  group_filter_list <- filter_class$logger[["group_filter-failing_list"]]

  expect_false(all(sapply(group_filter_list, is.null)))
  expect_true(all(group_filter_list$`ANG18 monoculture`
                  %in% as.character(ang_18$V1)))
  expect_true(all(group_filter_list$`ANGDT monoculture`
                  %in% as.character(angdt$V1)))
  expect_true(all(group_filter_list$`Blanks`
                  %in% as.character(blanks$V1)))
  expect_true(all(group_filter_list$`Coculture`
                  %in% as.character(coculture$V1)))
  expect_true(all(group_filter_list$`JC28 monoculture`
                  %in% as.character(jc28$V1)))
  expect_true(all(group_filter_list$`JC28 monoculture`
                  %in% as.character(jc28$V1)))
})

test_that("apply_group_filter removes the correct ions", {
  limit_cores()
  directory <- "exttestdata"
  peak_table_name <- "102623_peaktable_coculture_simple.csv"
  meta_data_name <- "102623_metadata_correct.csv"
  meta <- fread(test_path(directory, meta_data_name))
  pt_list <- progenesis_formatter(test_path(directory, peak_table_name))

  mpactr_class <- mpactr$new(
    pt_list,
    meta
  )
  mpactr_class$setup()
  filter_class <- filter_pactr$new(mpactr_class)
  filter_class$check_mismatched_peaks(
    ringwin = 0.5,
    isowin = 0.01,
    trwin = 0.005,
    max_iso_shift = 3,
    merge_peaks = TRUE,
    merge_method = "sum"
  )
  filter_class$filter_blank()
  filter_class$parse_ions_by_group(group_threshold = 0.01)

  filter_class$apply_group_filter("Blanks", remove_ions = FALSE)
  expect_equal(nrow(filter_class$mpactr_data$get_peak_table()), 1233)

  filter_class$apply_group_filter("Blanks", remove_ions = TRUE)
  log_failing <- "group_filter-failing_list"
  expect_true(all(!(filter_class$logger[[log_failing]]$Blanks %in%
                      filter_class$mpactr_data$get_peak_table()$Compound)))

  expect_false(is.null(filter_class$logger$list_of_summaries[["group-Blanks"]]))
  expect_equal(class(filter_class$logger$list_of_summaries[["group-Blanks"]]),
               c("summary", "R6"))
  # Check for bad input
  expect_error(filter_class$apply_group_filter("Empty", remove_ions = TRUE))
})

####  filter 3: cv filter    ###
test_that("cv_filter filters out data properly", {
  limit_cores()
  directory <- "exttestdata"
  peak_table_name <- "102623_peaktable_coculture_simple.csv"
  meta_data_name <- "102623_metadata_correct.csv"
  meta <- fread(test_path(directory, meta_data_name))
  pt_list <- progenesis_formatter(test_path(directory, peak_table_name))
  mpactr_class <- mpactr$new(
    pt_list,
    meta
  )
  mpactr_class$setup()
  filter_class <- filter_pactr$new(mpactr_class)
  filter_class$check_mismatched_peaks(
    ringwin = 0.5,
    isowin = 0.01,
    trwin = 0.005,
    max_iso_shift = 3,
    merge_peaks = TRUE,
    merge_method = "sum"
  )
  filter_class$filter_blank()
  filter_class$parse_ions_by_group(group_threshold = 0.01)
  filter_class$apply_group_filter("Blanks", remove_ions = TRUE)
  filter_class_median <- filter_class$clone(deep = TRUE)
  filter_class$cv_filter(cv_threshold = 0.2)
  cv_filter_passed_ions <- filter_class$logger[["list_of_summaries"]]$
    replicability$get_passed_ions()
  expect_equal(length(filter_class$logger[["list_of_summaries"]]$
                        replicability$get_failed_ions()), 33)
  expect_error(filter_class$cv_filter())
  expect_false(is.null(filter_class$logger$list_of_summaries$replicability))
  expect_equal(class(filter_class$logger$list_of_summaries$replicability),
               c("summary", "R6"))
})
test_that("cv_filter errors without threshold", {
  limit_cores()
  directory <- "exttestdata"
  peak_table_name <- "102623_peaktable_coculture_simple.csv"
  meta_data_name <- "102623_metadata_correct.csv"
  meta <- fread(test_path(directory, meta_data_name))
  pt_list <- progenesis_formatter(test_path(directory, peak_table_name))
  mpactr_class <- mpactr$new(
    pt_list,
    meta
  )
  mpactr_class$setup()
  filter_class <- filter_pactr$new(mpactr_class)
  filter_class$check_mismatched_peaks(
    ringwin = 0.5,
    isowin = 0.01,
    trwin = 0.005,
    max_iso_shift = 3,
    merge_peaks = TRUE,
    merge_method = "sum"
  )
  filter_class$filter_blank()
  filter_class$parse_ions_by_group(group_threshold = 0.01)
  filter_class$apply_group_filter("Blanks", remove_ions = TRUE)

  expect_error(filter_class$cv_filter(cv_params = c("mean")))
})

test_that("cv_filter errors with incorrect parameter", {
  limit_cores()
  directory <- "exttestdata"
  peak_table_name <- "102623_peaktable_coculture_simple.csv"
  meta_data_name <- "102623_metadata_correct.csv"
  meta <- fread(test_path(directory, meta_data_name))
  pt_list <- progenesis_formatter(test_path(directory, peak_table_name))
  mpactr_class <- mpactr$new(
    pt_list,
    meta
  )
  mpactr_class$setup()
  filter_class <- filter_pactr$new(mpactr_class)
  filter_class$check_mismatched_peaks(
    ringwin = 0.5,
    isowin = 0.01,
    trwin = 0.005,
    max_iso_shift = 3,
    merge_peaks = TRUE,
    merge_method = "sum"
  )
  filter_class$filter_blank()
  filter_class$parse_ions_by_group(group_threshold = 0.01)
  filter_class$apply_group_filter("Blanks", remove_ions = TRUE)

  expect_error(filter_class$cv_filter(cv_threshold = 0.2, cv_params = ""))
})

test_that("cv_filter errors when there are no technical replicates", { # hmm
  limit_cores()
  directory <- "exttestdata"
  peak_table_name <- "102623_peaktable_coculture_simple.csv"
  meta_data_name <- "102623_metadata_correct.csv"
  meta <- fread(test_path(directory, meta_data_name))
  meta_sub <- meta[, head(.SD, 1), by = sample_code]

  pt_list <- progenesis_formatter(test_path(directory, peak_table_name))
  sub <- c("Compound", "mz", "rt", meta_sub$injection)
  pt_list$peak_table <- pt_list$peak_table[, .SD, .SDcols = sub]
  pt_list$raw_table <- pt_list$raw_table[, .SD, .SDcols = sub]


  mpactr_class <- mpactr$new(
    pt_list,
    meta_sub
  )
  mpactr_class$setup()

  filter_class <- filter_pactr$new(mpactr_class)
  filter_class$check_mismatched_peaks(
    ringwin = 0.5,
    isowin = 0.01,
    trwin = 0.005,
    max_iso_shift = 3,
    merge_peaks = TRUE,
    merge_method = "sum"
  )
  filter_class$filter_blank()
  filter_class$parse_ions_by_group(group_threshold = 0.01)
  filter_class$apply_group_filter("Blanks", remove_ions = TRUE)


  expect_error(filter_class$cv_filter(cv_threshold = 0.2, cv_params = "mean"))
})




####  filter 4: insource ions    ###
test_that("filter_inscource_ions filters out data properly", {
  limit_cores()
  directory <- "exttestdata"
  peak_table_name <- "102623_peaktable_coculture_simple.csv"
  meta_data_name <- "102623_metadata_correct.csv"
  meta <- fread(test_path(directory, meta_data_name))
  pt_list <- progenesis_formatter(test_path(directory, peak_table_name))
  mpactr_class <- mpactr$new(
    pt_list,
    meta
  )
  mpactr_class$setup()
  filter_class <- filter_pactr$new(mpactr_class)
  filter_class$check_mismatched_peaks(
    ringwin = 0.5,
    isowin = 0.01,
    trwin = 0.005,
    max_iso_shift = 3,
    merge_peaks = TRUE,
    merge_method = "sum"
  )
  filter_class$filter_blank()
  filter_class$parse_ions_by_group(group_threshold = 0.01)
  filter_class$apply_group_filter("Blanks", remove_ions = TRUE)
  filter_class$filter_insource_ions(cluster_threshold = 0.95)

  insource_ion_expected_list <- c(
    38, 204, 214, 993, 270, 1003, 271, 294, 331, 349, 382,
    447, 498, 1233, 644, 1307, 677, 675, 689,
    690, 688, 758, 985, 982, 981, 1297, 1311
  )
  expect_true(length(filter_class$logger[["list_of_summaries"]]$
                       insource$get_failed_ions()) == 27)
  expect_true(all(!(insource_ion_expected_list %in%
                      filter_class$mpactr_data$get_peak_table()$Compound)))

  expect_false(is.null(filter_class$logger$list_of_summaries$insource))
  expect_true(is.null(filter_class$logger$list_of_summaries$replicability))
  expect_equal(class(filter_class$logger$list_of_summaries$insource),
               c("summary", "R6"))
})

test_that("summary class constructor works correctly", {
  limit_cores()
  summary_one <- summary$new("filter1", c(1, 2, 3), c(4, 5, 6))

  expect_equal(summary_one$get_filter(), "filter1")
  expect_equal(summary_one$get_failed_ions(), c(1, 2, 3))
  expect_equal(summary_one$get_passed_ions(), c(4, 5, 6))

  # Test for empty constructors too
  summary_two <- summary$new("filter1", NULL, NULL)
  expect_equal(summary_two$get_filter(), "filter1")
  expect_equal(summary_two$get_failed_ions(), c())
  expect_equal(summary_two$get_passed_ions(), NULL)
})

filter_pactr <- R6Class("filter_pactr", public = list(
  mpactr_data = NA,
  logger = NA,
  initialize = function(mpactr) {
    self$mpactr_data <- mpactr
    self$logger <- new.env(hash = TRUE)
    self$logger[["list_of_summaries"]] <- list()
  },
  print = function() {
    print(self$mpactr_data$get_peak_table())
  },
  is_filter_run = function(filter, group = NULL) {
    if (!is.null(group)) {
      filter <- paste(filter, group, sep = "-")
    }

    if ((filter %in% names(self$logger$list_of_summaries))) {
      return(TRUE)
    } else {
      return(FALSE)
    }
  },
  get_log = function(filter, group = NULL) {
    if (!(filter %in% c("mispicked", "group", "replicability", "insource"))) {
      cli::cli_abort("{.var filter} must be one of mpactr's supported filters:
                     mispicked, group, replicability, insource.")
    }

    if (!is.null(group)) {
      filter <- paste(filter, group, sep = "-")
    }

    if (!(filter %in% names(self$logger$list_of_summaries))) {
      cli::cli_abort("{.var filter} {filter} has not yet been applied to
                     the data. Run the corresponding filter function prior
                     to extracting the summary.")
    }

    return(list(
      "failed_ions" = self$logger$list_of_summaries[[filter]]$get_failed_ions(),
      "passed_ions" = self$logger$list_of_summaries[[filter]]$get_passed_ions()
    ))
  },
  get_mispicked_ions = function() {
    if (!exists("check_mismatched_peaks", self$logger)) {
      cli::cli_abort("The mispicked filter has not yet been applied to
                     the data - run filter_mispicked_ions() first.")
    }

    merge_groups <- self$logger$check_mismatched_peaks$merge_groups

    similar_ions <- data.table(
      "main_ion" = names(merge_groups),
      "similar_ions" = merge_groups
    )

    return(similar_ions)
  },
  get_group_averages = function() {
    # return averages and variations for all ions in filtered table
    b <- data.table::melt(self$mpactr_data$get_peak_table(),
      id.vars = c("Compound", "mz", "rt", "kmd"), variable.name =
        "sample", value.name = "intensity", variable.factor = FALSE
    )[
      data.table(self$mpactr_data$get_meta_data()),
      on = .(sample = injection)
    ][
      , .(
        average = mean(intensity),
        BiolRSD = rsd(intensity),
        Bioln = length(intensity)
      ),
      by = .(Compound, biological_group)
    ]

    t <- data.table::melt(self$mpactr_data$get_peak_table(),
      id.vars = c("Compound", "mz", "rt", "kmd"),
      variable.name = "sample",
      value.name = "intensity",
      variable.factor = FALSE
    )[
      data.table(self$mpactr_data$get_meta_data()),
      on = .(sample = injection)
    ][
      , .(sd = rsd(intensity), n = length(intensity)),
      by = .(Compound, biological_group, sample_code)
    ][
      , .(techRSD = mean(sd), techn = mean(n)),
      by = .(Compound, biological_group)
    ]

    group_stats <- b[t, on = .(Compound, biological_group)]
    setorder(group_stats, Compound, biological_group)

    return(group_stats)
  },
  get_cv = function() {
    if (!exists("cv_values", self$logger)) {
      cli::cli_abort("The cv filter has not yet been applied
                      to the data - run filter_cv() first.")
    }

    return(self$logger$cv_values)
  }
))

test_that("test that filter_pactr-class constructs properly", {
  limit_cores()
  directory <- "exttestdata"
  peak_table_name <- "102623_peaktable_coculture_simple.csv"
  meta_data_name <- "102623_metadata_correct.csv"
  meta <- fread(test_path(directory, meta_data_name))
  pt_list <- progenesis_formatter(test_path(directory, peak_table_name))

  mpactr_class <- mpactr$new(
    pt_list,
    meta
  )
  mpactr_class$setup()
  filter_class <- filter_pactr$new(mpactr_class)
  expect_true(all(class(filter_class) == c("filter_pactr", "R6")))
  expect_equal(address(mpactr_class), address(filter_class$mpactr_data))
  expect_true(exists("list_of_summaries", filter_class$logger))
  expect_true(all(sapply(filter_class$logger[["list_of_summaries"]],
                         is.null) == TRUE))
})

test_that("get_log returns an error 
when an incorrect filter argument is provided", {

            limit_cores()
            directory <- "exttestdata"
            peak_table_name <- "102623_peaktable_coculture_simple.csv"
            meta_data_name <- "102623_metadata_correct.csv"

            meta <- fread(test_path(directory, meta_data_name))
            pt_list <- progenesis_formatter(test_path(directory,
                                                      peak_table_name))

            mpactr_class <- mpactr$new(
              pt_list,
              meta
            )
            mpactr_class$setup()
            filter_class <- filter_pactr$new(mpactr_class)

            expect_error(filter_class$get_log(filter = "cv"),
                         "`filter` must be one of mpactr's")
          })

test_that("get_log returns an error when the 
filter argument provided has not yet been run (e.g., not in the log)", {
            limit_cores()
            directory <- "exttestdata"
            peak_table_name <- "102623_peaktable_coculture_simple.csv"
            meta_data_name <- "102623_metadata_correct.csv"

            meta <- fread(test_path(directory, meta_data_name))
            pt_list <- progenesis_formatter(test_path(directory,
                                                      peak_table_name))

            mpactr_class <- mpactr$new(
              pt_list,
              meta
            )
            mpactr_class$setup()
            filter_class <- filter_pactr$new(mpactr_class)
            er <- "`filter` mispicked has not yet been applied to the data"
            expect_error(filter_class$get_log(filter = "mispicked"), er)
          })

test_that("get_log returns the correct filter summary list", {
  limit_cores()
  directory <- "exttestdata"
  peak_table_name <- "102623_peaktable_coculture_simple.csv"
  meta_data_name <- "102623_metadata_correct.csv"

  meta <- fread(test_path(directory, meta_data_name))
  pt_list <- progenesis_formatter(test_path(directory, peak_table_name))

  mpactr_class <- mpactr$new(
    pt_list,
    meta
  )
  mpactr_class$setup()
  filter_class <- filter_pactr$new(mpactr_class)
  filter_class$check_mismatched_peaks(
    ringwin = 0.5,
    isowin = 0.01,
    trwin = 0.005,
    max_iso_shift = 3,
    merge_peaks = TRUE,
    merge_method = "sum"
  )

  mispicked_summary <- filter_class$get_log(filter = "mispicked")

  expect_type(mispicked_summary, "list")
  expect_equal(length(mispicked_summary), 2)
  expect_equal(length(mispicked_summary$passed_ions), 1233)
})

test_that("get_log returns the correct filter
 summary list when group is supplied", {
            limit_cores()
            directory <- "exttestdata"
            peak_table_name <- "102623_peaktable_coculture_simple.csv"
            meta_data_name <- "102623_metadata_correct.csv"
            meta <- fread(test_path(directory, meta_data_name))
            pt_list <- progenesis_formatter(test_path(directory,
                                                      peak_table_name))

            mpactr_class <- mpactr$new(
              pt_list,
              meta
            )
            mpactr_class$setup()
            filter_class <- filter_pactr$new(mpactr_class)

            filter_class$filter_blank()
            filter_class$parse_ions_by_group(group_threshold = 0.01)
            filter_class$apply_group_filter("Blanks", remove_ions = TRUE)

            group_summary <- filter_class$get_log(filter = "group",
                                                  group = "Blanks")

            expect_type(group_summary, "list")
          })


test_that("get_mispicked_ions returns error if 
check_mismatched_peaks has not been called", {
            limit_cores()
            directory <- "exttestdata"
            peak_table_name <- "102623_peaktable_coculture_simple.csv"
            meta_data_name <- "102623_metadata_correct.csv"
            meta <- fread(test_path(directory, meta_data_name))
            pt_list <- progenesis_formatter(test_path(directory,
                                                      peak_table_name))
            mpactr_class <- mpactr$new(
              pt_list,
              meta
            )
            mpactr_class$setup()
            filter_class <- filter_pactr$new(mpactr_class)

            expect_error(filter_class$get_mispicked_ions(),
                         "The mispicked filter has not yet been")
          })

test_that("get_mispicked_ions correctly returns
 the check_mismatched_peaks list", {
            limit_cores()
            directory <- "exttestdata"
            peak_table_name <- "102623_peaktable_coculture_simple.csv"
            meta_data_name <- "102623_metadata_correct.csv"
            meta <- fread(test_path(directory, meta_data_name))
            pt_list <- progenesis_formatter(test_path(directory,
                                                      peak_table_name))

            mpactr_class <- mpactr$new(
              pt_list,
              meta
            )
            mpactr_class$setup()
            filter_class <- filter_pactr$new(mpactr_class)
            filter_class$check_mismatched_peaks(
              ringwin = 0.5,
              isowin = 0.01,
              trwin = 0.005,
              max_iso_shift = 3,
              merge_peaks = TRUE,
              merge_method = "sum"
            )

            mispicked_groups <- filter_class$get_mispicked_ions()

            expect_equal(class(mispicked_groups), c("data.table", "data.frame"))
            expect_equal(length(mispicked_groups), 2)
            expect_equal(names(mispicked_groups), c("main_ion", "similar_ions"))
          })


test_that("get_group_averages calculates a group table", {
  limit_cores()
  directory <- "exttestdata"
  peak_table_name <- "102623_peaktable_coculture_simple.csv"
  meta_data_name <- "102623_metadata_correct.csv"
  meta <- fread(test_path(directory, meta_data_name))
  pt_list <- progenesis_formatter(test_path(directory,
                                            peak_table_name))


  mpactr_class <- mpactr$new(
    pt_list,
    meta
  )
  mpactr_class$setup()
  filter_class <- filter_pactr$new(mpactr_class)

  filter_class$check_mismatched_peaks(
    ringwin = 0.5,
    isowin = 0.01,
    trwin = 0.005,
    max_iso_shift = 3,
    merge_peaks = TRUE,
    merge_method = "sum"
  )

  avgs <- filter_class$get_group_averages()
  expect_equal(class(avgs), c("data.table", "data.frame"))
  expect_equal(nrow(avgs), (1233 * 6))

  meta <- fread(test_path(directory, meta_data_name))
  pt_list <- progenesis_formatter(test_path(directory,
                                            peak_table_name))



  mpactr_class <- mpactr$new(
    pt_list,
    meta
  )
  mpactr_class$setup()
  filter_class <- filter_pactr$new(mpactr_class)

  filter_class$check_mismatched_peaks(
    ringwin = 0.5,
    isowin = 0.01,
    trwin = 0.005,
    max_iso_shift = 3,
    merge_peaks = TRUE,
    merge_method = "sum"
  )
  filter_class$filter_blank()
  filter_class$parse_ions_by_group(group_threshold = 0.01)
  filter_class$apply_group_filter("Blanks", remove_ions = TRUE)

  avgs <- filter_class$get_group_averages()
  expect_equal(class(avgs), c("data.table", "data.frame"))
  expect_equal(nrow(avgs), (484 * 6))
})

test_that("get_cv returns the cv filter has been applied", {
  limit_cores()
  directory <- "exttestdata"
  peak_table_name <- "102623_peaktable_coculture_simple.csv"
  meta_data_name <- "102623_metadata_correct.csv"
  meta <- fread(test_path(directory, meta_data_name))
  pt_list <- progenesis_formatter(test_path(directory,
                                            peak_table_name))


  mpactr_class <- mpactr$new(
    pt_list,
    meta
  )
  mpactr_class$setup()
  filter_class <- filter_pactr$new(mpactr_class)
  filter_class$check_mismatched_peaks(
    ringwin = 0.5,
    isowin = 0.01,
    trwin = 0.005,
    max_iso_shift = 3,
    merge_peaks = TRUE,
    merge_method = "sum"
  )
  filter_class$filter_blank()
  filter_class$parse_ions_by_group(group_threshold = 0.01)
  filter_class$apply_group_filter("Blanks", remove_ions = TRUE)

  expect_error(filter_class$get_cv(), "The cv filter has not yet")

  filter_class$cv_filter(cv_threshold = 0.2)

  cv <- filter_class$get_cv()
  expect_equal(class(cv), c("data.table", "data.frame"))
})

test_that("Test that mpactr can be printed from the filter-pactR class", {
  limit_cores()
  directory <- "exttestdata"
  peak_table_name <- "102623_peaktable_coculture_simple.csv"
  meta_data_name <- "102623_metadata_correct.csv"
  meta <- fread(test_path(directory, meta_data_name))
  pt_list <- progenesis_formatter(test_path(directory,
                                            peak_table_name))

  mpactr_class <- mpactr$new(
    pt_list,
    meta
  )
  mpactr_class$setup()
  filter_class <- filter_pactr$new(mpactr_class)
  expect_output(filter_class$print())
})

test_that("is_filter_run correctly assesses if a filter has been run", {
  limit_cores()
  directory <- "exttestdata"
  peak_table_name <- "102623_peaktable_coculture_simple.csv"
  meta_data_name <- "102623_metadata_correct.csv"
  meta <- fread(test_path(directory, meta_data_name))
  pt_list <- progenesis_formatter(test_path(directory,
                                            peak_table_name))


  mpactr_class <- mpactr$new(
    pt_list,
    meta
  )
  mpactr_class$setup()
  filter_class <- filter_pactr$new(mpactr_class)
  filter_class$check_mismatched_peaks(
    ringwin = 0.5,
    isowin = 0.01,
    trwin = 0.005,
    max_iso_shift = 3,
    merge_peaks = TRUE,
    merge_method = "sum"
  )
  filter_class$filter_blank()
  filter_class$parse_ions_by_group(group_threshold = 0.01)
  filter_class$apply_group_filter("Blanks", remove_ions = TRUE)

  expect_true(filter_class$is_filter_run(filter = "group",
                                         group = "Blanks"))

  expect_false(filter_class$is_filter_run(filter = "insource"))
})

test_that("get_log returns an error when 
an incorrect filter argument is provided", {
            limit_cores()
            directory <- "exttestdata"
            peak_table_name <- "102623_peaktable_coculture_simple.csv"
            meta_data_name <- "102623_metadata_correct.csv"
            meta <- fread(test_path(directory, meta_data_name))
            pt_list <- progenesis_formatter(test_path(directory,
                                                      peak_table_name))
            mpactr_class <- mpactr$new(
              pt_list,
              meta
            )
            mpactr_class$setup()
            filter_class <- filter_pactr$new(mpactr_class)
            expect_error(filter_class$get_log(filter = "cv"),
                         "`filter` must be one of mpactr's")
          })

test_that("get_log returns an error when the filter
 argument provided has not yet been run (e.g., not in the log)", {
            limit_cores()
            directory <- "exttestdata"
            peak_table_name <- "102623_peaktable_coculture_simple.csv"
            meta_data_name <- "102623_metadata_correct.csv"
            meta <- fread(test_path(directory, meta_data_name))
            pt_list <- progenesis_formatter(test_path(directory,
                                                      peak_table_name))
            mpactr_class <- mpactr$new(
              pt_list,
              meta
            )
            mpactr_class$setup()
            filter_class <- filter_pactr$new(mpactr_class)
            err_msg <- "`filter` mispicked has not yet been applied to the data"
            expect_error(filter_class$get_log(filter = "mispicked"), err_msg)
          })

test_that("get_log returns the correct filter summary list", {
  limit_cores()
  directory <- "exttestdata"
  peak_table_name <- "102623_peaktable_coculture_simple.csv"
  meta_data_name <- "102623_metadata_correct.csv"
  meta <- fread(test_path(directory, meta_data_name))
  pt_list <- progenesis_formatter(test_path(directory,
                                            peak_table_name))
  mpactr_class <- mpactr$new(
    pt_list,
    meta
  )
  mpactr_class$setup()
  filter_class <- filter_pactr$new(mpactr_class)
  filter_class$check_mismatched_peaks(ringwin = 0.5,
                                      isowin = 0.01,
                                      trwin = 0.005,
                                      max_iso_shift = 3,
                                      merge_peaks = TRUE,
                                      merge_method = "sum")
  mispicked_summary <- filter_class$get_log(filter = "mispicked")
  expect_type(mispicked_summary, "list")
  expect_equal(length(mispicked_summary), 2)
  expect_equal(length(mispicked_summary$passed_ions), 1233)
})

test_that("get_mispicked_ions returns error if 
check_mismatched_peaks has not been called", {
            limit_cores()
            directory <- "exttestdata"
            peak_table_name <- "102623_peaktable_coculture_simple.csv"
            meta_data_name <- "102623_metadata_correct.csv"
            meta <- fread(test_path(directory, meta_data_name))
            pt_list <- progenesis_formatter(test_path(directory,
                                                      peak_table_name))
            mpactr_class <- mpactr$new(
              pt_list,
              meta
            )
            mpactr_class$setup()
            filter_class <- filter_pactr$new(mpactr_class)

            expect_error(filter_class$get_mispicked_ions(),
                         "The mispicked filter has not yet been")
          })

test_that("get_mispicked_ions correctly 
returns the check_mismatched_peaks list", {
            limit_cores()
            directory <- "exttestdata"
            peak_table_name <- "102623_peaktable_coculture_simple.csv"
            meta_data_name <- "102623_metadata_correct.csv"
            meta <- fread(test_path(directory, meta_data_name))
            pt_list <- progenesis_formatter(test_path(directory,
                                                      peak_table_name))
            mpactr_class <- mpactr$new(
              pt_list,
              meta
            )
            mpactr_class$setup()
            filter_class <- filter_pactr$new(mpactr_class)
            filter_class$check_mismatched_peaks(ringwin = 0.5,
                                                isowin = 0.01,
                                                trwin = 0.005,
                                                max_iso_shift = 3,
                                                merge_peaks = TRUE,
                                                merge_method = "sum")

            mispicked_groups <- filter_class$get_mispicked_ions()

            expect_equal(class(mispicked_groups), c("data.table", "data.frame"))
            expect_equal(length(mispicked_groups), 2)
            expect_equal(names(mispicked_groups), c("main_ion", "similar_ions"))
          })


test_that("get_group_averages calculates a group table", {
  limit_cores()
  directory <- "exttestdata"
  peak_table_name <- "102623_peaktable_coculture_simple.csv"
  meta_data_name <- "102623_metadata_correct.csv"
  meta <- fread(test_path(directory, meta_data_name))
  pt_list <- progenesis_formatter(test_path(directory,
                                            peak_table_name))
  mpactr_class <- mpactr$new(
    pt_list,
    meta
  )
  mpactr_class$setup()
  filter_class <- filter_pactr$new(mpactr_class)
  filter_class$check_mismatched_peaks(ringwin = 0.5,
                                      isowin = 0.01,
                                      trwin = 0.005,
                                      max_iso_shift = 3,
                                      merge_peaks = TRUE,
                                      merge_method = "sum")
  avgs <- filter_class$get_group_averages()
  expect_equal(class(avgs), c("data.table", "data.frame"))
  expect_equal(nrow(avgs), (1233 * 6))

  mpactr_class <- mpactr$new(
    pt_list,
    meta
  )
  mpactr_class$setup()
  filter_class <- filter_pactr$new(mpactr_class)

  filter_class$check_mismatched_peaks(ringwin = 0.5,
                                      isowin = 0.01,
                                      trwin = 0.005,
                                      max_iso_shift = 3,
                                      merge_peaks = TRUE,
                                      merge_method = "sum")
  filter_class$filter_blank()
  filter_class$parse_ions_by_group(group_threshold = 0.01)
  filter_class$apply_group_filter("Blanks", remove_ions = TRUE)

  avgs <- filter_class$get_group_averages()
  expect_equal(class(avgs), c("data.table", "data.frame"))
  expect_equal(nrow(avgs), (484 * 6))
})

test_that("get_cv returns the cv filter has been applied", {
  limit_cores()
  directory <- "exttestdata"
  peak_table_name <- "102623_peaktable_coculture_simple.csv"
  meta_data_name <- "102623_metadata_correct.csv"
  meta <- fread(test_path(directory, meta_data_name))
  pt_list <- progenesis_formatter(test_path(directory,
                                            peak_table_name))
  mpactr_class <- mpactr$new(
    pt_list,
    meta
  )
  mpactr_class$setup()
  filter_class <- filter_pactr$new(mpactr_class)
  filter_class$check_mismatched_peaks(ringwin = 0.5,
                                      isowin = 0.01,
                                      trwin = 0.005,
                                      max_iso_shift = 3,
                                      merge_peaks = TRUE,
                                      merge_method = "sum")
  filter_class$filter_blank()
  filter_class$parse_ions_by_group(group_threshold = 0.01)
  filter_class$apply_group_filter("Blanks", remove_ions = TRUE)

  expect_error(filter_class$get_cv(), "The cv filter has not yet")

  filter_class$cv_filter(cv_threshold = 0.2)

  cv <- filter_class$get_cv()
  expect_equal(class(cv), c("data.table", "data.frame"))
})

#' @keywords internal
"_PACKAGE"

## usethis namespace: start
#' @importFrom cli cli_abort
#' @importFrom cli cli_alert_info
#' @importFrom cli cli_alert_success
#' @importFrom data.table address
#' @importFrom data.table as.data.table
#' @importFrom data.table copy
#' @importFrom data.table data.table
#' @importFrom data.table dcast
#' @importFrom data.table fread
#' @importFrom data.table melt
#' @importFrom data.table setDTthreads
#' @importFrom data.table setnames
#' @importFrom data.table setorder
#' @importFrom ggplot2 aes
#' @importFrom ggplot2 ggplot
#' @importFrom ggplot2 is_ggplot
#' @importFrom ggplot2 theme
#' @importFrom R6 R6Class
#' @importFrom Rcpp sourceCpp
#' @importFrom stats cor
#' @importFrom stats cutree
#' @importFrom stats dist
#' @importFrom stats hclust
#' @importFrom stats sd
#' @importFrom treemapify geom_treemap
#' @importFrom treemapify geom_treemap_text
#' @importFrom viridis scale_fill_viridis
#' @useDynLib mpactr, .registration = TRUE
## usethis namespace: end
NULL

mpactr <- R6Class("mpactr",
  public = list(
    # Properties

    # Constructor
    initialize = function(peak_table, meta_data) {
      stopifnot(any(class(peak_table$raw_table) == "data.table"))
      stopifnot(any(class(peak_table$peak_table) == "data.table"))
      stopifnot(any(class(meta_data) == "data.table"))
      private$raw_peak_table <- data.table(peak_table$raw_table)
      private$peak_table <- data.table(peak_table$peak_table)
      private$meta_data <- data.table(meta_data)
    },
    isMultipleTechReps = function() {
      any(private$meta_data[, .N, by = sample_code][["N"]] > 1)
    }
  ),
  private = list(
    peak_table = NA,
    meta_data = NA,
    raw_peak_table = NA
  )
)

test_that("mpactr class initialize works correctly", {
  limit_cores()
  directory <- "exttestdata"
  peak_table_name <- "102623_peaktable_coculture_simple.csv"
  meta_data_name <- "102623_metadata_correct.csv"

  meta <- fread(test_path(directory, meta_data_name))
  pt_list <- progenesis_formatter(test_path(directory, peak_table_name))

  mpactr_class <- mpactr$new(
    pt_list,
    meta
  )

  expect_true(all(class(mpactr_class) == c("mpactr", "R6")))
  expect_equal(length(mpactr_class$get_meta_data()), 3)
  expect_equal(length(mpactr_class$get_peak_table()), 21)
  expect_error(mpactr$new(peak_table_path = 2, meta_data_path = 5), NULL)
})

test_that("mpactr isMultipleTechReps correctly
  determines if there are technical replicates", {
            limit_cores()
            directory <- "exttestdata"
            peak_table_name <- "102623_peaktable_coculture_simple.csv"
            meta_data_name <- "102623_metadata_correct.csv"
            meta <- fread(test_path(directory, meta_data_name))
            pt_list <- progenesis_formatter(test_path(directory,
                                                      peak_table_name))

            mpactr_class <- mpactr$new(
              pt_list,
              meta
            )
            expect_true(mpactr_class$isMultipleTechReps())
          })

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

## ----setup, message=FALSE-----------------------------------------------------
library(mpactr)
library(viridis)
library(plotly)
library(data.table)
library(tidyverse)
library(Hmisc)
library(corrplot)
library(ggdendro)
library(ggtext)

## ----include = FALSE----------------------------------------------------------
limit_cores()

## -----------------------------------------------------------------------------
data <- import_data(example_path("cultures_peak_table.csv"),
  example_path("cultures_metadata.csv"),
  format = "Progenesis"
)

## -----------------------------------------------------------------------------
data_filtered <- data |>
  filter_mispicked_ions(merge_peaks = TRUE, merge_method = "sum") |>
  filter_group(group_to_remove = "Solvent_Blank") |>
  filter_group(group_to_remove = "Media") |>
  filter_cv(cv_threshold = 0.2)

## -----------------------------------------------------------------------------
plot_qc_tree(data_filtered)

## -----------------------------------------------------------------------------
get_raw_data(data_filtered) %>%
  select(Compound, mz, rt) %>%
  head()

## -----------------------------------------------------------------------------
qc_summary(data_filtered) %>%
  head()

## -----------------------------------------------------------------------------
get_raw_data(data_filtered) %>%
  mutate(Compound = as.character(Compound)) %>%
  select(Compound, mz, rt) %>%
  left_join(qc_summary(data_filtered),
    by = join_by("Compound" == "compounds")
  ) %>%
  head()

## -----------------------------------------------------------------------------
get_raw_data(data_filtered) %>%
  mutate(Compound = as.character(Compound)) %>%
  select(Compound, mz, rt) %>%
  left_join(qc_summary(data_filtered),
    by = join_by("Compound" == "compounds")
  ) %>%
  ggplot() +
  aes(x = rt, y = mz, color = status) +
  geom_point() +
  viridis::scale_color_viridis(discrete = TRUE) +
  labs(
    x = "Retention time (min)",
    y = "m/z",
    color = "Ion Status"
  ) +
  theme_bw() +
  theme(legend.position = "top")

## -----------------------------------------------------------------------------
feature_plot <- get_raw_data(data_filtered) %>%
  mutate(Compound = as.character(Compound)) %>%
  select(Compound, mz, rt) %>%
  left_join(qc_summary(data_filtered),
    by = join_by("Compound" == "compounds")
  ) %>%
  ggplot() +
  aes(
    x = rt, y = mz, color = status,
    text = paste0("Compound: ", Compound)
  ) +
  geom_point() +
  viridis::scale_color_viridis(discrete = TRUE) +
  labs(
    x = "Retention time (min)",
    y = "m/z",
    color = "Ion Status"
  ) +
  theme_bw() +
  theme(legend.position = "top")

ggplotly(feature_plot)

## -----------------------------------------------------------------------------
ft <- get_peak_table(data_filtered)

ft[1:5, 1:7]

## -----------------------------------------------------------------------------
counts <- ft %>%
  select(Compound, all_of(get_metadata(data_filtered)$injection)) %>%
  column_to_rownames(var = "Compound") %>%
  select(where(~ sum(.x) != 0))

counts[1:5, 1:2]

## -----------------------------------------------------------------------------
counts_cor <- rcorr(as.matrix(counts), type = "spearman")

## -----------------------------------------------------------------------------
counts_cor$r[, 1]

## ----warning=FALSE------------------------------------------------------------
corrplot(counts_cor$r,
  type = "lower",
  method = "square",
  order = "hclust",
  col = COL2("BrBG", 10),
  tl.col = "black",
  tl.cex = .5
)

## -----------------------------------------------------------------------------
meta <- get_metadata(data_filtered)

counts %>%
  rownames_to_column(var = "Compound") %>%
  pivot_longer(
    cols = starts_with("102623"),
    names_to = "injection",
    values_to = "intensity"
  ) %>%
  head()

## -----------------------------------------------------------------------------
counts %>%
  rownames_to_column(var = "Compound") %>%
  pivot_longer(
    cols = starts_with("102623"),
    names_to = "injection",
    values_to = "intensity"
  ) %>%
  left_join(meta, by = "injection") %>%
  head()

## -----------------------------------------------------------------------------
counts %>%
  rownames_to_column(var = "Compound") %>%
  pivot_longer(
    cols = starts_with("102623"),
    names_to = "injection",
    values_to = "intensity"
  ) %>%
  left_join(meta, by = "injection") %>%
  summarise(
    mean_intensity = mean(intensity),
    .by = c(Compound, sample_code)
  ) %>%
  head()

## ----warning=FALSE------------------------------------------------------------
sample_counts <- counts %>%
  rownames_to_column(var = "Compound") %>%
  pivot_longer(
    cols = starts_with("102623"),
    names_to = "injection",
    values_to = "intensity"
  ) %>%
  left_join(meta, by = "injection") %>%
  summarise(
    mean_intensity = mean(intensity),
    .by = c(Compound, sample_code)
  ) %>%
  pivot_wider(
    names_from = sample_code,
    values_from = mean_intensity
  ) %>%
  column_to_rownames(var = "Compound")

sample_counts[1:5, 1:5]

## -----------------------------------------------------------------------------
sample_counts_cor <- rcorr(as.matrix(sample_counts), type = "spearman")

## -----------------------------------------------------------------------------
corrplot(sample_counts_cor$r,
  type = "lower",
  method = "square",
  order = "alphabet",
  col = COL2("BrBG", 10),
  tl.col = "black"
)

## ----warning=FALSE------------------------------------------------------------
group_counts <- counts %>%
  rownames_to_column(var = "Compound") %>%
  pivot_longer(
    cols = starts_with("102623"),
    names_to = "injection",
    values_to = "intensity"
  ) %>%
  left_join(meta, by = "injection") %>%
  summarise(
    mean_intensity = mean(intensity),
    .by = c(Compound, biological_group)
  ) %>%
  pivot_wider(
    names_from = biological_group,
    values_from = mean_intensity
  ) %>%
  column_to_rownames(var = "Compound")

## -----------------------------------------------------------------------------
group_counts_cor <- rcorr(as.matrix(group_counts), type = "spearman")

## -----------------------------------------------------------------------------
corrplot(group_counts_cor$r,
  type = "lower",
  method = "square",
  order = "alphabet",
  col = COL2("BrBG", 10),
  tl.col = "black"
)

## -----------------------------------------------------------------------------
dist <- stats::dist(t(counts), method = "euclidian")
cluster <- stats::hclust(dist, method = "complete")

## -----------------------------------------------------------------------------
dendro <- as.dendrogram(cluster)

## -----------------------------------------------------------------------------
den_data <- dendro_data(dendro, type = "rectangle")

## -----------------------------------------------------------------------------
ggplot(segment(den_data)) +
  geom_segment(aes(x = x, y = y, xend = xend, yend = yend)) +
  geom_text(
    data = den_data$labels,
    aes(x = x, y = y, label = label),
    size = 3,
    hjust = "outward"
  ) +
  coord_cartesian(ylim = c((min(segment(den_data)$y) + 10),
                           (max(segment(den_data)$y)))) +
  coord_flip() +
  scale_y_reverse(expand = c(.5, 0)) +
  theme_dendro()

## -----------------------------------------------------------------------------
get_group_averages(data_filtered) %>%
  filter(biological_group == "Coculture" |
           biological_group == "ANG18") %>%
  select(Compound, biological_group, average) %>%
  pivot_wider(names_from = biological_group, values_from = average) %>%
  mutate(fc = Coculture / ANG18) %>%
  head()

## -----------------------------------------------------------------------------
get_group_averages(data_filtered) %>%
  filter(biological_group == "Coculture" |
           biological_group == "ANG18") %>%
  select(Compound, biological_group, average) %>%
  pivot_wider(names_from = biological_group, values_from = average) %>%
  mutate(nonzero_compound = if_else(Coculture == 0 & ANG18 == 0,
                                    FALSE,
                                    TRUE)) %>%
  filter(nonzero_compound == TRUE) %>%
  mutate(fc = Coculture / ANG18) %>%
  head()

## -----------------------------------------------------------------------------
get_group_averages(data_filtered) %>%
  filter(biological_group == "Coculture" |
           biological_group == "ANG18") %>%
  select(Compound, biological_group, average) %>%
  head()

## -----------------------------------------------------------------------------
get_group_averages(data_filtered) %>%
  filter(biological_group == "Coculture" |
           biological_group == "ANG18") %>%
  select(Compound, biological_group, average) %>%
  head()

## -----------------------------------------------------------------------------
get_group_averages(data_filtered) %>%
  filter(biological_group == "Coculture" |
           biological_group == "ANG18") %>%
  select(Compound, biological_group, average) %>%
  mutate(average = average + 0.001) %>%
  pivot_wider(names_from = biological_group, values_from = average) %>%
  head()

## -----------------------------------------------------------------------------
get_group_averages(data_filtered) %>%
  filter(biological_group == "Coculture" |
           biological_group == "ANG18") %>%
  select(Compound, biological_group, average) %>%
  mutate(average = average + 0.001) %>%
  pivot_wider(names_from = biological_group, values_from = average) %>%
  mutate(nonzero_compound = if_else(Coculture == 0.001 & ANG18 == 0.001,
                                    FALSE,
                                    TRUE)) %>%
  filter(nonzero_compound == TRUE) %>%
  head()

## -----------------------------------------------------------------------------
get_group_averages(data_filtered) %>%
  filter(biological_group == "Coculture" |
           biological_group == "ANG18") %>%
  select(Compound, biological_group, average) %>%
  mutate(average = average + 0.001) %>%
  pivot_wider(names_from = biological_group, values_from = average) %>%
  mutate(nonzero_compound = if_else(Coculture == 0.001 & ANG18 == 0.001,
                                    FALSE,
                                    TRUE)) %>%
  filter(nonzero_compound == TRUE) %>%
  mutate(fc = Coculture / ANG18) %>%
  head()

## -----------------------------------------------------------------------------
foldchanges <- get_group_averages(data_filtered) %>%
  filter(biological_group == "Coculture" |
           biological_group == "ANG18") %>%
  select(Compound, biological_group, average) %>%
  mutate(average = average + 0.001) %>%
  pivot_wider(names_from = biological_group, values_from = average) %>%
  mutate(nonzero_compound = if_else(Coculture == 0.001 & ANG18 == 0.001,
                                    FALSE,
                                    TRUE)) %>%
  filter(nonzero_compound == TRUE) %>%
  mutate(fc = Coculture / ANG18,
         logfc = log2(fc))

head(foldchanges)

## -----------------------------------------------------------------------------
fc_plotting <- foldchanges %>%
  left_join(select(ft, Compound, mz, rt), by = "Compound")

plot_ly(fc_plotting,
  x = ~logfc, y = ~rt, z = ~mz,
  marker = list(color = ~logfc, showscale = TRUE)
) %>%
  add_markers() %>%
  layout(
    scene = list(
      xaxis = list(title = "log2 Fold Change"),
      yaxis = list(title = "Retention Time (min)"),
      zaxis = list(title = "m/z")
    ),
    annotations = list(
      x = 1.13,
      y = 1.05,
      text = "log2 Fold Change",
      xref = "paper",
      yref = "paper",
      showarrow = FALSE
    )
  )

## -----------------------------------------------------------------------------
# Satterwaite
calc_samplesize_ws <- function(sd1, n1, sd2, n2) {
  s1 <- sd1 / (n1^0.5)
  s2 <- sd2 / (n2^0.5)

  n <- (s1^2 / n1 + s2^2 / n2)^2
  d1 <- s1^4 / ((n1^2) * (n1 - 1))
  d2 <- s2^4 / ((n2^2) * (n2 - 1))

  d1[which(!is.finite(d1))] <- 0
  d2[which(!is.finite(d2))] <- 0

  d <- d1 + d2

  return(n / d)
}

my_comp <- c("Coculture", "ANG18")

stats <- get_group_averages(data_filtered) %>%
  mutate(
    combRSD = sqrt(techRSD^2 + BiolRSD^2),
    combASD = combRSD * average,
    combASD = if_else(is.na(combASD), 0, combASD),
    BiolRSD = if_else(is.na(BiolRSD), 0, BiolRSD),
    techRSD = if_else(is.na(techRSD), 0, techRSD),
    neff = calc_samplesize_ws(techRSD, techn, BiolRSD, Bioln) + 1
  ) %>%
  filter(biological_group %in% my_comp)

head(stats)

## -----------------------------------------------------------------------------
denom <- stats %>%
  summarise(den = combASD^2 / (neff),
            .by = c("Compound", "biological_group")) %>%
  mutate(den = if_else(!is.finite(den), 0, den)) %>%
  summarise(denom = sqrt(sum(den)), .by = c("Compound"))

t_test <- stats %>%
  select(Compound, biological_group, average) %>%
  pivot_wider(names_from = biological_group, values_from = average) %>%
  mutate(numerator = (Coculture - ANG18)) %>% # experimental - control
  left_join(denom, by = "Compound") %>%
  mutate(t = abs(numerator / denom))

head(t_test)

## -----------------------------------------------------------------------------
df <- stats %>%
  select(Compound, biological_group, neff) %>%
  mutate(neff = if_else(!is.finite(neff), 0, neff)) %>%
  pivot_wider(names_from = biological_group, values_from = neff) %>%
  mutate(deg = Coculture + ANG18 - 2) %>%
  select(Compound, deg)

head(df)

## -----------------------------------------------------------------------------
t <- t_test %>%
  left_join(df, by = "Compound") %>%
  mutate(
    p = (1 - pt(t, deg)) * 2,
    logp = log10(p),
    neg_logp = -logp
  ) %>%
  select(Compound, t, deg, p, logp, neg_logp)

head(t)

## -----------------------------------------------------------------------------
num_ions <- t %>%
  filter(p <= 1) %>%
  count() %>%
  pull()

fc <- foldchanges %>%
  left_join(t, by = "Compound") %>%
  arrange(p) %>%
  mutate(
    qval = seq_len(length(p)),
    qval = p * num_ions / qval
  ) %>%
  arrange(desc(p))

min_qval <- 1

for (i in seq_len(nrow(fc))) {
  if (!is.finite(fc$qval[i])) {
    next
  }

  if (fc$qval[i] < min_qval) {
    min_qval <- fc$qval[i]
  } else {
    fc$qval[i] <- min_qval
  }
}

fc2 <- fc %>%
  mutate(neg_logq = -log10(qval))

## -----------------------------------------------------------------------------
fc2 %>%
  ggplot() +
  aes(x = logfc, y = neg_logp) +
  geom_point() +
  labs(x = "log2 Fold Change",
       y = "-Log~10~ *P*",
       color = "Differential Abundance") +
  theme_bw()

## -----------------------------------------------------------------------------
fc2 %>%
  ggplot() +
  aes(x = logfc, y = neg_logp) +
  geom_point() +
  geom_hline(yintercept = -log10(0.05), linetype = "dashed") +
  labs(x = "log2 Fold Change",
       y = "-Log~10~ *P*",
       color = "Differential Abundance") +
  theme_bw()

## -----------------------------------------------------------------------------
fc2 %>%
  ggplot() +
  aes(x = logfc, y = neg_logp) +
  geom_point() +
  geom_hline(yintercept = -log10(0.05), linetype = "dashed") +
  labs(x = "log2 Fold Change",
       y = "-Log~10~ *P*",
       color = "Differential Abundance") +
  geom_vline(xintercept = log2(1.5), linetype = "dashed") +
  geom_vline(xintercept = -log2(1.5), linetype = "dashed") +
  theme_bw()

## -----------------------------------------------------------------------------
fc2 %>%
  mutate(
    sig = case_when(
      p > 0.05 ~ "not_sig",
      p <= 0.05 & logfc > log2(1.5) ~ "Increased",
      p <= 0.05 & logfc < -log2(1.5) ~ "Decreased",
      TRUE ~ "Inconclusive"
    ),
    sig = factor(sig,
      levels = c("Increased", "Decreased", "Inconclusive", "not_sig"),
      labels = c("Increased", "Decreased", "Inconclusive", "Not significant")
    )
  ) %>%
  select(Compound, ANG18, Coculture, fc, logfc, p, sig) %>%
  head()

## -----------------------------------------------------------------------------
fc2 %>%
  mutate(
    sig = case_when(
      p > 0.05 ~ "not_sig",
      p <= 0.05 & logfc > log2(1.5) ~ "Increased",
      p <= 0.05 & logfc < -log2(1.5) ~ "Decreased",
      TRUE ~ "Inconclusive"
    ),
    sig = factor(sig,
      levels = c("Increased", "Decreased", "Inconclusive", "not_sig"),
      labels = c("Increased", "Decreased", "Inconclusive", "Not significant")
    )
  ) %>%
  ggplot() +
  aes(x = logfc, y = neg_logp, color = sig) +
  geom_point() +
  geom_hline(yintercept = -log10(0.05), linetype = "dashed") +
  geom_vline(xintercept = log2(1.5), linetype = "dashed") +
  geom_vline(xintercept = -log2(1.5), linetype = "dashed") +
  scale_color_manual(values = c("red", "blue", "grey", "black")) +
  labs(
    x = "log2 Fold Change",
    y = "-Log~10~ *P*",
    color = "Differential Abundance"
  ) +
  theme_bw()

## -----------------------------------------------------------------------------
fc2 %>%
  mutate(
    sig = case_when(
      p > 0.05 ~ "not_sig",
      p <= 0.05 & logfc > log2(1.5) ~ "Increased",
      p <= 0.05 & logfc < -log2(1.5) ~ "Decreased",
      TRUE ~ "Inconclusive"
    ),
    sig = factor(sig,
      levels = c("Increased", "Decreased", "Inconclusive", "not_sig"),
      labels = c("Increased", "Decreased", "Inconclusive", "Not significant")
    )
  ) %>%
  ggplot() +
  aes(x = logfc, y = neg_logp, color = sig) +
  geom_point() +
  geom_hline(yintercept = -log10(0.05), linetype = "dashed") +
  geom_vline(xintercept = log2(1.5), linetype = "dashed") +
  geom_vline(xintercept = -log2(1.5), linetype = "dashed") +
  scale_color_manual(values = c("red", "blue", "grey", "black")) +
  labs(
    x = "log2 Fold Change",
    y = "-Log~10~ *P*",
    color = "Differential Abundance"
  ) +
  theme_bw() +
  theme(
    axis.title = element_markdown(size = 20),
    axis.text = element_text(size = 15)
  )

## -----------------------------------------------------------------------------
volcano <- fc2 %>%
  mutate(
    sig = case_when(
      p > 0.05 ~ "not_sig",
      p <= 0.05 & logfc > log2(1.5) ~ "Increased",
      p <= 0.05 & logfc < -log2(1.5) ~ "Decreased",
      TRUE ~ "Inconclusive"
    ),
    sig = factor(sig,
      levels = c("Increased", "Decreased", "Inconclusive", "not_sig"),
      labels = c("Increased", "Decreased", "Inconclusive", "Not significant")
    )
  ) %>%
  ggplot() +
  aes(
    x = logfc, y = neg_logp, color = sig,
    text = paste0("Compound: ", Compound)
  ) +
  geom_point() +
  geom_hline(yintercept = -log10(0.05), linetype = "dashed") +
  geom_vline(xintercept = log2(1.5), linetype = "dashed") +
  geom_vline(xintercept = -log2(1.5), linetype = "dashed") +
  scale_color_manual(values = c("red", "blue", "grey", "black")) +
  labs(
    x = "log2 Fold Change",
    y = "-Log~10~ *P*",
    color = "Differential Abundance"
  ) +
  theme_bw() +
  theme(
    axis.title = element_markdown(size = 20),
    axis.text = element_text(size = 15)
  )

ggplotly(volcano)

## -----------------------------------------------------------------------------
fc2 %>%
  mutate(
    sig = case_when(
      qval > 0.05 ~ "not_sig",
      qval <= 0.05 & logfc > log2(1.5) ~ "Increased",
      qval <= 0.05 & logfc < -log2(1.5) ~ "Decreased",
      TRUE ~ "Inconclusive"
    ),
    sig = factor(sig,
      levels = c("Increased", "Decreased", "Inconclusive", "not_sig"),
      labels = c("Increased", "Decreased", "Inconclusive", "Not significant")
    )
  ) %>%
  ggplot() +
  aes(x = logfc, y = neg_logq, color = sig) +
  geom_point() +
  geom_hline(yintercept = -log10(0.05), linetype = "dashed") +
  geom_vline(xintercept = log2(1.5), linetype = "dashed") +
  geom_vline(xintercept = -log2(1.5), linetype = "dashed") +
  scale_color_manual(values = c("red", "blue", "grey", "black")) +
  labs(
    x = "log2 Fold Change",
    y = "-Log~10~ q-value",
    color = "Differential Abundance"
  ) +
  theme_bw() +
  theme(
    axis.title = element_markdown(size = 20),
    axis.text = element_text(size = 15)
  )

#' Import data into an mpactr object.
#'
#' @description
#' `import_data()` takes two file paths, one for the pre-processed feature
#' table and one for sample metadata. Both files should be .csv.
#'
#' @details
#' mpactr requires a peak table and meta data as input. Files are expected to be
#' comma separated files (*.csv*).
#'
#' 1. `peak_table`: a peak table where rows are expected to be compounds. mpactr
#' supports import of feature table files from multiple tools through the
#' `format` argument. Currently supported value for `format` are "Progenesis",
#' "Metaboscape", or "None".
#'
#' `format` = "Progenesis." allows users to provide a feature table exported by
#' Progenesis. To export a compatible peak table in Progenesis, navigate to the
#' *Review Compounds* tab then File -> Export Compound Measurements. Select
#' the following properties: Compound, m/z, Retention time (min), and Raw
#' abundance and click ok.
#'
#' `format` = "Metaboscape" allows users to provide a feature table exported by
#' Metaboscape with default settings. The import function will save the raw peak
#' table in the `mpactr_object` and store a formatted peak table for filtering.
#' Reformatting includes selecting "FEATURE_ID", "RT", "PEPMASS", and sample
#' columns. Sample columns are determined from the "Injection" column in
#' `meta_data` (see below). "PEPMASS" is converted to m/z using the "ADDUCT"
#' column and compound metadata columns are renamed for mpactr.
#'
#' `format` = "None" allows users to provide a feature table file in the
#' expected format. This can be useful if you have a file from another tool and
#' want to manually format it in R. The table rows are expected to be individual
#' features, while columns are compound metadata and samples. The feature table
#' must have the compound metadata columns "Compound", "mz", and "rt". Where
#' "Compound" is the compound id, and can be `numeric` or `character`. "mz" is
#' the compound m/z, and should be `numeric`. "rt" is the retention time, in
#' minutes, and should be `numeric`. The remaining columns should be samples,
#' and match the names in the "Injection" column of the `meta_data` file.

#' 2. `meta_data`: a table with sample information. Either a file path or
#' `data.frame` can be supplied. At minimum the following columns are expected:
#' "Injection", "Sample_Code", and "Biological_Group". "Injection" is the sample
#' name and is expected to match sample column names in the `peak_table`.
#' "Sample_Code" is the id for technical replicate groups. "Biological_Group"
#' is the id for biological replicate groups. Other sample metadata can be
#' added, and is encouraged for downstream analysis following filtering with
#' mpactr.
#'
#' @param peak_table The file path or valid `https` url to your feature table
#' file.
#' @param meta_data The file path to your meta_data file or `data.frame`.
#' @param format The expected exported type of your peak table, can be
#' one of "Progenesis", "Metaboscape", "None".
#'
#' @return an `mpactr_object`.
#' @export
#'
#' @examples
#'  \dontshow{
#' limit_cores()
#' }
#' data <- import_data(
#'   example_path("coculture_peak_table.csv"),
#'   example_path("metadata.csv"),
#'   format = "Progenesis"
#' )
#'
#' meta_data <- read.csv(example_path("metadata.csv"))
#' data <- import_data(example_path("coculture_peak_table.csv"),
#'   meta_data,
#'   format = "Progenesis"
#' )
#'
import_data <- function(peak_table, meta_data, format = "none") {
  if (!any(class(meta_data) %in% c("data.table", "data.frame"))) {
    meta_data <- fread(meta_data)
  }

  #*** check for Injection, Sample_Code, Biological_Group
  cols <- c("injection", "sample_code", "biological_group")
  colnames(meta_data) <- tolower(colnames(meta_data))
  if (any(cols %in% colnames(meta_data) == FALSE)) {
    cli::cli_abort("{.cls {cols[which(!(cols %in% colnames(meta_data)))]}}
                    are not columns in the provided metadata. Please see
                     function documentation for more details.")
  }


  df <- format_by_type(
    peak_table_path = peak_table,
    type_of_peak_table = format,
    sample_names = meta_data$injection
  )

  mpactr_object <- mpactr$new(
    peak_table = unique_compounds(df),
    meta_data = data.table(meta_data)
  )
  mpactr_object$setup()
  filter_object <- filter_pactr$new(mpactr_object)
  return(filter_object)
}

unique_compounds <- function(peak_table_list, show_message = TRUE) {
  peak_table <- peak_table_list$peak_table
  duplicates <- names(which(table(peak_table$Compound) > 1))
  if (any(is.na(peak_table$Compound))) {
    stop("Found NA values inside your compounds names,
    please remove them.")
  }
  if (length(duplicates) > 0 && show_message) {
    cli::cli_inform("Found duplicate compound values, will add a suffix to
    unique the value.")
    peak_table$Compound <- UniqueDuplicates(as.character(peak_table$Compound))
  }
  return(list(
    "peak_table" = peak_table,
    "raw_table" = peak_table
  ))
}

#' Summary of Filtering
#'
#' @description
#' Parses an mpactr object and extracts a summary of all applied filters.
#' Specifically, the fate of each input ion is reported as ion status. Status
#' options are: Passed, mispicked, group, replicability, and insouce. A status
#' of Passed ions is returned for ions that passed all applied filters and
#' therefore are expected to be high quality ions. Ions tagged as group,
#' mispicked, replicability, or ionsource were removed during the corresponding
#' filter.
#'
#' @param mpactr_object an `mpactr_object`.
#'
#' @return a `data.table` reporting the number of high quality ions
#' ("Passed") or the filter in which they were removed.
#' @export
#'
#' @examples
#'  \dontshow{
#' limit_cores()
#' }
#' data <- import_data(
#'   example_path("coculture_peak_table.csv"),
#'   example_path("metadata.csv"),
#'   format = "Progenesis"
#' )
#'
#' data_filter <- filter_mispicked_ions(data,
#'   ringwin = 0.5,
#'   isowin = 0.01,
#'   trwin = 0.005,
#'   max_iso_shift = 3,
#'   merge_peaks = TRUE
#' )
#'
#' summary <- qc_summary(data_filter)
#' summary
qc_summary <- function(mpactr_object) {
  graph_pactr_object <- graph_qc_pactr$new(mpactr_object)
  graph_pactr_object$generate_QC_Summary()
  return(graph_pactr_object$get_summarized_dt())
}

#' Visualize Filtering Summary as Tree Map
#'
#' @description
#' `plot_qc_tree()` visualizes the filtering summary as a treemap. Ion
#' status (see [qc_summary()]) is reported here as percentage of all
#' pre-filtered ions.
#'
#' @param mpactr_object an `mpactr_object`.
#'
#' @return a tree map plot of class `ggplot`.
#' @export
#'
#' @examples
#' data <- import_data(
#'   example_path("coculture_peak_table.csv"),
#'   example_path("metadata.csv"),
#'   format = "Progenesis"
#' )
#'
#' data_filter <- filter_mispicked_ions(data,
#'   ringwin = 0.5,
#'   isowin = 0.01,
#'   trwin = 0.005,
#'   max_iso_shift = 3,
#'   merge_peaks = TRUE
#' )
#'
#' plot_qc_tree(data_filter)
plot_qc_tree <- function(mpactr_object) {
  graph_pactr_object <- graph_qc_pactr$new(mpactr_object)
  graph_pactr_object$generate_QC_Summary()
  return(graph_pactr_object$plot_QC_Tree())
}

#' @title Limit Cores
#'
#' @description
#' Limits the amount of cores used to two. Mostly used in test
#' and examples so we can pass cran checks.
#' @return does not return anything.
#' @export
#' @examples
#' limit_cores()
#'
limit_cores <- function() {
  Sys.setenv("OMP_THREAD_LIMIT" = 1)
  setDTthreads(threads = 1)
}

test_that("mpactr class setup works properly", {
  limit_cores()
  directory <- "exttestdata"
  peak_table_name <- "102623_peaktable_coculture_simple.csv"
  meta_data_name <- "102623_metadata_correct.csv"
  meta <- fread(test_path(directory, meta_data_name))
  pt_list <- progenesis_formatter(test_path(directory, peak_table_name))

  mpactr_class <- mpactr$new(
    pt_list,
    meta
  )
  mpactr_class$setup()

  expect_true(all(c("kmd", "mz", "rt") %in%
                    colnames(mpactr_class$get_peak_table())))
  expect_equal(nrow(mpactr_class$get_peak_table()), 1303)
})

test_that("example path returns data paths properly", {
  limit_cores()
  expect_true(length(example_path()) > 0)
  expect_true(length(example_path("coculture_peak_table.csv")) >
                0)
  expect_error(example_path("a"))
})

####  filter 1: mismatched peaks    ###
filter_pactr$set(
  "public", "check_mismatched_peaks",
  function(ringwin,
           isowin,
           trwin,
           max_iso_shift,
           merge_peaks,
           merge_method = NULL) {
    l <- nrow(self$mpactr_data$get_peak_table())
    cli::cli_alert_info("Checking {l} peaks for mispicked peaks.")
    self$mpactr_data$set_peak_table(self$mpactr_data$get_peak_table()[
      order(self$mpactr_data$get_peak_table()$mz,
        decreasing = FALSE
      ),
    ])
    ion_filter_list <- list()
    results <- FilterMispickedIons(self$mpactr_data$get_peak_table(), ringwin,
                                   isowin, trwin, max_iso_shift)

    ion_filter_list[["cut_ions"]] <- results$cut_ions
    ion_filter_list[["merge_groups"]] <- results$merge_groups
    self$logger[["check_mismatched_peaks"]] <- ion_filter_list

    if (isTRUE(merge_peaks)) {
      cli::cli_alert_info(c("Argument merge_peaks is: {merge_peaks}. ",
                            "Merging mispicked peaks with method ",
                            "{merge_method}."))
      private$merge_ions(ion_filter_list, merge_method)
    } else {
      cli::cli_alert_warning(c("Argument merge_peaks is: {merge_peaks}. ",
                               "Mispicked peaks will not be merged."))
    }

    self$logger$list_of_summaries$mispicked <- summary$new(
      filter = "mispicked", failed_ions = results$cut_ions,
      passed_ions = self$mpactr_data$get_peak_table()$Compound
    )

    self$logger$list_of_summaries$mispicked$summarize()
  }
)

filter_pactr$set("private", "merge_ions", function(ion_filter_list, method) {
  if (is.null(method)) {
    cli::cli_abort(c("No method has been supplied for merging peaks. ",
                     "{.var method} must be one of: sum"))
  }

  if (length(ion_filter_list[["cut_ions"]]) <= 0) {
    return()
  }
  if (method == "sum") {
    dat <- melt(self$mpactr_data$get_peak_table(),
      id.vars = c("Compound", "mz", "rt", "kmd"),
      variable.name = "sample",
      value.name = "intensity",
      variable.factor = FALSE
    )

    for (ion in names(ion_filter_list$merge_groups)) {
      dat <- dat[Compound %in% c(ion, ion_filter_list$merge_groups[[ion]]),
        intensity := sum(intensity),
        by = .(sample)
      ]
    }
    self$mpactr_data$set_peak_table(dcast(dat,
      Compound + mz + kmd + rt ~ sample,
      value.var = "intensity"
    )[
      (!Compound %in% ion_filter_list$cut_ions),
    ])
  }
})

####  filter 2: group filter    ###
# Calculates statisics for each feature (rsd, n)
# across biological groups and technical replicates
filter_pactr$set("public", "filter_blank", function() {
  b <- data.table::melt(self$mpactr_data$get_peak_table(),
    id.vars = c("Compound", "mz", "rt", "kmd"), variable.name =
      "sample", value.name = "intensity", variable.factor = FALSE
  )[
    data.table(self$mpactr_data$get_meta_data()),
    on = .(sample = injection)
  ][
    , .(
      average = mean(intensity),
      BiolRSD = rsd(intensity),
      Bioln = length(intensity)
    ),
    by = .(Compound, biological_group)
  ]

  t <- data.table::melt(self$mpactr_data$get_peak_table(),
    id.vars = c("Compound", "mz", "rt", "kmd"),
    variable.name = "sample",
    value.name = "intensity",
    variable.factor = FALSE
  )[
    data.table(self$mpactr_data$get_meta_data()),
    on = .(sample = injection)
  ][
    , .(sd = rsd(intensity), n = length(intensity)),
    by = .(Compound, biological_group, sample_code)
  ][
    , .(techRSD = mean(sd), techn = mean(n)),
    by = .(Compound, biological_group)
  ]
  group_stats <- b[t, on = .(Compound, biological_group)]
  setorder(group_stats, Compound, biological_group)
  self$logger[["group_filter-group_stats"]] <- group_stats
})

# this function determines group ions above the group threshold
# given group statistics (see filter_blank).
# The result is a list of ions by group whose relative abundance
# is greater than the threshold.
filter_pactr$set(
  "public", "parse_ions_by_group",
  function(group_threshold = 0.01) {
    group_avgs <- dcast(self$logger[["group_filter-group_stats"]],
      Compound ~ biological_group,
      value.var = "average"
    )

    max <- self$logger[["group_filter-group_stats"]][
      , .(Compound, biological_group, average)
    ][
      , .(max = max(average)),
      by = Compound
    ]

    group_max <- group_avgs[max, on = .(Compound = Compound)][
      , lapply(.SD, function(x) {
        x / max
      }),
      .SDcols = unique(
        self$logger[["group_filter-group_stats"]]$biological_group
      )
    ]

    biol_groups <- as.list(group_max)
    biol_groups <- lapply(biol_groups, setNames, group_avgs[, Compound])
    group_filter_list <- lapply(biol_groups, function(x) {
      names(x)[which(x > group_threshold)]
    })
    self$logger[["group_filter-failing_list"]] <- group_filter_list
  }
)


# Given a group name, removes flagged ions from the peak table.
filter_pactr$set(
  "public", "apply_group_filter",
  function(group, remove_ions = TRUE) {
    groups <- unique(self$mpactr_data$get_meta_data()$biological_group)
    if (isFALSE(group %in% groups)) {

      cli::cli_abort(c("{.var group} {group} is not in ",
                       "{.var biological_group}.",
                       "Options are: {groups}"))
    }

    l <- nrow(self$mpactr_data$get_peak_table())
    cli::cli_alert_info("Parsing {l} peaks based on the sample group: {group}.")

    if (isFALSE(remove_ions)) {
      cli::cli_alert_warning(c("Argument remove_ions is {remove_ions}. ",
                               "Peaks from {group} will not be removed."))
      return()
    }
    cli::cli_alert_info(c("Argument remove_ions is: {remove_ions}.",
                          "Removing peaks from {group}."))

    ions <- self$logger[["group_filter-failing_list"]][[group]]
    self$mpactr_data$set_peak_table(self$mpactr_data$get_peak_table()[
      !(self$mpactr_data$get_peak_table()$Compound
        %in% ions),
    ])

    self$logger$list_of_summaries[[paste0("group-", group)]] <- summary$new(
      filter = group,
      failed_ions = ions,
      passed_ions = self$mpactr_data$get_peak_table()$Compound
    )


    self$logger$list_of_summaries[[paste0("group-", group)]]$summarize()
  }
)

####  filter 3: cv filter    ###
filter_pactr$set(
  "public", "cv_filter",
  function(cv_threshold = NULL) {
    if (is.null(cv_threshold)) {
      cli::cli_abort("{.var cv_threshold} must be supplied.")
    }

    ## abort if there are no technical replicates.
    if (isFALSE(self$mpactr_data$isMultipleTechReps())) {
      cli_abort(c("There are no technical replicates in the dataset provided. ",
                  "In order to run the replicability filter, technical ",
                  "replicates are required."))
    }

    input_ions <- self$mpactr_data$get_peak_table()$Compound
    cli <- cli::cli_alert_info
    n <- length(input_ions)
    cli("Parsing {n} peaks for replicability across technical replicates.")

    cv <- data.table::melt(self$mpactr_data$get_peak_table(),
      id.vars = c("Compound", "mz", "rt", "kmd"), variable.name =
        "sample", value.name = "intensity", variable.factor = FALSE
    )[
      self$mpactr_data$get_meta_data(),
      on = .(sample = injection)
    ][order(Compound)]


    peak_table <- self$mpactr_data$get_peak_table()
    meta_data <- self$mpactr_data$get_meta_data()

    cv <-
      as.data.table(FilterCV(cv, unique(meta_data$sample_code), cv_threshold,
                             table(meta_data$sample_code)[[1]]))

    samples <- unique(meta_data$sample_code)
    for (i in seq_along(samples)) {
      peak_table[Compound %in% cv[sample_code == samples[[i]] &
                                    !passes_cv_filter]$Compound,
                 meta_data$injection[which(meta_data$sample_code
                                           == samples[[i]])] := 0]
    }

    failed_indexes <- which(rowSums(peak_table[, meta_data$injection,
                                               with = FALSE]) == 0)


    self$logger[["cv_values"]] <- cv
    failed_ions <- peak_table$Compound[failed_indexes]
    self$mpactr_data$set_peak_table(self$mpactr_data$get_peak_table()[
      Compound %in% setdiff(
        input_ions,
        failed_ions
      ),
    ])

    self$logger$list_of_summaries$replicability <- summary$new(
      filter = "cv_filter",
      failed_ions = failed_ions,
      passed_ions = self$mpactr_data$get_peak_table()$Compound
    )

    self$logger$list_of_summaries$replicability$summarize()
  }
)
####  filter 4: insource ions   ###

filter_pactr$set(
  "public", "filter_insource_ions",
  function(cluster_threshold = 0.95) {
    input_ions <- self$mpactr_data$get_peak_table()$Compound
    cli::cli_alert_info("Parsing {length(input_ions)} peaks for insource ions.")

    self$mpactr_data$set_peak_table(self$mpactr_data$get_peak_table()[
      , cor := private$deconvolute_correlation(.SD, cluster_threshold),
      by = .(rt)
    ][cor == TRUE, ])

    self$logger$list_of_summaries$insource <- summary$new(
      filter = "insource",
      failed_ions = setdiff(
        input_ions,
        self$mpactr_data$get_peak_table()$Compound
      ),
      passed_ions = self$mpactr_data$get_peak_table()$Compound
    )

    self$logger$list_of_summaries$insource$summarize()
  }
)

filter_pactr$set(
  "private", "deconvolute_correlation",
  function(group_1, cluster_threshold = 0.95) {
    if (nrow(group_1) <= 1) {
      return(TRUE)
    }

    dat <- melt(group_1,
      id.vars = c("Compound", "mz", "kmd"),
      variable.name = "sample",
      value.name = "intensity",
      variable.factor = FALSE
    )[
      , c("Compound", "sample", "intensity")
    ]
    data <- dcast(dat, sample ~ Compound, value.var = "intensity")
    data[, c("sample") := NULL]
    corr <- stats::cor(data, method = c("pearson"))
    dist <- stats::dist(corr, method = "euclidian")
    cluster <- stats::hclust(dist, method = "complete")
    cut_tree <- stats::cutree(cluster, h = 1 - cluster_threshold)

    x <- as.data.table(cut_tree, keep.rownames = "Compound")[
      group_1,
      on = .(Compound = Compound)
    ][
      , keep := private$cluster_max(mz),
      by = .(cut_tree)
    ]

    return(x$keep)
  }
)

# Given a list of mz values, this function will determine
# which value has the largest mz and modify its "KEEP" Column.
filter_pactr$set("private", "cluster_max", function(mz) {
  keep <- rep(FALSE, length(mz))
  keep[which.max(mz)] <- TRUE
  return(keep)
})

graph_qc_pactr <- R6Class("graph_qc_pactr", public = list(
  initialize = function(filter_pactr) {
    private$filter_pactr_data <- filter_pactr
  },
  generate_QC_Summary = function() {
    list_of_failed_ions <-
      lapply(
        private$filter_pactr_data$logger$list_of_summaries,
        function(x) {
          x$get_failed_ions()
        }
      )
    status <- gsub("[[:digit:]]+", "", names(unlist(list_of_failed_ions)))
    compounds <- unlist(unname(list_of_failed_ions))

    failed_ions_dt <- data.table(status, compounds)

    passed_ions_dt <- data.table(
      "status" = rep("Passed", length(
        private$filter_pactr_data$mpactr_data$get_peak_table()$Compound
      )),
      "compounds" =
        private$filter_pactr_data$mpactr_data$get_peak_table()$Compound
    )

    private$filter_summarized <- rbind(passed_ions_dt, failed_ions_dt)
  },
  plot_QC_Tree = function() {
    ion_counts <- private$filter_summarized[, .(count = .N), by = status][
      , percent := (count / sum(count) * 100)
    ]

    return(ggplot2::ggplot(ion_counts) +
      ggplot2::aes(area = percent, fill = status) +
      treemapify::geom_treemap() +
      treemapify::geom_treemap_text(
        aes(
          label = paste(status, count, paste0("(", round(percent, 2), "%)"),
                        sep = "\n")
        ),
        colour = "darkorchid1",
        fontface = c("bold")
      ) +
      ggplot2::theme(legend.position = "none") +
      viridis::scale_fill_viridis(option = "G", discrete = TRUE))
  },
  get_summarized_dt = function() {
    return(private$filter_summarized)
  }
), private = list(
  filter_summarized = NA,
  filter_pactr_data = NA
))

test_that("plot_QC_Tree generates the correct plot", {
  limit_cores()
  peak_table_name <- "102623_peaktable_coculture_simple.csv"
  meta <- fread(test_path("exttestdata",
                          "102623_metadata_correct.csv"))
  pt_list <- progenesis_formatter(test_path("exttestdata", peak_table_name))

  mpactr_class <- mpactr$new(
    pt_list,
    meta
  )
  mpactr_class$setup()
  filter_class <- filter_pactr$new(mpactr_class)
  filter_class$check_mismatched_peaks(
    ringwin = 0.5,
    isowin = 0.01,
    trwin = 0.005,
    max_iso_shift = 3,
    merge_peaks = TRUE,
    merge_method = "sum"
  )
  filter_class$filter_blank()
  filter_class$parse_ions_by_group(group_threshold = 0.01)
  filter_class$apply_group_filter("Blanks", remove_ions = TRUE)
  filter_class$filter_insource_ions(cluster_threshold = 0.95)

  graph_qc_pactr_class <- graph_qc_pactr$new(filter_class)
  graph_qc_pactr_class$generate_QC_Summary()
  plot <- graph_qc_pactr_class$plot_QC_Tree()
  expect_true(is_ggplot(plot))
})

#' LC-MS/MS sample data
#'
#' A `mpactr` R6 class object containing a feature table and associated sample
#' metadata.
#'
#' @return An `mpactr` R6 class object.
#' @format ## `culture_data`
#' A `mpactr` with 2 attributes:
#' \describe{
#'  \item{peak_table}{A feature table of class `data.table`}
#'  \item{meta_data}{A `data.table` with associated sample metadata}
#'  }
"cultures_data"


#' Get file paths for examples
#'
#' mpactr contains a number of example files in the `inst/extdata` directory.
#' This function makes them accessible in documentation that shows how file
#' paths are used in function examples.
#'
#' @param file Name of a file. If `NULL`, all examples files will be listed.
#'
#' @export
#' @return A file path to example data stored in the `inst/extdata` directory
#' of the package.
#' @examples
#' example_path()
#'
#' example_path("metadata.csv")
example_path <- function(file = NULL) {
  path <- ""
  if (is.null(file)) {
    path <- dir(system.file("extdata", package = "mpactr"))
  } else {
    path <- system.file("extdata", file, package = "mpactr", mustWork = TRUE)
  }
  return(path)
}

# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

FilterMispickedIons <- function(peakTable, ringWin, isoWin, trWin, maxIsoShift) {
    .Call(`_mpactr_FilterMispickedIons`, peakTable, ringWin, isoWin, trWin, maxIsoShift)
}

FilterCV <- function(peakTable, uniqueSampleList, cvCutOff, replicates) {
    .Call(`_mpactr_FilterCV`, peakTable, uniqueSampleList, cvCutOff, replicates)
}

UniqueDuplicates <- function(compoundNames) {
    .Call(`_mpactr_UniqueDuplicates`, compoundNames)
}

mpactr$set("public", "setup", function() {
  private$initialize_data()
})
mpactr$set("private", "initialize_data", function() {
  private$peak_table <- private$peak_table[which(rowSums(
    private$peak_table[, .SD, .SDcols = private$meta_data$injection]
  ) > 0), ]
  private$set_kmd()
  private$peak_table$Compound <- as.character(private$peak_table$Compound)
})
mpactr$set("private", "set_kmd", function() {
  private$peak_table[, kmd := mz - floor(mz)]
})
mpactr$set("public", "get_peak_table", function() {
  return(private$peak_table)
})
mpactr$set("public", "set_peak_table", function(peak_table) {
  private$peak_table <- peak_table
})
mpactr$set("public", "get_meta_data", function() {
  return(private$meta_data)
})
mpactr$set("public", "get_raw_data", function() {
  return(private$raw_peak_table)
})

format_by_type <- function(peak_table_path,
                           type_of_peak_table,
                           sample_names = NULL) {
  if (!(type_of_peak_table
        %in% c("Progenesis", "MzMine", "Metaboscape", "None"))) {
    cli::cli_abort("{.var type_of_peak_table} must be one of Progenesis,
                    MzMine, Metaboscape, or none. See documentation
                    for more details.")
  }
  if (isFALSE(grepl("https://", peak_table_path)) &&
        !file.exists(peak_table_path)) {
    cli::cli_abort("Your peak_table is not a valid file path,
                    please enter a new one")
  }
  if (type_of_peak_table == "Progenesis") {
    return(progenesis_formatter(peak_table_path))
  } else if (type_of_peak_table == "MzMine") {
    return(mz_mine_formatter(peak_table_path))
  } else if (type_of_peak_table == "Metaboscape") {
    return(metaboscape_formatter(peak_table_path, sample_names))
  } else if (type_of_peak_table == "None") {
    peak_table <- data.frame()
    if (!(any(c("data.table", "data.frame") %in% class(peak_table_path)))) {
      peak_table <- fread(peak_table_path,
                          sep = ",")
    } else {
      peak_table <- peak_table_path
    }
    return(list(
      "peak_table" = peak_table,
      "raw_table" = peak_table
    ))
  } else {

  } # default condition = NULL
}

progenesis_formatter <- function(peak_table) {
  if (!(any(c("data.table", "data.frame") %in% class(peak_table)))) {
    peak_table <- fread(peak_table,
      sep = ",",
      skip = 2,
    )
  }
  raw_peak_table <- peak_table
  with(peak_table, setnames(
    peak_table,
    c("m/z", "Retention time (min)"),
    c("mz", "rt")
  ))

  return(list(
    "peak_table" = peak_table,
    "raw_table" = raw_peak_table
  ))
}


mz_mine_formatter <- function(peak_table) {
  if (!(any(c("data.table", "data.frame") %in% class(peak_table)))) {
    peak_table <- fread(peak_table,
      sep = ",",
      skip = 2,
    )
  }
  raw_peak_table <- peak_table

  return(list(
    "peak_table" = peak_table,
    "raw_table" = raw_peak_table
  ))
}

metaboscape_formatter <- function(peak_table, sample_names) {
  if (!(any(c("data.table", "data.frame") %in% class(peak_table)))) {
    peak_table <- fread(peak_table,
      sep = ","
    )
  }
  peak_table_convert <- data.table::copy(peak_table)
  peak_table_convert <- with(peak_table_convert, peak_table_convert[
    , ion := gsub(
      ".*\\[(.+)\\].*", "\\1",
      ADDUCT
    )
  ][
    , charge_string := gsub(".*\\](.+)", "\\1", ADDUCT)
  ][
    charge_string == "+", charge := 1
  ][
    charge_string == "2+", charge := 2
  ][
    charge_string == "3+", charge := 3
  ][
    utils::read.csv(system.file("extdata/ion_masses",
      "DefinedIons.csv",
      package = "mpactr"
    )),
    on = .(ion = IONS),
    nomatch = NULL
  ][
    , mz := (PEPMASS / charge) + MASS
  ])

  with(peak_table_convert, setnames(
    peak_table_convert,
    c("FEATURE_ID", "RT"),
    c("Compound", "rt")
  ))
  peak_table_mpactr <- with(peak_table_convert, peak_table_convert[
    , .SD,
    .SDcols = c(
      "Compound", "mz", "rt",
      sample_names
    )
  ])

  return(list(
    "peak_table" = peak_table_mpactr,
    "raw_table" = peak_table_convert
  ))
}

test_that("progenesis_formatter reads data correctly", {
  limit_cores()
  pt <- progenesis_formatter(test_path("exttestdata",
                                       "102623_peaktable_coculture_simple.csv"))

  expect_equal(class(pt$peak_table), c("data.table", "data.frame"))
  expect_true(all(c("data.table", "data.frame") %in% class(pt$peak_table)))
  expect_true(nrow(pt$peak_table) > 0)
})
test_that("metaboscape reads data correctly", {
  limit_cores()
  samples <- c(
    "UM1850B_ANGDT_0.25_mL_36_1_4792",
    "UM1850B_ANGDT_0.25_mL_36_1_4802",
    "UM1850B_ANGDT_0.25_mL_36_1_4815",
    "MixedMonoculture_1mg_mL total_31_1_4799",
    "MixedMonoculture_1mg_mL total_31_1_4822",
    "MixedMonoculture_31_1_4787",
    "UM1852B_Coculture_42_1_4685",
    "UM1852B_Coculture_42_1_4695",
    "UM1852B_Coculture_42_1_4709",
    "UM1852B_Coculture_42_1_4785"
  )
  peak_table_name <- "MJB_MonoVSCoculture_metaboscape_ft.csv"
  pt <- metaboscape_formatter(test_path("exttestdata",
                                        peak_table_name),
                              sample_names = samples)

  expect_equal(class(pt$peak_table), c("data.table", "data.frame"))
  expect_true(all(c("data.table", "data.frame") %in% class(pt$peak_table)))
  expect_true(nrow(pt$peak_table) > 0)
  expect_equal(colnames(pt$peak_table), c("Compound", "mz", "rt", samples))
})

test_that("format_by_type formats the data properly based on the parameters", {
  limit_cores()
  samples <- c(
    "UM1850B_ANGDT_0.25_mL_36_1_4792",
    "UM1850B_ANGDT_0.25_mL_36_1_4802",
    "UM1850B_ANGDT_0.25_mL_36_1_4815",
    "MixedMonoculture_1mg_mL total_31_1_4799",
    "MixedMonoculture_1mg_mL total_31_1_4822",
    "MixedMonoculture_31_1_4787",
    "UM1852B_Coculture_42_1_4685",
    "UM1852B_Coculture_42_1_4695",
    "UM1852B_Coculture_42_1_4709",
    "UM1852B_Coculture_42_1_4785"
  )
  progenesis_peak_table <- "102623_peaktable_coculture_simple.csv"
  metabscape_peak_table <- "MJB_MonoVSCoculture_metaboscape_ft.csv"
  progenesis_format_table <- format_by_type(test_path("exttestdata",
                                                      progenesis_peak_table),
                                            type_of_peak_table = "Progenesis")
  # mzmine_format_table <- format_by_type(test_path("../../inst/extdata",
  # "MJB_MonoVSCoculture_metaboscape_ft.csv"),
  # sample_names = samples, type_of_peak_table = "MzMine")
  metaboscape_format_table <- format_by_type(test_path("exttestdata",
                                                       metabscape_peak_table),
                                             sample_names = samples,
                                             type_of_peak_table = "Metaboscape")

  expect_true(all(c("data.table", "data.frame") %in%
                    class(progenesis_format_table$peak_table)))
  # expect_true(all(c("data.table",
  # "data.frame") %in% class(mzmine_format_table$peak_table)))
  expect_true(all(c("data.table", "data.frame") %in%
                    class(metaboscape_format_table$peak_table)))
})

test_that("Formatter errors when giving a type that
does not exist", {
            limit_cores()
            samples <- c(
              "UM1850B_ANGDT_0.25_mL_36_1_4792",
              "UM1850B_ANGDT_0.25_mL_36_1_4802",
              "UM1850B_ANGDT_0.25_mL_36_1_4815",
              "MixedMonoculture_1mg_mL total_31_1_4799",
              "MixedMonoculture_1mg_mL total_31_1_4822",
              "MixedMonoculture_31_1_4787",
              "UM1852B_Coculture_42_1_4685",
              "UM1852B_Coculture_42_1_4695",
              "UM1852B_Coculture_42_1_4709",
              "UM1852B_Coculture_42_1_4785"
            )
            progenesis_peak_table <- "102623_peaktable_coculture_simple.csv"
            expect_error(format_by_type(test_path("exttestdata",
                                                  progenesis_peak_table),
                                        type_of_peak_table = "Wrong"))

          })

test_that("The 'None' format option works as expected", {
  limit_cores()
  samples <- c(
    "UM1850B_ANGDT_0.25_mL_36_1_4792",
    "UM1850B_ANGDT_0.25_mL_36_1_4802",
    "UM1850B_ANGDT_0.25_mL_36_1_4815",
    "MixedMonoculture_1mg_mL total_31_1_4799",
    "MixedMonoculture_1mg_mL total_31_1_4822",
    "MixedMonoculture_31_1_4787",
    "UM1852B_Coculture_42_1_4685",
    "UM1852B_Coculture_42_1_4695",
    "UM1852B_Coculture_42_1_4709",
    "UM1852B_Coculture_42_1_4785"
  )
  pt <- "peak_table_post_formatted.csv"
  pt <- format_by_type(test_path("exttestdata",
                                 pt),
                       type_of_peak_table = "None")

  expect_equal(class(pt$peak_table), c("data.table", "data.frame"))
  expect_true(all(c("data.table", "data.frame") %in% class(pt$peak_table)))
  expect_true(nrow(pt$peak_table) > 0)
})

test_that("format_by_type errors when given incomplete file paths", {
  limit_cores()
  pt <- "peak_table_post_formatted.csv"
  expect_error(format_by_type(pt,
                              type_of_peak_table = "None"))
})

#' Return the peak table data.table from the mpactr object.
#'
#' @description
#' `get_peak_table()` a wrapper function to return the peak table
#' object of the given mpactr object.
#'
#' @param mpactr_object The mpactr object that is created by calling
#' the import_data() function.
#'
#' @return a `data.table`.
#' @export
#'
#' @examples
#'  \dontshow{
#' limit_cores()
#' }
#' data <- import_data(
#'   example_path("coculture_peak_table.csv"),
#'   example_path("metadata.csv"),
#'   format = "Progenesis"
#' )
#'
#' peak_table <- get_peak_table(data)
#'
get_peak_table <- function(mpactr_object) {
  return(data.table::copy(mpactr_object$mpactr_data$get_peak_table()))
}

#' Return the meta_data data.table from the mpactr object.
#'
#' @description
#' `get_metadata()` a wrapper function to return the meta data object
#' of the given mpactr object.
#'
#' @param mpactr_object The mpactr object that is created by calling
#' the import_data() function.
#'
#' @return a `data.table`.
#' @export
#'
#' @examples
#'  \dontshow{
#' limit_cores()
#' }
#' data <- import_data(
#'   example_path("coculture_peak_table.csv"),
#'   example_path("metadata.csv"),
#'   format = "Progenesis"
#' )
#'
#' meta_data <- get_metadata(data)
#'
get_metadata <- function(mpactr_object) {
  return(data.table::copy(mpactr_object$mpactr_data$get_meta_data()))
}

#' Return the input peak table from mpactr object.
#'
#' @description
#' `get_raw_data` a wrapper function to return the meta data object of the
#' given mpactr object.
#'
#' @param mpactr_object The mpactr object that is created by calling the
#' import_data() function.
#'
#' @return a `data.table`.
#' @export
#'
#' @examples
#'  \dontshow{
#' limit_cores()
#' }
#' data <- import_data(
#'   example_path("coculture_peak_table.csv"),
#'   example_path("metadata.csv"),
#'   format = "Progenesis"
#' )
#'
#' raw_data <- get_raw_data(data)
#'
get_raw_data <- function(mpactr_object) {
  return(data.table::copy(mpactr_object$mpactr_data$get_raw_data()))
}

test_that("get_peak_table correctly returns the table", {
  limit_cores()
  directory <- "exttestdata"
  peak_table_name <- "102623_peaktable_coculture_simple.csv"
  meta_data_name <- "102623_metadata_correct.csv"
  data <- import_data(test_path(directory, peak_table_name),
    test_path(directory, meta_data_name),
    format = "Progenesis"
  )

  peak_table <- get_peak_table(data)
  expect_equal(class(peak_table), c("data.table", "data.frame"))
  expect_equal(nrow(peak_table), 1303)
})

test_that("get_meta_data correctly returns the table", {
  limit_cores()
  directory <- "exttestdata"
  peak_table_name <- "102623_peaktable_coculture_simple.csv"
  meta_data_name <- "102623_metadata_correct.csv"
  data <- import_data(test_path(directory, peak_table_name),
    test_path(directory, meta_data_name),
    format = "Progenesis"
  )

  metadata <- get_metadata(data)
  expect_equal(class(metadata), c("data.table", "data.frame"))
  expect_equal(nrow(metadata), 18)
})

test_that("get_raw_data correctly returns the table", {
  limit_cores()
  directory <- "exttestdata"
  peak_table_name <- "102623_peaktable_coculture_simple.csv"
  meta_data_name <- "102623_metadata_correct.csv"
  data <- import_data(test_path(directory, peak_table_name),
    test_path(directory, meta_data_name),
    format = "Progenesis"
  )

  raw_data <- get_raw_data(data)
  expect_equal(class(raw_data), c("data.table", "data.frame"))
  expect_equal(nrow(raw_data), 1306)
})

#' Return the summary for a single mpactr filter.
#'
#' @description
#' `filter_summary()` is a wrapper function to return the summary
#' from a single filter within the given mpactr object.
#'
#' @param mpactr_object The mpactr object that is created by calling
#' the import_data() function.
#'
#' @param filter The name of a filter whose summary is to be extracted.
#' Must be one of: "mispicked", "group", "replicability", or "insource".
#' @param group If filter = "group", the name of the Biological_Group
#' used to filter.
#'
#' @return a `list` reporting 1) compound ids for compounds which failed
#' the filter and 2) compound ids for compounds which passed the filter.
#' @export
#'
#' @examples
#'  \dontshow{
#' limit_cores()
#' }
#' data <- import_data(example_path("coculture_peak_table.csv"),
#'   example_path("metadata.csv"),
#'   format = "Progenesis"
#' )
#'
#' data_filter <- filter_mispicked_ions(data)
#'
#' mispicked_summary <- filter_summary(data_filter, filter = "mispicked")
#' mispicked_summary
#'
filter_summary <- function(mpactr_object, filter, group = NULL) {
  return(mpactr_object$get_log(filter = filter, group = group))
}

#' Get similar ion groups.
#'
#' @description
#' `get_similar_ions()` is a wrapper function to return similar ion groups
#' determined with the [filter_mispicked_ions()].
#'
#' @param mpactr_object The mpactr object that is created by calling the
#' import_data() function.
#'
#' @return a `data.table` reporting the main ion and those found to be
#' similar with [filter_mispicked_ions()].
#' @export
#'
#' @examples
#'  \dontshow{
#' limit_cores()
#' }
#' data <- import_data(
#'   example_path("coculture_peak_table.csv"),
#'   example_path("metadata.csv"),
#'   format = "Progenesis"
#' )
#'
#' data_filter <- filter_mispicked_ions(data)
#'
#' mispicked_ion_groups <- get_similar_ions(data_filter)
#' mispicked_ion_groups
#'
get_similar_ions <- function(mpactr_object) {
  return(mpactr_object$get_mispicked_ions())
}

#' Get groups averages.
#'
#' @description
#' `get_group_averages()` is a wrapper function to return group averages
#' for the filtered peak table.
#'
#' @param mpactr_object The mpactr object that is created by calling the
#' import_data() function.
#'
#' @return a `data.table` reporting the average and relative standard
#' deviation across biological groups and technical replicates within
#' each group.
#' @export
#'
#' @examples
#'  \dontshow{
#' limit_cores()
#' }
#' data <- import_data(
#'   example_path("coculture_peak_table.csv"),
#'   example_path("metadata.csv"),
#'   format = "Progenesis"
#' )
#'
#' data_filter <- filter_group(data, group_to_remove = "Blanks")
#'
#' group_averages <- get_group_averages(data_filter)
#' head(group_averages)
#'
get_group_averages <- function(mpactr_object) {
  return(mpactr_object$get_group_averages())
}

#' Get CV values.
#'
#' @description
#' `get_cv_data()` is a wrapper function to return cv (coefficient of
#' variation) calculated with [filter_cv()].
#'
#' @param mpactr_object The mpactr object that is created by calling
#' the import_data() function.
#'
#' @return a `data.table` reporting the mean and median coefficient
#' of variation for each input ion.
#' @export
#'
#' @examples
#'  \dontshow{
#' limit_cores()
#' }
#' data <- import_data(
#'   example_path("coculture_peak_table.csv"),
#'   example_path("metadata.csv"),
#'   format = "Progenesis"
#' )
#'
#' data_filter <- filter_cv(data,
#'   cv_threshold = 0.01,
#' )
#'
#' cv <- get_cv_data(data_filter)
#' head(cv)
#'
get_cv_data <- function(mpactr_object) {
  return(mpactr_object$get_cv())
}

test_that("filter_summary returns an error when an incorrect filter argument
 is provided", {
            limit_cores()
            peak_table_name <- "102623_peaktable_coculture_simple.csv"
            meta_data_name <- "102623_metadata_correct.csv"
            data <- import_data(test_path("exttestdata",
                                          peak_table_name),
              test_path("exttestdata", meta_data_name),
              format = "Progenesis"
            )

            expect_error(filter_summary(data,
                                        filter = "cv"),
                         "`filter` must be one of mpactr's")
          })

test_that("filter_summary returns an error when the filter
 argument provided has not yet been run (e.g., not in the log)", {
            limit_cores()
            directory <- "exttestdata"
            peak_table_name <- "102623_peaktable_coculture_simple.csv"
            meta_data_name <- "102623_metadata_correct.csv"
            er <-  "`filter` replicability has not yet been applied to the data"
            data <- import_data(test_path(directory, peak_table_name),
              test_path(directory, meta_data_name),
              format = "Progenesis"
            )

            data_mpactr <- filter_mispicked_ions(data,
                                                 ringwin = 0.5,
                                                 isowin = 0.01,
                                                 trwin = 0.005,
                                                 max_iso_shift = 3,
                                                 merge_peaks = TRUE)

            expect_error(filter_summary(data_mpactr,
                                        filter = "replicability"),
                         er)
          })

test_that(" returns the correct filter summary list", {
  limit_cores()
  directory <- "exttestdata"
  peak_table_name <- "102623_peaktable_coculture_simple.csv"
  meta_data_name <- "102623_metadata_correct.csv"

  data <- import_data(test_path("exttestdata", peak_table_name),
    test_path("exttestdata", meta_data_name),
    format = "Progenesis"
  )

  data_mpactr <- filter_mispicked_ions(data,
                                       ringwin = 0.5,
                                       isowin = 0.01,
                                       trwin = 0.005,
                                       max_iso_shift = 3,
                                       merge_peaks = TRUE)

  mispicked_summary <- filter_summary(data_mpactr, filter = "mispicked")

  expect_type(mispicked_summary, "list")
  expect_equal(length(mispicked_summary), 2)
  expect_equal(length(mispicked_summary$passed_ions), 1233)
})

test_that("get_similar_ions returns error if 
check_mismatched_peaks has not been called", {
            limit_cores()
            directory <- "exttestdata"
            peak_table_name <- "102623_peaktable_coculture_simple.csv"
            meta_data_name <- "102623_metadata_correct.csv"

            data <- import_data(test_path(directory, peak_table_name),
              test_path(directory, meta_data_name),
              format = "Progenesis"
            )
            er <- "The mispicked filter has not yet been"
            expect_error(get_similar_ions(data), er)
          })

test_that("get_similar_ions correctly returns the
 check_mismatched_peaks list", {
            limit_cores()
            directory <- "exttestdata"
            peak_table_name <- "102623_peaktable_coculture_simple.csv"
            meta_data_name <- "102623_metadata_correct.csv"
            data <- import_data(test_path(directory, peak_table_name),
              test_path(directory, meta_data_name),
              format = "Progenesis"
            )

            data_mpactr <- filter_mispicked_ions(data,
                                                 ringwin = 0.5,
                                                 isowin = 0.01,
                                                 trwin = 0.005,
                                                 max_iso_shift = 3,
                                                 merge_peaks = TRUE)

            mispicked_groups <- get_similar_ions(data_mpactr)

            expect_equal(class(mispicked_groups), c("data.table", "data.frame"))
            expect_equal(length(mispicked_groups), 2)
            expect_equal(names(mispicked_groups), c("main_ion", "similar_ions"))
          })

test_that("get_group_averages calculates a group table", {
  limit_cores()
  directory <- "exttestdata"
  peak_table_name <- "102623_peaktable_coculture_simple.csv"
  meta_data_name <- "102623_metadata_correct.csv"
  data <- import_data(test_path(directory, peak_table_name),
    test_path(directory, meta_data_name),
    format = "Progenesis"
  )

  data_mpactr <- filter_mispicked_ions(data,
                                       ringwin = 0.5,
                                       isowin = 0.01,
                                       trwin = 0.005,
                                       max_iso_shift = 3,
                                       merge_peaks = TRUE)


  avgs <- get_group_averages(data_mpactr)
  expect_equal(class(avgs), c("data.table", "data.frame"))
  expect_equal(nrow(avgs), (1233 * 6))
})


test_that("get_cv_data returns the cv table if filter cv has been run", {
  limit_cores()
  directory <- "exttestdata"
  peak_table_name <- "102623_peaktable_coculture_simple.csv"
  meta_data_name <- "102623_metadata_correct.csv"
  data <- import_data(test_path(directory, peak_table_name),
    test_path(directory, meta_data_name),
    format = "Progenesis"
  )

  data_mpactr <- filter_mispicked_ions(data,
                                       ringwin = 0.5,
                                       isowin = 0.01,
                                       trwin = 0.005,
                                       max_iso_shift = 3,
                                       merge_peaks = TRUE) |>
    filter_group(group_to_remove = "Blanks")

  expect_error(get_cv_data(data_mpactr), "The cv filter has not yet")

  data_mpactr |>
    filter_cv(cv_threshold = 0.2)

  cv <- get_cv_data(data_mpactr)

  expect_equal(class(cv), c("data.table", "data.frame"))
})

#######################
### Mismatched peaks ##
#######################

#' Mispicked ions filter
#'
#' @description
#' `filter_mispicked_ions()` identifies ions that were incorrectly split into
#' separate features during preprocessing. This filter checks the feature table
#' for similar ions in terms of mass and retention time. Peaks found to be
#' similar are merged into a single feature given `merge_peaks` is `TRUE`.
#'
#' The parameter `ringwin` is the detector saturation mass window, specific for
#' some instruments, such as Waters Synapse G2-Si-Q-ToF, to account for high
#' concentration samples.
#'
#' Parameter `isowin` is the isotopic mass window, which accounts for isotopic
#' peaks of the same precursor mass that were incorrectly assigned during
#' preprocessing.
#'
#'
#' `copy_object`: mpactr is built on an R6 class-system, meaning it operates on
#' reference semantics in which data is updated *in-place*. Compared to a
#' shallow copy, where only data pointers are copied, or a deep copy, where
#' the entire data object is copied in memory, any changes to the original
#' data object, regardless if they are assigned to a new object, result in
#' changes to the original data object. We recommend using the default
#' `copy_object = FALSE` as this makes for an extremely fast and
#' memory-efficient way to chain mpactr filters together; however, if you
#' would like to run the filters individually with traditional R style objects,
#' you can set `copy_object` to `TRUE` as shown in the filter examples.
#'
#' @param mpactr_object An `mpactr_object`. See [import_data()].
#' @param ringwin Ringing mass window or detector saturation mass window.
#' Default = 0.5 atomic mass units (AMU).
#' @param isowin Isotopic mass window. Default = 0.01 AMU.
#' @param trwin A `numeric` denoting the retention time threshold for assessing
#' if ions should be merged. Default = 0.005.
#' @param max_iso_shift A `numeric`. Default = 3.
#' @param merge_peaks A `boolean` parameter to determine if peaks found to
#' belong to the same ion should be merged in the feature table.
#' @param merge_method If merge_peaks is TRUE, a method for how similar peaks
#' should be merged. Can be one of "sum".
#' @param copy_object A `boolean` parameter that allows users to return a copied
#' object instead of modifying the object.
#' @return an `mpactr_object`.
#' @export
#'
#' @examples
#'  \dontshow{
#' limit_cores()
#' }
#' data <- import_data(
#'   example_path("coculture_peak_table.csv"),
#'   example_path("metadata.csv"),
#'   format = "Progenesis"
#' )
#'
#' data_filter <- filter_mispicked_ions(data,
#'   ringwin = 0.5,
#'   isowin = 0.01,
#'   trwin = 0.005,
#'   max_iso_shift = 3,
#'   merge_peaks = TRUE,
#'   merge_method = "sum"
#' )
#'
filter_mispicked_ions <- function(mpactr_object,
                                  ringwin = 0.5,
                                  isowin = 0.01,
                                  trwin = 0.005,
                                  max_iso_shift = 3,
                                  merge_peaks = TRUE,
                                  merge_method = "sum",
                                  copy_object = FALSE) {

  if (isTRUE(mpactr_object$is_filter_run(filter = "mispicked"))) {

    cli::cli_abort("{.var filter_mispicked_ions()} has already been run on
                   {deparse(substitute(mpactr_object))}. Reload the data
                   and set {.var copy_object} to {.var TRUE} before
                   running the filter if you would like to run more than once.
                   See more by typing {.var ?? filter_mispicked_ions} in your
                   terminal.")
  }

  if (copy_object) {
    mpactr_object <- clone(mpactr_object)
  }
  mpactr_object$check_mismatched_peaks(
    ringwin = ringwin,
    isowin = isowin,
    trwin = trwin,
    max_iso_shift = max_iso_shift,
    merge_peaks = merge_peaks, merge_method = "sum"
  )
  return(mpactr_object)
}

#######################
### Group filter     ##
#######################
#' Filter Ions by Group
#'
#' @details
#' `filter_group()` removes feature ions that are present in a user-defined
#' group based on a relative abundance threshold. This could be particularly
#' useful to filter out features found present in solvent blank samples.
#' Further, this filter can be utilized to remove features in media blank
#' sample for experiments on microbial cultures.

#' The presence or absence of features in a group of samples is determined by
#' first averaging injection replicates and then averaging biological
#' replicates within each biological treatment group. A feature is present in
#' a group if its abundance is greater than the user-defined `group_threshold`.
#' The default is 0.01, meaning a feature is removed if its abundance is 1% of
#' that in the sample group in which it is most abundant. For example, blank
#' filtering can remove features whose mean abundance in solvent blank
#' injections is greater than 1% of their maximum mean abundance in experimental
#' samples.
#'
#' If you would like to remove features found in media blank
#' samples, we recommend testing the `group_threshold` parameter.
#'
#' `copy_object`: mpactr is built on an R6 class-system, meaning it operates on
#' reference semantics in which data is updated *in-place*. Compared to a
#' shallow copy, where only data pointers are copied, or a deep copy, where
#' the entire data object is copied in memory, any changes to the original
#' data object, regardless if they are assigned to a new object, result in
#' changes to the original data object. We recommend using the default
#' `copy_object = FALSE` as this makes for an extremely fast and
#' memory-efficient way to chain mpactr filters together; however, if you
#' would like to run the filters individually with traditional R style objects,
#' you can set `copy_object` to `TRUE` as shown in the filter examples.
#'
#'
#' @param mpactr_object An `mpactr_object`. See [import_data()].
#' @param group_threshold Relative abundance threshold at which to remove ions.
#' Default = 0.01.
#' @param group_to_remove Biological group name to remove ions from.
#' @param remove_ions A `boolean` parameter. If `TRUE` failing ions will be
#' removed from the peak table. Default = TRUE.
#' @param copy_object A `boolean` parameter that allows users to return a copied
#' object instead of modifying the object.

#' @return an `mpactr_object`.
#' @export
#'
#' @examples
#'  \dontshow{
#' limit_cores()
#' }
#' data <- import_data(
#'   example_path("coculture_peak_table.csv"),
#'   example_path("metadata.csv"),
#'   format = "Progenesis"
#' )
#'
#' data_filter <- filter_group(data,
#'   group_threshold = 0.01,
#'   group_to_remove = "Blanks",
#'   remove_ions = TRUE
#' )
#'
filter_group <- function(mpactr_object,
                         group_threshold = 0.01,
                         group_to_remove,
                         remove_ions = TRUE,
                         copy_object = FALSE) {
  if (isTRUE(mpactr_object$is_filter_run(filter = "group",
                                         group = group_to_remove))) {

    cli::cli_abort("{.var filter_group()} has already been run for group
                   {group_to_remove} on {deparse(substitute(mpactr_object))}.
                   Reload the data and set {.var copy_object} to {.var TRUE}
                   before running the filter if you would like to run more
                   than once. See more by typing {.var ?? filter_group} in 
                   your terminal.")
  }

  if (copy_object) {
    mpactr_object <- clone(mpactr_object)
  }
  mpactr_object$filter_blank()
  mpactr_object$parse_ions_by_group(group_threshold = group_threshold)
  mpactr_object$apply_group_filter(
    group = group_to_remove,
    remove_ions = remove_ions
  )
  return(mpactr_object)
}

#######################
### CV filter        ##
#######################
#' Filter Non-reproducible ions
#'
#' @description
#' `filter_cv()` removes feature ions that are found to be non-reproducible
#' between technical injection replicates. Reproducibility is assessed via mean
#' coefficient of variation (CV) between technical replicates. As
#' such, this filter is expecting an input dataset with at least two replicate
#' injections per sample.
#'
#' `copy_object`: mpactr is built on an R6 class-system, meaning it operates on
#' reference semantics in which data is updated *in-place*. Compared to a
#' shallow copy, where only data pointers are copied, or a deep copy, where
#' the entire data object is copied in memory, any changes to the original
#' data object, regardless if they are assigned to a new object, result in
#' changes to the original data object. We recommend using the default
#' `copy_object = FALSE` as this makes for an extremely fast and
#' memory-efficient way to chain mpactr filters together; however, if you
#' would like to run the filters individually with traditional R style objects,
#' you can set `copy_object` to `TRUE` as shown in the filter examples.
#'
#' @param mpactr_object An `mpactr_object`. See [import_data()].
#' @param cv_threshold Coefficient of variation threshold.
#' A lower cv_threshold will result in more stringent filtering and higher
#' reproducibility. Recommended values between 0.2 - 0.5.
#' @param copy_object A `boolean` parameter that allows users to return a copied
#' object instead of modifying the object.
#'
#' @return an `mpactr_object`.
#' @export
#'
#' @examples
#'  \dontshow{
#' limit_cores()
#' }
#' data <- import_data(
#'   example_path("coculture_peak_table.csv"),
#'   example_path("metadata.csv"),
#'   format = "Progenesis"
#' )
#'
#' data_filter <- filter_cv(data,
#'   cv_threshold = 0.01,
#'   copy_object = TRUE
#' )
#'
filter_cv <- function(mpactr_object,
                      cv_threshold = NULL,
                      copy_object = FALSE) {

  if (isTRUE(mpactr_object$is_filter_run(filter = "replicability"))) {

    cli::cli_abort("{.var filter_cv()} has already been run on
                   {deparse(substitute(mpactr_object))}. Reload the data
                   and set {.var copy_object} to {.var TRUE} before
                   running the filter if you would like to run more than once.
                   See more by typing {.var ?? filter_cv} in your terminal.")
  }

  if (copy_object) {
    mpactr_object <- clone(mpactr_object)
  }

  mpactr_object$cv_filter(
    cv_threshold = cv_threshold
  )
  return(mpactr_object)
}

###########################
### Insource ions filter ##
###########################
#' Filter Insource ions
#'
#' @description
#' `filter_insource_ions()` identifies and removes in-source ion clusters based
#' on a Pearson correlation threshold. Groups of co-eluting features with
#' identical retention time are identified and used to generate Pearson
#' correlation matrices. Clusters with self-similarity greater than the
#' user-defined `cluster_threshold` within these matrices are identified as
#' likely belonging to a single precursor ion and is associated insource ion.
#' Highly correlated ions are identified and removed.
#'
#' `copy_object`: mpactr is built on an R6 class-system, meaning it operates on
#' reference semantics in which data is updated *in-place*. Compared to a
#' shallow copy, where only data pointers are copied, or a deep copy, where
#' the entire data object is copied in memory, any changes to the original
#' data object, regardless if they are assigned to a new object, result in
#' changes to the original data object. We recommend using the default
#' `copy_object = FALSE` as this makes for an extremely fast and
#' memory-efficient way to chain mpactr filters together; however, if you
#' would like to run the filters individually with traditional R style objects,
#' you can set `copy_object` to `TRUE` as shown in the filter examples.
#'
#' @param mpactr_object An `mpactr_object`. See [import_data()].
#' @param cluster_threshold Cluster threshold for ion deconvolution.
#' Default = 0.95.
#' @param copy_object A `boolean` parameter that allows users to return
#' a copied object instead of modifying the object.
#'
#' @return an `mpactr_object`
#' @export
#'
#' @examples
#'  \dontshow{
#' limit_cores()
#' }
#' data <- import_data(
#'   example_path("coculture_peak_table.csv"),
#'   example_path("metadata.csv"),
#'   format = "Progenesis"
#' )
#'
#' data_filter <- filter_insource_ions(data,
#'   cluster_threshold = 0.95
#' )
#'
filter_insource_ions <- function(mpactr_object,
                                 cluster_threshold = 0.95,
                                 copy_object = FALSE) {

  if (isTRUE(mpactr_object$is_filter_run(filter = "insource"))) {

    cli::cli_abort("{.var filter_insource_ions()} has already been run on
                   {deparse(substitute(mpactr_object))}. Reload the data
                   and set {.var copy_object} to {.var TRUE} before
                   running the filter if you would like to run more than once.
                   See more by typing {.var ?? filter_insource_ions} in your
                   terminal.")
  }

  if (copy_object) {
    mpactr_object <- clone(mpactr_object)
  }
  mpactr_object$filter_insource_ions(cluster_threshold = cluster_threshold)
  return(mpactr_object)
}

test_that("import_data creates a proper mpactr and filter-pactr object", {
  limit_cores()
  directory <- "exttestdata"
  peak_table_name <- "102623_peaktable_coculture_simple.csv"
  meta_data_name <- "102623_metadata_correct.csv"

  data <- import_data(
    peak_table = test_path(directory, peak_table_name),
    meta_data = test_path(directory,  meta_data_name),
    format = "Progenesis"
  )

  expect_true(all(class(data) == c("filter_pactr", "R6")))
  expect_true(exists("list_of_summaries", data$logger))
  expect_true(all(sapply(data$logger[["list_of_summaries"]], is.null) == TRUE))
  expect_true(nrow(data$mpactr_data$get_peak_table()) > 1)
  expect_true(nrow(data$mpactr_data$get_meta_data()) > 1)
})

test_that("We can use a data.frame as input our peak table in import_data", {
  limit_cores()
  directory <- "exttestdata"
  peak_table_name <- "102623_peaktable_coculture_simple.csv"
  meta_data_name <- "102623_metadata_correct.csv"
  meta_data_path <- test_path(directory,  meta_data_name)
  data <- import_data(
    peak_table = test_path(directory, peak_table_name),
    meta_data = meta_data_path,
    format = "Progenesis"
  )
  peak_table <- get_peak_table(data)

  data_df <- import_data(get_peak_table(data), get_metadata(data), "None")
  peak_table_df <- get_peak_table(data_df)

  expect_true(all(peak_table == peak_table_df))

  data_df_meta_data_file <- import_data(get_peak_table(data),
                                        meta_data_path, "None")
  peak_table_meta_data_file <- get_peak_table(data_df_meta_data_file)
  expect_true(all(peak_table == peak_table_meta_data_file))

  # Even when filtered

  data <- import_data(
    peak_table = test_path(directory, peak_table_name),
    meta_data = meta_data_path,
    format = "Progenesis"
  ) |>
    filter_mispicked_ions() |>
    filter_cv(0.1) |>
    filter_group(0.1, "Blanks") |>
    filter_insource_ions()

  data_df <- import_data(get_peak_table(data), get_metadata(data), "None")
  peak_table_df <- get_peak_table(data_df)
  df <- get_peak_table(data)
  temp <- data.frame(Compound = peak_table_df$Compound,
                     mz = peak_table_df$mz, kmd = peak_table_df$kmd,
                     rt = peak_table_df$rt)
  dat <- cbind(temp, peak_table_df[, 5:22])
  expect_true(all(dat == as.data.frame(df)[, 1:22]))
})

test_that("We can use a data.frame as input our peak table in import_data
          for all formats", {
            limit_cores()
            directory <- "exttestdata"
            peak_table_name <- "102623_peaktable_coculture_simple.csv"
            meta_data_name <- "102623_metadata_correct.csv"
            meta_data_path <- test_path(directory,  meta_data_name)
            peak_table <- fread(test_path(directory,
                                          peak_table_name), skip = 2)
            data <- import_data(
              peak_table = peak_table,
              meta_data = meta_data_path,
              format = "Progenesis"
            )
            formatted_peak_table <- get_raw_data(data)
            expect_true(ncol(formatted_peak_table) == ncol(peak_table))
            expect_true(nrow(formatted_peak_table) == nrow(peak_table))
            metabscape_peak_table <- "MJB_MonoVSCoculture_metaboscape_ft.csv"
            peak_table <-
              fread(test_path(directory,
                              metabscape_peak_table))
            samples <- colnames(peak_table)[27:72]
            meta_data <-
              data.frame(Injection = samples,
                         Biological_Group = rep("Blank",
                                                times = length(samples)),
                         Sample_Code = rep("Blank", times = length(samples)))

            data <- import_data(
              peak_table = peak_table,
              meta_data = meta_data,
              format = "Metaboscape"
            )
            formatted_peak_table <- get_raw_data(data)
            expect_true(nrow(formatted_peak_table) == nrow(peak_table))
          })


test_that("import_data aborts when expected
 metadata columns are not provided", {
            limit_cores()
            directory <- "exttestdata"
            peak_table_name <- "102623_peaktable_coculture_simple.csv"
            meta_data_name <- "102623_metadata_correct.csv"
            meta_data_abort <- fread(test_path(directory,  meta_data_name))
            colnames(meta_data_abort) <- c("Injection",
                                           "Sample", "Biological_Group")

            expect_error(import_data(
              peak_table = test_path(directory, peak_table_name),
              meta_data = meta_data_abort,
              format = "Progenesis"
            ))
          })



test_that("unique_compounds annotate duplicates properly", {
  limit_cores()
  df <- data.frame(Compound = c(1, 2, 3, 1:3, 4:7))
  ls <- list(peak_table = df, raw_table = df)
  uniqued_list <- unique_compounds(ls)

  expect_true(length(unique(uniqued_list$peak_table$Compound)) ==
                length(uniqued_list$peak_table$Compound))
  expect_true(length(unique(uniqued_list$raw_table$Compound)) ==
                length(uniqued_list$raw_table$Compound))

  expect_false(length(unique(df$Compound)) == length(df$Compound))

  df <- data.frame(Compound = c("1", "1", "1_1", "1_1_1"))
  ls <- list(peak_table = df, raw_table = df)
  uniqued_list <- unique_compounds(list(peak_table = df, raw_table = df))

  expect_true(uniqued_list$peak_table$Compound[[3]] == "1_1_1")
  expect_true(uniqued_list$peak_table$Compound[[4]] == "1_1_1_1")

  df <- data.frame(Compound = c("1", "1", "1_1", "1_1_1"))
  ls <- list(peak_table = df, raw_table = df)
  expect_message(unique_compounds(ls))

  df <- data.frame(Compound = c("1", "1", NA, "1_1_1"))
  ls <- list(peak_table = df, raw_table = df)
  expect_error(unique_compounds(ls))
})

test_that("filter mismatch ions wrapper works
 as expected when merge_peaks is TRUE", {
            limit_cores()
            directory <- "exttestdata"
            peak_table_name <- "102623_peaktable_coculture_simple.csv"
            meta_data_name <- "102623_metadata_correct.csv"
            data <- import_data(test_path(directory, peak_table_name),
              test_path(directory, meta_data_name),
              format = "Progenesis"
            )

            data_mpactr_copy <- filter_mispicked_ions(data,
              ringwin = 0.5, isowin = 0.01, trwin = 0.005, max_iso_shift = 3,
              merge_peaks = TRUE, merge_method = "sum", copy_object = TRUE
            )

            data_mpactr <- filter_mispicked_ions(data,
              ringwin = 0.5, isowin = 0.01, trwin = 0.005, max_iso_shift = 3,
              merge_peaks = TRUE, merge_method = "sum", copy_object = FALSE
            )

            expected_cut_ions <- fread(test_path(directory,
                                                 "cut_ions.csv"),
                                       col.names = c("V1", "V2"))
            expected_cut_ions <- as.character(expected_cut_ions$V1)

            expect_equal(nrow(data_mpactr$mpactr_data$get_peak_table()),
                         nrow((data_mpactr$mpactr_data$get_peak_table())))
            log_mm <- "check_mismatched_peaks"
            expect_equal(data_mpactr$logger[[log_mm]][["cut_ions"]],
                         expected_cut_ions)
            expect_equal(data_mpactr_copy$logger[[log_mm]][["cut_ions"]],
                         expected_cut_ions)

            expect_equal(nrow(data_mpactr$mpactr_data$get_peak_table()), 1233)

            dat <- import_data(peak_table = get_peak_table(data_mpactr_copy),
                               meta_data = get_metadata(data_mpactr_copy),
                               format = "None")
            filtered_data <- dat |>
              filter_mispicked_ions(merge_peaks = TRUE,
                                    merge_method = "sum",
                                    copy_object = TRUE)
            expect_equal(nrow(get_peak_table(dat)),
                         nrow(get_peak_table(filtered_data)))
          })

test_that("filter mismatch ions wrapper works 
as expected when merge_peaks is FALSE", {
            limit_cores()
            directory <- "exttestdata"
            peak_table_name <- "102623_peaktable_coculture_simple.csv"
            meta_data_name <- "102623_metadata_correct.csv"
            data <- import_data(test_path(directory, peak_table_name),
              test_path(directory, meta_data_name),
              format = "Progenesis"
            )

            data_mpactr_copy <- filter_mispicked_ions(data,
              ringwin = 0.5, isowin = 0.01, trwin = 0.005, max_iso_shift = 3,
              merge_peaks = FALSE, merge_method = "sum", copy_object = TRUE
            )


            data_mpactr <- filter_mispicked_ions(data,
              ringwin = 0.5, isowin = 0.01, trwin = 0.005, max_iso_shift = 3,
              merge_peaks = FALSE
            )

            expect_equal(nrow(data_mpactr$mpactr_data$get_peak_table()), 1303)
            expect_equal(nrow(data_mpactr_copy$mpactr_data$get_peak_table()),
                         1303)
          })

test_that("group filter wrapper works as expected", {
  limit_cores()
  directory <- "exttestdata"
  peak_table_name <- "102623_peaktable_coculture_simple.csv"
  meta_data_name <- "102623_metadata_correct.csv"
  data <- import_data(test_path(directory, peak_table_name),
    test_path(directory, meta_data_name),
    format = "Progenesis"
  )
  data_mpactr <- filter_mispicked_ions(data,
    ringwin = 0.5,
    isowin = 0.01,
    trwin = 0.005,
    max_iso_shift = 3,
    merge_peaks = TRUE,
    merge_method = "sum"
  )
  data_mpactr <- filter_group(data_mpactr, 0.01, "Blanks", FALSE)
  expect_equal(nrow(data_mpactr$mpactr_data$get_peak_table()), 1233)

  data_mpactr_copy <- filter_group(data_mpactr, 0.01, "Blanks", TRUE,
    copy_object = TRUE
  )

  data_mpactr <- filter_group(data_mpactr, 0.01, "Blanks", TRUE)
  expect_equal(nrow(data_mpactr$mpactr_data$get_peak_table()), 484)

  expect_equal(
    nrow(data_mpactr$mpactr_data$get_peak_table()),
    nrow(data_mpactr_copy$mpactr_data$get_peak_table())
  )

  log_name <- "group_filter-failing_list"
  expect_true(all(!(data_mpactr$logger[[log_name]]$Blanks
                    %in% data_mpactr$mpactr_data$get_peak_table()$Compound)))

  expect_true(all(!(data_mpactr_copy$logger[[log_name]]$Blanks %in%
                      data_mpactr_copy$mpactr_data$get_peak_table()$Compound)))
})

test_that("filter cv filter wrapper works as expected", {
  limit_cores()
  directory <- "exttestdata"
  peak_table_name <- "102623_peaktable_coculture_simple.csv"
  meta_data_name <- "102623_metadata_correct.csv"
  data <- import_data(test_path(directory, peak_table_name),
    test_path(directory, meta_data_name),
    format = "Progenesis"
  )

  data_mpactr <- filter_mispicked_ions(data,
    ringwin = 0.5,
    isowin = 0.01,
    trwin = 0.005,
    max_iso_shift = 3,
    merge_peaks = TRUE,
    merge_method = "sum"
  )
  data_mpactr <- filter_group(data_mpactr, 0.01, "Blanks", TRUE)
  data_mpactr_copy <- filter_cv(data_mpactr, 0.2, copy_object = TRUE)
  data_mpactr <- filter_cv(data_mpactr, 0.2)


  expect_equal(length(data_mpactr$logger[["list_of_summaries"]]$
                        replicability$get_failed_ions()), 33)
  expect_equal(length(data_mpactr_copy$logger[["list_of_summaries"]]$
                        replicability$get_failed_ions()), 33)

  meta <- get_metadata(data)[c(1, 4), ]
  df <- get_raw_data(data)
  df <- cbind(df[, 1:3], df[, meta$injection, with = FALSE])

  data_no_replicates <- import_data(peak_table = df,
                                    meta_data = meta,
                                    format = "None")
  expect_error(filter_cv(data_no_replicates, cv_threshold = 0.1),
               "There are no technical replicates")
})

test_that("filter insource ions wrapper works as expected", {
  limit_cores()
  directory <- "exttestdata"
  peak_table_name <- "102623_peaktable_coculture_simple.csv"
  meta_data_name <- "102623_metadata_correct.csv"
  data <- import_data(test_path(directory, peak_table_name),
    test_path(directory, meta_data_name),
    format = "Progenesis"
  )

  data_mpactr <- filter_mispicked_ions(data,
    ringwin = 0.5,
    isowin = 0.01,
    trwin = 0.005,
    max_iso_shift = 3,
    merge_peaks = TRUE,
    merge_method = "sum"
  )
  data_mpactr <- filter_group(data_mpactr, 0.01, "Blanks", TRUE)

  data_mpactr_copy <- filter_insource_ions(data_mpactr,
                                           cluster_threshold = 0.95,
                                           copy_object = TRUE)

  data_mpactr <- filter_insource_ions(data_mpactr, cluster_threshold = 0.95)

  insource_ion_expected_list <- c(
    38, 204, 214, 993, 270, 1003, 271, 294, 331, 349, 382,
    447, 498, 1233, 644, 1307, 677, 675, 689,
    690, 688, 758, 985, 982, 981, 1297, 1311
  )

  expect_true(length(data_mpactr$logger[["list_of_summaries"]]$
                       insource$get_failed_ions()) == 27)
  expect_true(length(data_mpactr_copy$logger[["list_of_summaries"]]$
                       insource$get_failed_ions()) == 27)

  expect_true(all(!(insource_ion_expected_list %in%
                      data_mpactr$mpactr_data$get_peak_table()$Compound)))
  expect_true(all(!(insource_ion_expected_list %in% data_mpactr_copy$
                      mpactr_data$get_peak_table()$Compound)))
})

test_that("filters abort if the filter has already been run", {
  limit_cores()
  directory <- "exttestdata"
  peak_table_name <- "102623_peaktable_coculture_simple.csv"
  meta_data_name <- "102623_metadata_correct.csv"
  data <- import_data(test_path(directory, peak_table_name),
    test_path(directory, meta_data_name),
    format = "Progenesis"
  )

  data_mpactr <- filter_mispicked_ions(data,
    ringwin = 0.5,
    isowin = 0.01,
    trwin = 0.005,
    max_iso_shift = 3,
    merge_peaks = TRUE,
    merge_method = "sum"
  )
  data_mpactr <- filter_group(data_mpactr, 0.01, "Blanks", TRUE)
  data_mpactr <- filter_cv(data_mpactr, cv_threshold = 0.2)
  data_mpactr <- filter_insource_ions(data_mpactr, cluster_threshold = 0.95)

  expect_error(filter_mispicked_ions(data_mpactr,
                                     ringwin = 0.5,
                                     isowin = 0.01,
                                     trwin = 0.005,
                                     max_iso_shift = 3,
                                     merge_peaks = TRUE,
                                     merge_method = "sum"))
  expect_error(filter_group(data_mpactr, 0.01, "Blanks", TRUE))
  expect_error(filter_cv(data_mpactr, cv_threshold = 0.2))
  expect_error(filter_insource_ions(data_mpactr, cluster_threshold = 0.95))
})

test_that("qc_summary returns a summary data.table", {
  limit_cores()
  data <- import_data(test_path("exttestdata",
                                "102623_peaktable_coculture_simple.csv"),
    test_path("exttestdata", "102623_metadata_correct.csv"),
    format = "Progenesis"
  )

  data_mpactr <- filter_mispicked_ions(data,
    ringwin = 0.5,
    isowin = 0.01,
    trwin = 0.005,
    max_iso_shift = 3,
    merge_peaks = TRUE
  )
  data_mpactr <- filter_group(data, 0.01, "Blanks", TRUE)
  data_mpactr <- filter_group(data, 0.01, "Coculture", TRUE)
  data_mpactr <- filter_insource_ions(data, cluster_threshold = 0.95)

  data_mpactr_summary <- qc_summary(data)
  expect_true(nrow(data_mpactr_summary) > 1)
  expect_equal(unique(data_mpactr_summary$status), c(
    "Passed", "mispicked", "group-Blanks",
    "group-Coculture", "insource"
  ))
})

test_that("qc plot returns a generates a plot", {
  limit_cores()
  data <- import_data(test_path("exttestdata",
                                "102623_peaktable_coculture_simple.csv"),
    test_path("exttestdata", "102623_metadata_correct.csv"),
    format = "Progenesis"
  )

  data_mpactr <- filter_mispicked_ions(data,
    ringwin = 0.5,
    isowin = 0.01,
    trwin = 0.005,
    max_iso_shift = 3,
    merge_peaks = TRUE
  )
  data_mpactr <- filter_group(data, 0.01, "Blanks", TRUE)
  data_mpactr <- filter_insource_ions(data, cluster_threshold = 0.95)
  plot <- plot_qc_tree(data)
  expect_true(inherits(plot, c("ggplot", "ggplot2::ggplot")))
})

test_that("The clone function properly deep clones the mpactr object", {
  limit_cores()
  directory <- "exttestdata"
  peak_table_name <- "102623_peaktable_coculture_simple.csv"
  meta_data_name <- "102623_metadata_correct.csv"
  mpactr_object <- import_data(test_path(directory, peak_table_name),
    test_path(directory, meta_data_name),
    format = "Progenesis"
  )

  mpactr_object <- filter_mispicked_ions(mpactr_object,
    ringwin = 0.5, isowin = 0.01, trwin = 0.005, max_iso_shift = 3,
    merge_peaks = TRUE
  )

  mpactr_object_clone <- clone(mpactr_object)

  mpactr_object_clone <- filter_group(mpactr_object_clone,
                                      0.01, "Coculture", TRUE)
  expect_false(nrow(get_peak_table(mpactr_object)) ==
                 nrow(get_peak_table(mpactr_object_clone)))

  identical(mpactr_object, mpactr_object_clone)
  expect_false(address(mpactr_object) == address(mpactr_object_clone))
  expect_false(address(mpactr_object$mpactr_data) ==
                 address(mpactr_object_clone$mpactr_data))
  expect_false(address(mpactr_object$logger) ==
                 address(mpactr_object_clone$logger))
  address(mpactr_object$mpactr_data$get_peak_table()) ==
    address(mpactr_object_clone$mpactr_data$get_peak_table())
})

---
title: "Downstream Analyses"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Downstream Analyses}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

The goal of mpactr is to correct for errors that occur during the pre-processing of raw tandem MS/MS data. This data needs to be filtered prior to downstream analyses because of limitations in mass spectrometry detection capabilities. Once filtering is complete, you should have a feature table containing high quality MS1 features. This table can be used for a variety of analyses that can be conducted in R to better understand if and how samples differ from one another. 

In this article, we will highlight just a few analyses. This is not an exhaustive list, and there are certainly other ways to conduct the analyses shown below - the beauty of R! We will walk you through how to do the following:

- creating an interactive plot of input features and the filters they failed, if any, using `ggplot` and `plotly`
- correlating sample profiles at multiple levels (sample, technical replicates, and biological replicates) using `Hmisc` and `corrplot`
- visualizing sample clustering as a dendrogram with `ggdendro`
- differential abundance analysis:
    * calculating fold change between two biological groups
    * visualizing fold change, m/z, and retention time as a 3D scatter plot
    * conducting t-tests for fold change differences, calculating p-values, and calculating q-values for false discovery rate (FDR) correction
    * visualizing significant fold change differences in a volcano plot


# Load required libraries

We will be using multiple libraries for data analysis and visualization, which can be loaded below. If you do not have some or all of the packages listed, you can install them with `install.packages("packagename")`.

```{r setup, message=FALSE}
library(mpactr)
library(viridis)
library(plotly)
library(data.table)
library(tidyverse)
library(Hmisc)
library(corrplot)
library(ggdendro)
library(ggtext)
```

```{r, include = FALSE}
limit_cores()
```

# Filter MS1 feature table with `mpactr`

```{r}
data <- import_data(example_path("cultures_peak_table.csv"),
  example_path("cultures_metadata.csv"),
  format = "Progenesis"
)
```

We will be using example data in the `mpactr` package, which has `r nrow(get_metadata(data))` samples and `r length(unique(get_metadata(data)$biological_group))` biological groups. Biological groups are: `r unique(get_metadata(data)$biological_group)`.

We will check our feature table for mispicked ions, remove solvent blank and media blanks features with a relative ion abundance > 0.01, relative to other groups, and check ions for replicability.

```{r}
data_filtered <- data |>
  filter_mispicked_ions(merge_peaks = TRUE, merge_method = "sum") |>
  filter_group(group_to_remove = "Solvent_Blank") |>
  filter_group(group_to_remove = "Media") |>
  filter_cv(cv_threshold = 0.2)
```

Overall, `r nrow(get_peak_table(data_filtered))` ions remain in the feature table. A summary of the filtering, as a tree map is below:

```{r}
plot_qc_tree(data_filtered)
```

# Interactive scatter plot of all input ions and their fate during filtering

Visualizing each compound by m/z and retention time, and their fate during filtering may be useful to see if filters are removing features at certain retention time or m/z ranges. To create this plot, we first need to extract the raw data table, which has all pre-filtered ions (including m/z and retention time). We can do this with the mpactr function `get_raw_data()`, and then select the Compound, mz, and rt columns.

```{r}
get_raw_data(data_filtered) %>%
  select(Compound, mz, rt) %>%
  head()
```

Next, we need to extract the ion status with `mpactr` function `qc_summary()`. This function returns a `data.table` reporting the ion status for each input ion. This includes which filter each ion failed or passed, or if the ion passed all applied filters.

```{r}
qc_summary(data_filtered) %>%
  head()
```

We can join these two `data.table`s for plotting data set:

```{r}
get_raw_data(data_filtered) %>%
  mutate(Compound = as.character(Compound)) %>%
  select(Compound, mz, rt) %>%
  left_join(qc_summary(data_filtered),
    by = join_by("Compound" == "compounds")
  ) %>%
  head()
```

Now we can create a scatter plot to show the input features (m/z ~ retention time) and their fate (status) using `ggolot` and `geom_point`.

```{r}
get_raw_data(data_filtered) %>%
  mutate(Compound = as.character(Compound)) %>%
  select(Compound, mz, rt) %>%
  left_join(qc_summary(data_filtered),
    by = join_by("Compound" == "compounds")
  ) %>%
  ggplot() +
  aes(x = rt, y = mz, color = status) +
  geom_point() +
  viridis::scale_color_viridis(discrete = TRUE) +
  labs(
    x = "Retention time (min)",
    y = "m/z",
    color = "Ion Status"
  ) +
  theme_bw() +
  theme(legend.position = "top")
```

We can also make the plot interactive with the `plotly` package function `ggplotly`.

```{r}
feature_plot <- get_raw_data(data_filtered) %>%
  mutate(Compound = as.character(Compound)) %>%
  select(Compound, mz, rt) %>%
  left_join(qc_summary(data_filtered),
    by = join_by("Compound" == "compounds")
  ) %>%
  ggplot() +
  aes(
    x = rt, y = mz, color = status,
    text = paste0("Compound: ", Compound)
  ) +
  geom_point() +
  viridis::scale_color_viridis(discrete = TRUE) +
  labs(
    x = "Retention time (min)",
    y = "m/z",
    color = "Ion Status"
  ) +
  theme_bw() +
  theme(legend.position = "top")

ggplotly(feature_plot)
```


# Correlation between samples at different levels

We may be interested in how individual samples, or biological groups correlate to each other. For this analysis, we will need to extract the filtered feature table with `mpactr` function `get_peak_table()`. This function will return the filtered `data.table` where rows are features and columns are either compound metadata or intensity values for samples.

```{r}
ft <- get_peak_table(data_filtered)

ft[1:5, 1:7]
```

### Correlation between each individual sample

First, we can look at the correlation between each individual sample, regardless of technical replicates and biological groups. This will show how well technical and biological replicates correlate, where we are expecting to see near perfect correlation replicates.

We will use `Hmisc::rcorr()` to perform spearman correlations. This function is expecting a `matrix` where columns are samples to correlate and rows are features. Since the feature table has columns for both feature metadata and samples, we need to remove feature metadata for `rcorr()`. We can do this by extracting sample names from the metadata `injection` column, which match sample column names in the feature table. We also need to remove any samples that no longer have any features post-filtering (likely solvent blanks).

```{r}
counts <- ft %>%
  select(Compound, all_of(get_metadata(data_filtered)$injection)) %>%
  column_to_rownames(var = "Compound") %>%
  select(where(~ sum(.x) != 0))

counts[1:5, 1:2]
```

Next we can run the correlation:

```{r}
counts_cor <- rcorr(as.matrix(counts), type = "spearman")
```

The `counts_cor` object is a list of data for the correlations. The correlation coefficients are stored in the `r` slot, and p-values in the `p` slot. We can see the correlations for first sample below

```{r}
counts_cor$r[, 1]
```

Finally, we can plot the correlations map with `corrplot`. Style options can be customized to your liking (see [corrplot](https://CRAN.R-project.org/package=corrplot/vignettes/corrplot-intro.html)).

```{r, warning=FALSE}
corrplot(counts_cor$r,
  type = "lower",
  method = "square",
  order = "hclust",
  col = COL2("BrBG", 10),
  tl.col = "black",
  tl.cex = .5
)
```

### Correlation between technical replicates

Next, we can look at the correlation between technical replicates to see how they compare across the dataset. For this, we will calculate the average intensity for each compound across technical replicates and run a correlation in the same manner shown above.

In the original `counts` table, we set the row names from the column `Compound`, which holds the compound id. Here we need to reset our `Compound` column and pivot the data table for calculating averages.

```{r}
meta <- get_metadata(data_filtered)

counts %>%
  rownames_to_column(var = "Compound") %>%
  pivot_longer(
    cols = starts_with("102623"),
    names_to = "injection",
    values_to = "intensity"
  ) %>%
  head()
```

Next, we will join with sample meta data so we can calculate averages by `sample_code`.

```{r}
counts %>%
  rownames_to_column(var = "Compound") %>%
  pivot_longer(
    cols = starts_with("102623"),
    names_to = "injection",
    values_to = "intensity"
  ) %>%
  left_join(meta, by = "injection") %>%
  head()
```

Now we can calculate mean intensity for each `Compound` and `sample_code`.

```{r}
counts %>%
  rownames_to_column(var = "Compound") %>%
  pivot_longer(
    cols = starts_with("102623"),
    names_to = "injection",
    values_to = "intensity"
  ) %>%
  left_join(meta, by = "injection") %>%
  summarise(
    mean_intensity = mean(intensity),
    .by = c(Compound, sample_code)
  ) %>%
  head()
```

Finally, we need to reformat the table so columns are sample codes for the correlation.

```{r, warning=FALSE}
#| warning: false

sample_counts <- counts %>%
  rownames_to_column(var = "Compound") %>%
  pivot_longer(
    cols = starts_with("102623"),
    names_to = "injection",
    values_to = "intensity"
  ) %>%
  left_join(meta, by = "injection") %>%
  summarise(
    mean_intensity = mean(intensity),
    .by = c(Compound, sample_code)
  ) %>%
  pivot_wider(
    names_from = sample_code,
    values_from = mean_intensity
  ) %>%
  column_to_rownames(var = "Compound")

sample_counts[1:5, 1:5]
```

Run the correlation: 

```{r}
sample_counts_cor <- rcorr(as.matrix(sample_counts), type = "spearman")
```

And finally visualize:

```{r}
corrplot(sample_counts_cor$r,
  type = "lower",
  method = "square",
  order = "alphabet",
  col = COL2("BrBG", 10),
  tl.col = "black"
)
```


### Correlation between biological replicates

We can also look at the correlation between biological groups. In this dataset, technical replicates and biological groups are the same; however if you have multiple technical replicates you may also be interested in the correlation between biological groups. We will calculate mean intensity values for `biological_group` in the same manner as we did for `sample_code`.

```{r, warning=FALSE}
group_counts <- counts %>%
  rownames_to_column(var = "Compound") %>%
  pivot_longer(
    cols = starts_with("102623"),
    names_to = "injection",
    values_to = "intensity"
  ) %>%
  left_join(meta, by = "injection") %>%
  summarise(
    mean_intensity = mean(intensity),
    .by = c(Compound, biological_group)
  ) %>%
  pivot_wider(
    names_from = biological_group,
    values_from = mean_intensity
  ) %>%
  column_to_rownames(var = "Compound")
```    
  
  
Run the correlation analysis with `rcorr()`:
    
```{r}
group_counts_cor <- rcorr(as.matrix(group_counts), type = "spearman")
```

Visualize correlation:

```{r}
corrplot(group_counts_cor$r,
  type = "lower",
  method = "square",
  order = "alphabet",
  col = COL2("BrBG", 10),
  tl.col = "black"
)
```

As expected, the biological group correlation matrix matches the technical replicates correlation.

# Dendrogram

You may be interested in visualizing how similar samples are with a dendrogram. For this we will need to calculate the distance between samples with `stats::dist()`, cluster with `hclust()`, and visualize with `ggplot` and `ggdendro` packages.

First, calculate distance and cluster:

```{r}
dist <- stats::dist(t(counts), method = "euclidian")
cluster <- stats::hclust(dist, method = "complete")
```

Calculate dendrogram components with `stats::as.dendrogram()`:

```{r}
dendro <- as.dendrogram(cluster)
```

Extract plotting elements with `ggdendrogram::dendro_data()`

```{r}
den_data <- dendro_data(dendro, type = "rectangle")
```

Use `ggplot` and `ggdendrogram` to plot the dendrogram. For more customizations, see [ggdendro documentation](https://CRAN.R-project.org/package=ggdendro/vignettes/ggdendro.html).

```{r}
ggplot(segment(den_data)) +
  geom_segment(aes(x = x, y = y, xend = xend, yend = yend)) +
  geom_text(
    data = den_data$labels,
    aes(x = x, y = y, label = label),
    size = 3,
    hjust = "outward"
  ) +
  coord_cartesian(ylim = c((min(segment(den_data)$y) + 10),
                           (max(segment(den_data)$y)))) +
  coord_flip() +
  scale_y_reverse(expand = c(.5, 0)) +
  theme_dendro()
```

# Fold change analysis

Say we wanted to compare metabolite profiles between our Coculture and ANG18 groups. We can extract group means from the filtered compounds using the mpactr function `get_group_averages()`, isolate the groups of interest, here Coculture and ANG18 monocoluture, and calculate compound fold change. 

### Calculate fold change

```{r}
get_group_averages(data_filtered) %>%
  filter(biological_group == "Coculture" |
           biological_group == "ANG18") %>%
  select(Compound, biological_group, average) %>%
  pivot_wider(names_from = biological_group, values_from = average) %>%
  mutate(fc = Coculture / ANG18) %>%
  head()
```

Inherently, tandem MS/MS datasets can be filled with many zeros. In some instances a compound is found in the experimental group but not in the control group, or vice versa. In these cases, the fold change calculation yields an infinite number as there is a zero either in the numerator or denominator ($fold change = experimental / control$). There is also the chance that the compound is not found in either group, yielding a fold change on NaN (0/0). 

Compounds that are not in either group are of no interest in this comparison and can therefore be removed from the analysis. 

```{r}
get_group_averages(data_filtered) %>%
  filter(biological_group == "Coculture" |
           biological_group == "ANG18") %>%
  select(Compound, biological_group, average) %>%
  pivot_wider(names_from = biological_group, values_from = average) %>%
  mutate(nonzero_compound = if_else(Coculture == 0 & ANG18 == 0,
                                    FALSE,
                                    TRUE)) %>%
  filter(nonzero_compound == TRUE) %>%
  mutate(fc = Coculture / ANG18) %>%
  head()
```

There are multiple ways to handle compounds that are found in the one group but not the other. Here we will create pseudo-counts by adding a small number (0.001) to all counts prior to calculating fold change. This will alleviate division by 0. 

First, extract group averages from the `mpactr` object and select the two groups of interest. Here we are interested in the columns Coculture and ANG18:

```{r}
get_group_averages(data_filtered) %>%
  filter(biological_group == "Coculture" |
           biological_group == "ANG18") %>%
  select(Compound, biological_group, average) %>%
  head()
```

To calculate fold change, we need to have two columns: one for the control group, here ANG18, and one for the experimental group, Coculture:

```{r}
get_group_averages(data_filtered) %>%
  filter(biological_group == "Coculture" |
           biological_group == "ANG18") %>%
  select(Compound, biological_group, average) %>%
  head()
```


Now we create pseudo-counts by adding 0.001 to the counts column:

```{r}
get_group_averages(data_filtered) %>%
  filter(biological_group == "Coculture" |
           biological_group == "ANG18") %>%
  select(Compound, biological_group, average) %>%
  mutate(average = average + 0.001) %>%
  pivot_wider(names_from = biological_group, values_from = average) %>%
  head()
```


Now we can remove compounds with an intensity of 0 in both groups. This equates to a pseudo-count of 0.001:

```{r}
get_group_averages(data_filtered) %>%
  filter(biological_group == "Coculture" |
           biological_group == "ANG18") %>%
  select(Compound, biological_group, average) %>%
  mutate(average = average + 0.001) %>%
  pivot_wider(names_from = biological_group, values_from = average) %>%
  mutate(nonzero_compound = if_else(Coculture == 0.001 & ANG18 == 0.001,
                                    FALSE,
                                    TRUE)) %>%
  filter(nonzero_compound == TRUE) %>%
  head()
```

Next, we calculate fold change for all remaining compounds:

```{r}
get_group_averages(data_filtered) %>%
  filter(biological_group == "Coculture" |
           biological_group == "ANG18") %>%
  select(Compound, biological_group, average) %>%
  mutate(average = average + 0.001) %>%
  pivot_wider(names_from = biological_group, values_from = average) %>%
  mutate(nonzero_compound = if_else(Coculture == 0.001 & ANG18 == 0.001,
                                    FALSE,
                                    TRUE)) %>%
  filter(nonzero_compound == TRUE) %>%
  mutate(fc = Coculture / ANG18) %>%
  head()
```

Finally, transform fold change to log2:

```{r}
foldchanges <- get_group_averages(data_filtered) %>%
  filter(biological_group == "Coculture" |
           biological_group == "ANG18") %>%
  select(Compound, biological_group, average) %>%
  mutate(average = average + 0.001) %>%
  pivot_wider(names_from = biological_group, values_from = average) %>%
  mutate(nonzero_compound = if_else(Coculture == 0.001 & ANG18 == 0.001,
                                    FALSE,
                                    TRUE)) %>%
  filter(nonzero_compound == TRUE) %>%
  mutate(fc = Coculture / ANG18,
         logfc = log2(fc))

head(foldchanges)
```

### Visualize fold change for compounds in a 3D scatter plot

We can then probe compounds by plotting a 3D scatter plot of log2 fold changes as a function of m/z and retention time:

```{r}
fc_plotting <- foldchanges %>%
  left_join(select(ft, Compound, mz, rt), by = "Compound")

plot_ly(fc_plotting,
  x = ~logfc, y = ~rt, z = ~mz,
  marker = list(color = ~logfc, showscale = TRUE)
) %>%
  add_markers() %>%
  layout(
    scene = list(
      xaxis = list(title = "log2 Fold Change"),
      yaxis = list(title = "Retention Time (min)"),
      zaxis = list(title = "m/z")
    ),
    annotations = list(
      x = 1.13,
      y = 1.05,
      text = "log2 Fold Change",
      xref = "paper",
      yref = "paper",
      showarrow = FALSE
    )
  )
```

### Test for compounds with significant fold changes

For the t-tests, we have already calculated variation for compounds within biological groups as well as within technical replicates in biological groups. We can calculate the t statistic and degrees of freedom and use the t distribution (`pt()`) to calculate the p-value.

*combine biological and technical variation and calculate sample size*

```{r}
# Satterwaite
calc_samplesize_ws <- function(sd1, n1, sd2, n2) {
  s1 <- sd1 / (n1^0.5)
  s2 <- sd2 / (n2^0.5)

  n <- (s1^2 / n1 + s2^2 / n2)^2
  d1 <- s1^4 / ((n1^2) * (n1 - 1))
  d2 <- s2^4 / ((n2^2) * (n2 - 1))

  d1[which(!is.finite(d1))] <- 0
  d2[which(!is.finite(d2))] <- 0

  d <- d1 + d2

  return(n / d)
}

my_comp <- c("Coculture", "ANG18")

stats <- get_group_averages(data_filtered) %>%
  mutate(
    combRSD = sqrt(techRSD^2 + BiolRSD^2),
    combASD = combRSD * average,
    combASD = if_else(is.na(combASD), 0, combASD),
    BiolRSD = if_else(is.na(BiolRSD), 0, BiolRSD),
    techRSD = if_else(is.na(techRSD), 0, techRSD),
    neff = calc_samplesize_ws(techRSD, techn, BiolRSD, Bioln) + 1
  ) %>%
  filter(biological_group %in% my_comp)

head(stats)
```

*calculate the t statistic*

```{r}
denom <- stats %>%
  summarise(den = combASD^2 / (neff),
            .by = c("Compound", "biological_group")) %>%
  mutate(den = if_else(!is.finite(den), 0, den)) %>%
  summarise(denom = sqrt(sum(den)), .by = c("Compound"))

t_test <- stats %>%
  select(Compound, biological_group, average) %>%
  pivot_wider(names_from = biological_group, values_from = average) %>%
  mutate(numerator = (Coculture - ANG18)) %>% # experimental - control
  left_join(denom, by = "Compound") %>%
  mutate(t = abs(numerator / denom))

head(t_test)
```

*calculate degrees of freedom*

```{r}
df <- stats %>%
  select(Compound, biological_group, neff) %>%
  mutate(neff = if_else(!is.finite(neff), 0, neff)) %>%
  pivot_wider(names_from = biological_group, values_from = neff) %>%
  mutate(deg = Coculture + ANG18 - 2) %>%
  select(Compound, deg)

head(df)
```

*calculate p-value*

```{r} 
t <- t_test %>%
  left_join(df, by = "Compound") %>%
  mutate(
    p = (1 - pt(t, deg)) * 2,
    logp = log10(p),
    neg_logp = -logp
  ) %>%
  select(Compound, t, deg, p, logp, neg_logp)

head(t)
```

*combine t-test results with fold changes we calculated*

```{r}
num_ions <- t %>%
  filter(p <= 1) %>%
  count() %>%
  pull()

fc <- foldchanges %>%
  left_join(t, by = "Compound") %>%
  arrange(p) %>%
  mutate(
    qval = seq_len(length(p)),
    qval = p * num_ions / qval
  ) %>%
  arrange(desc(p))

min_qval <- 1

for (i in seq_len(nrow(fc))) {
  if (!is.finite(fc$qval[i])) {
    next
  }

  if (fc$qval[i] < min_qval) {
    min_qval <- fc$qval[i]
  } else {
    fc$qval[i] <- min_qval
  }
}

fc2 <- fc %>%
  mutate(neg_logq = -log10(qval))
```

### Volcano plot

Now we can create a volcano plot to see the magnitude of metabolite abundance differences between the Cocoulture and ANG18 groups.

We can plot log2 fold change values against log~10~ p-values or log~10~ q-values, both of which we calculated in our t-tests above.

*log2 fold change ~ log~10~ p-values*

The base of a volcano plot is a scatter plot with log2 fold changes on the x axis and -log~10~ p-values on the y axis:

```{r}
fc2 %>%
  ggplot() +
  aes(x = logfc, y = neg_logp) +
  geom_point() +
  labs(x = "log2 Fold Change",
       y = "-Log~10~ *P*",
       color = "Differential Abundance") +
  theme_bw()
```

This is good, but we probably want to know which compounds are above the p-value threshold and outside the log2 fold change threshold. Fold change threshold commonly ranges from 1.5 - 2. For this plot, we will show a p-value threshold of 0.05 and fold change threshold of 1.5. 

First add a horizontal line to denote the cutoff of -log~10~ 0.05:

```{r}
fc2 %>%
  ggplot() +
  aes(x = logfc, y = neg_logp) +
  geom_point() +
  geom_hline(yintercept = -log10(0.05), linetype = "dashed") +
  labs(x = "log2 Fold Change",
       y = "-Log~10~ *P*",
       color = "Differential Abundance") +
  theme_bw()
```

Now we can add vertical lines showing the positive and negative cutoffs for log2 fold change:

```{r}
fc2 %>%
  ggplot() +
  aes(x = logfc, y = neg_logp) +
  geom_point() +
  geom_hline(yintercept = -log10(0.05), linetype = "dashed") +
  labs(x = "log2 Fold Change",
       y = "-Log~10~ *P*",
       color = "Differential Abundance") +
  geom_vline(xintercept = log2(1.5), linetype = "dashed") +
  geom_vline(xintercept = -log2(1.5), linetype = "dashed") +
  theme_bw()
```

So now we can see the thresholds, but it's difficult to make out which compounds have significant fold change values. It would be nice to color the compounds by their significance. Specifically, compounds whose intensity are significantly higher in the experimental group, compounds whose intensity are significantly lower in the experimental group, compounds with significant p-values but the fold change is below the fold change threshold, and compounds whose fold change are not significant.

We can do this by adding a new variable to our dataset and using this variable to color our points.

First, add a new variable `sig` which has values `Increased` (fold change > 1.5 & pvalue < 0.05), `Decreased` (fold change < -1.5 & pvalue < 0.05), `Inconclusive` (fold change -1.5 - 1.5 & pvalue < 0.05), and `not_sig` (pvalue > 0.05).

```{r}
fc2 %>%
  mutate(
    sig = case_when(
      p > 0.05 ~ "not_sig",
      p <= 0.05 & logfc > log2(1.5) ~ "Increased",
      p <= 0.05 & logfc < -log2(1.5) ~ "Decreased",
      TRUE ~ "Inconclusive"
    ),
    sig = factor(sig,
      levels = c("Increased", "Decreased", "Inconclusive", "not_sig"),
      labels = c("Increased", "Decreased", "Inconclusive", "Not significant")
    )
  ) %>%
  select(Compound, ANG18, Coculture, fc, logfc, p, sig) %>%
  head()
```

Now we can set the color aesthetic to the variable `sig` and select custom colors with `scale_color_manual()`:

```{r}
fc2 %>%
  mutate(
    sig = case_when(
      p > 0.05 ~ "not_sig",
      p <= 0.05 & logfc > log2(1.5) ~ "Increased",
      p <= 0.05 & logfc < -log2(1.5) ~ "Decreased",
      TRUE ~ "Inconclusive"
    ),
    sig = factor(sig,
      levels = c("Increased", "Decreased", "Inconclusive", "not_sig"),
      labels = c("Increased", "Decreased", "Inconclusive", "Not significant")
    )
  ) %>%
  ggplot() +
  aes(x = logfc, y = neg_logp, color = sig) +
  geom_point() +
  geom_hline(yintercept = -log10(0.05), linetype = "dashed") +
  geom_vline(xintercept = log2(1.5), linetype = "dashed") +
  geom_vline(xintercept = -log2(1.5), linetype = "dashed") +
  scale_color_manual(values = c("red", "blue", "grey", "black")) +
  labs(
    x = "log2 Fold Change",
    y = "-Log~10~ *P*",
    color = "Differential Abundance"
  ) +
  theme_bw()
```

Finally, adjust axis text and titles to your liking:

```{r}
fc2 %>%
  mutate(
    sig = case_when(
      p > 0.05 ~ "not_sig",
      p <= 0.05 & logfc > log2(1.5) ~ "Increased",
      p <= 0.05 & logfc < -log2(1.5) ~ "Decreased",
      TRUE ~ "Inconclusive"
    ),
    sig = factor(sig,
      levels = c("Increased", "Decreased", "Inconclusive", "not_sig"),
      labels = c("Increased", "Decreased", "Inconclusive", "Not significant")
    )
  ) %>%
  ggplot() +
  aes(x = logfc, y = neg_logp, color = sig) +
  geom_point() +
  geom_hline(yintercept = -log10(0.05), linetype = "dashed") +
  geom_vline(xintercept = log2(1.5), linetype = "dashed") +
  geom_vline(xintercept = -log2(1.5), linetype = "dashed") +
  scale_color_manual(values = c("red", "blue", "grey", "black")) +
  labs(
    x = "log2 Fold Change",
    y = "-Log~10~ *P*",
    color = "Differential Abundance"
  ) +
  theme_bw() +
  theme(
    axis.title = element_markdown(size = 20),
    axis.text = element_text(size = 15)
  )
```

We can also make the plot interactive, showing the compound id for each point:

```{r}
volcano <- fc2 %>%
  mutate(
    sig = case_when(
      p > 0.05 ~ "not_sig",
      p <= 0.05 & logfc > log2(1.5) ~ "Increased",
      p <= 0.05 & logfc < -log2(1.5) ~ "Decreased",
      TRUE ~ "Inconclusive"
    ),
    sig = factor(sig,
      levels = c("Increased", "Decreased", "Inconclusive", "not_sig"),
      labels = c("Increased", "Decreased", "Inconclusive", "Not significant")
    )
  ) %>%
  ggplot() +
  aes(
    x = logfc, y = neg_logp, color = sig,
    text = paste0("Compound: ", Compound)
  ) +
  geom_point() +
  geom_hline(yintercept = -log10(0.05), linetype = "dashed") +
  geom_vline(xintercept = log2(1.5), linetype = "dashed") +
  geom_vline(xintercept = -log2(1.5), linetype = "dashed") +
  scale_color_manual(values = c("red", "blue", "grey", "black")) +
  labs(
    x = "log2 Fold Change",
    y = "-Log~10~ *P*",
    color = "Differential Abundance"
  ) +
  theme_bw() +
  theme(
    axis.title = element_markdown(size = 20),
    axis.text = element_text(size = 15)
  )

ggplotly(volcano)
```

*log2 fold change ~ log~10~ q-values*

You can also look at the volcano plot with the q value to control for false discovery rate (FDR) using the same approach shown for p values:

```{r}
fc2 %>%
  mutate(
    sig = case_when(
      qval > 0.05 ~ "not_sig",
      qval <= 0.05 & logfc > log2(1.5) ~ "Increased",
      qval <= 0.05 & logfc < -log2(1.5) ~ "Decreased",
      TRUE ~ "Inconclusive"
    ),
    sig = factor(sig,
      levels = c("Increased", "Decreased", "Inconclusive", "not_sig"),
      labels = c("Increased", "Decreased", "Inconclusive", "Not significant")
    )
  ) %>%
  ggplot() +
  aes(x = logfc, y = neg_logq, color = sig) +
  geom_point() +
  geom_hline(yintercept = -log10(0.05), linetype = "dashed") +
  geom_vline(xintercept = log2(1.5), linetype = "dashed") +
  geom_vline(xintercept = -log2(1.5), linetype = "dashed") +
  scale_color_manual(values = c("red", "blue", "grey", "black")) +
  labs(
    x = "log2 Fold Change",
    y = "-Log~10~ q-value",
    color = "Differential Abundance"
  ) +
  theme_bw() +
  theme(
    axis.title = element_markdown(size = 20),
    axis.text = element_text(size = 15)
  )
```

---
title: "Filter with mpact original data"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{mpactr}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

```r
# library(mpactr)
library(mpactr)
library(tidyverse)
library(data.table)
```

```{r setup, echo = FALSE, message=FALSE}
library(mpactr)
library(tidyverse)
library(data.table)
```

```{r, include = FALSE}
limit_cores()
```

## Load data into R

mpactr requires 2 files as input: a feature table and metadata file. Both are expected to be comma separated files (*.csv*).

1. peak_table: a peak table in Progenesis format is expected. To export a compatible peak table in Progenesis, navigate to the *Review compounds* tab then File -> Export compound Measurements. Select the following properties: compound, m/z, Retention time (min), and Raw abundance and click ok. 
2. metadata: a table with sample information. At minimum the following columns are expected: Injection, Sample_Code, Biological_Group. Injection is the sample name and is expected to match sample column names in the peak_table. Sample_Code is the id for technical replicate groups. Biological_Group is the id for biological replicate groups. Other sample metadata can be added, and is encouraged for downstream analysis following filtering with mpactr.

To import these data into R, use the mpactr function `import_data()`, which has the arguments: `peak_table_file_path` and `metadata_file_path`. This tutorial will show you examples with data from the original mpact program, found on [GitHub](https://github.com/BalunasLab/mpact/tree/main/rawdata/PTY087I2). This dataset contain 38 samples for biological groups solvent blanks, media blanks, Streptomyces sp. PTY08712 grown with (250um_Ce) and without (0um_Ce) rare earth element cerium. For more information about the experiments conducted see [Samples, Puckett, and Balunas 2023](https://pubs.acs.org/doi/abs/10.1021/acs.analchem.2c04632). 

The original program has sample metadata split across two files: the sample list and metadata. mpactr accepts a single file, so we need to combine these in one prior to import with `import_data()`.

Loading the sample list... 

```{r}
samplelist <- fread(example_path("PTY087I2_samplelist.csv"))
```


Loading the metadata list...

```{r}
metadata <- fread(example_path("PTY087I2_extractmetadata.csv"))
```


Our sample list contains additional blank samples that are not in the feature table, and therefore should be removed prior to import.

```{r}
samples <- fread(example_path("PTY087I2_dataset.csv"), skip = 2) %>%
  colnames() %>%
  str_subset(., "200826")

metadata <- samplelist %>%
  left_join(metadata, by = "Sample_Code") %>%
  filter(Injection %in% samples)
```

Now we can import the data. We will provide the url for the `peak_table`, and our reformatted metadata object. This peak table was exported from Progenesis, so we will set the `format` argument to Progenesis.

```{r}
data <- import_data(peak_table = example_path("PTY087I2_dataset.csv"),
  metadata = metadata,
  format = "Progenesis"
)
```

This will create an R6 class object, which will store both the peak table and metadata.

Calling the new mpactr object will print the current peak table in the terminal:

```{r}
data
```

## Accessing data in mpactr object

You can extract the peak table or metadata at any point with `get_raw_data()`, `get_peak_table()` and `get_metadata()` functions. Both functions will return a `data.table` object with the corresponding information.

### Extract peak table

The raw peak table is the unfiltered peak table used as input to mpactr. To extract the raw input peak table, use the function `get_raw_data()`. 

```{r}
get_raw_data(data)[1:5, 1:8]
```

The raw peak table will not change as filters are applied to the data. If you want to extract the filtered peak table, with filters that have been applied, use `get_peak_table()`:

```{r}
get_peak_table(data)[1:5, 1:8]
```

### Extract metadata

Metadata can be accessed with `get_metadata()`:

```{r}
get_metadata(data)[1:5, ]
```

## Reference semantics

mpactr is built on an R6 class-system, meaning it operates on reference semantics in which data is updated *in-place*. Compared to a shallow copy, where only data pointers are copied, or a deep copy, where the entire data object is copied in memory, any changes to the original data object, regardless if they are assigned to a new object, result in changes to the original data object. We can see this below.

Where the raw data object has `r nrow(get_peak_table(data))` ions in the feature table:

```{r}
data2 <- import_data(peak_table = example_path("PTY087I2_dataset.csv"),
  metadata = metadata,
  format = "Progenesis"
)

get_peak_table(data2)[, 1:5]
```

We can run the `filter_mispicked_ions` filter, with default setting `copy_object = FALSE` (operates on reference semantics).

```{r}
data2_mispicked <- filter_mispicked_ions(data2,
  ringwin = 0.5,
  isowin = 0.01, trwin = 0.005,
  max_iso_shift = 3, merge_peaks = TRUE,
  merge_method = "sum",
  copy_object = FALSE
)

get_peak_table(data2_mispicked)[, 1:5]
```

This results in `r nrow(get_peak_table(data2_mispicked))` ions in the feature table (above). Even though we created an object called `data2_mispicked`, the original `data2` object was also updated and now has `r nrow(get_peak_table(data2))` ions in the feature table:

```{r}
get_peak_table(data2)[, 1:5]
```

We recommend using the default `copy_object = FALSE` as this makes for an extremely fast and memory-efficient way to chain mpactr filters together (see **Chaining filters together** section and [Reference Semantics](reference_semantics.html)); however, if you would like to run the filters individually with traditional R style objects, you can set `copy_object` to `TRUE` as shown in the filter examples. 

## Filtering

mpactr provides filters to correct for the following issues observed during preprocessing of tandem MS/MS data: 

- mispicked ions: isotopic patterns that are incorrectly split during preprocessing.
- solvent blank contamination: removal of features present in solvent blanks due to carryover between samples.
- background components: features whose abundance is greater than user-defined abundance threshold in a specific group of samples, for example media blanks.
- non-reproducible ions: those that are inconsistent between technical replicates.
- insource ions: fragment ions created during ionization before fragmentation in the tandem MS/MS workflow.

#### Mispicked ions filter

To check for mispicked ions, use mpactr function `filter_mispicked_ions()`. This function takes an `mpactr object` as input, and checks for similar ions with the arguments `ringwin`, `isowin`, `trwin` and `max_iso_shift`. 

Ions in the feature table are flagged as similar based on retention time and mass. Flagged ion groups are suggested to be the result of incorrect splitting of isotopic patterns during peak picking, detector saturation artifacts, or incorrect identification of multiply charged oligomers.

```{r}
data_mispicked <- filter_mispicked_ions(data,
  ringwin = 0.5,
  isowin = 0.01, trwin = 0.005,
  max_iso_shift = 3, merge_peaks = TRUE,
  merge_method = "sum",
  copy_object = TRUE
)
```

Each filter reports progress of filtering, here we can see that `r length(filter_summary(data_mispicked, "mispicked")$failed_ions) + length(filter_summary(data_mispicked, "mispicked")$passed_ions)` ions were present prior to checking for mispicked ions. `r length(filter_summary(data_mispicked, "mispicked")$failed_ions)` ions were found to be similar to another ion and following merging, `r (length(filter_summary(data_mispicked, "mispicked")$failed_ions) + length(filter_summary(data_mispicked, "mispicked")$passed_ions)) - length(filter_summary(data_mispicked, "mispicked")$failed_ions)` ions remain.

If you are interested in the groups of similar ions flagged in this filter, you can use `get_similar_ions()`. This function returns a `data.table` reporting the main ion (the ion retained post-merging) and the ions similar to it.

```{r}
head(get_similar_ions(data_mispicked))
```

#### Remove ions that are above a threshold in one biological group

Removing solvent blank impurities is important for correcting for between-sample carryover and contamination in experimental samples. You can identify and remove these ions with mpactr's `filter_group()` function. `filter_group()` identifies ions above a relative abundance threshold (`group_threshold`) in a specific group (`group_to_remove`). To remove solvent blank impurities, set `group_to_remove` to the `Biological_Group` in your metadata file which corresponds to your solvent blank samples, here "Blanks". 

```{r}
data_blank <- filter_group(data,
  group_threshold = 0.01,
  group_to_remove = "Blanks", remove_ions = TRUE,
  copy_object = TRUE
)
```

In this example, `r length(filter_summary(data_blank, "group", "Blanks")$failed_ions) + length(filter_summary(data_blank, "group", "Blanks")$passed_ions)` ions were present prior to the group filter. `r length(filter_summary(data_blank, "group", "Blanks")$failed_ions)` ions were found to be above the relative abundance threshold of 0.01 in "Solvent_Blank" samples, leaving `r (length(filter_summary(data_blank, "group", "Blanks")$failed_ions) + length(filter_summary(data_blank, "group", "Blanks")$passed_ions)) - length(filter_summary(data_blank, "group", "Blanks")$failed_ions)`  ions in the peak table.

We can also use this filter to remove ions from other groups, such as media blanks. This can be useful for experiments on cell cultures. The example data contains samples belonging to the `Biological_Group` "Media". These samples are from media blanks, which are negative controls from the growth experiments conducted in this study. We can remove features whose abundance is greater than 1% of the largest group in media blank samples by specifying `group_to_remove` = "Media". We recommend removing media blank ions following all other filters so all high-quality ions are identified (see Chaining filters together below).

```{r}
data_media_blank <- filter_group(data,
  group_threshold = 0.01,
  group_to_remove = "Media", remove_ions = TRUE,
  copy_object = TRUE
)
```

#### Remove non-reproducible ions

Ions whose abundance are not consistent between technical replicates (*i.e.*, non-reproducible) may not be reliable for analysis and therefore should be removed from the feature table. Non-reproducible ions are identified by coefficient of variation (CV) between technical replicates with `filter_cv()`. Note - this filter cannot be applied to data that does not contain technical replicates. 

```{r}
data_rep <- filter_cv(data,
  cv_threshold = 0.2,
  copy_object = TRUE
)
```

In our example dataset, `r length(filter_summary(data_rep, "replicability")$failed_ions)` ions were flagged as non-reproducible. These ions were removed, leaving `r (length(filter_summary(data_rep, "replicability")$failed_ions) + length(filter_summary(data_rep, "replicability")$passed_ions)) - length(filter_summary(data_rep, "replicability")$failed_ions)` ions in the feature table.

If you would like to visualize how the CV threshold performed on your dataset, you can extract the CV calculated during `filer_cv()` using mpactr's `get_cv_data()` function, and calculate the percentage of features for plotting. You can look at both mean and median CV as shown in the example below, or you can filter the data by the parameter of choice. 

```{r}
cv <- get_cv_data(data_rep) %>%
  pivot_longer(cols = c("cv"),
               names_to = "param",
               values_to = "cv") %>%
  nest(.by = param) %>%
  mutate(
    data = map(data, arrange, cv),
    data = map(data, mutate, index = 0:(length(cv) - 1)),
    data = map(data, mutate, index_scale = index * 100 / length(cv))
  )

head(cv)
```

The nested data are tibbles with the columns `r colnames(cv$data[[1]])`:

```{r, echo=FALSE}
head(cv$data[[1]])
```

There is one tibble for each parameter. We also want to calculate the percentage of features represented by the CV threshold.

```{r}
cv_thresh_percent <- cv %>%
  filter(param == "cv") %>%
  unnest(cols = data) %>%
  mutate(diff_cv_thresh = abs(cv - 0.2)) %>%
  slice_min(diff_cv_thresh, n = 1) %>%
  pull(index_scale)

cv_thresh_percent
```


Then we can plot percentage of features by CV:

```{r}
cv %>%
  unnest(cols = data) %>%
  mutate(param = factor(param, levels = c("cv"),
                        labels = c("cv"))) %>%
  ggplot() +
  aes(x = cv, y = index_scale, group = param, color = param) +
  geom_line(linewidth = 2) +
  geom_vline(xintercept = 0.2,
             color = "darkgrey",
             linetype = "dashed",
             linewidth = 1) +
  geom_hline(yintercept = cv_thresh_percent,
             color = "darkgrey",
             linewidth = 1) +
  labs(x = "CV",
       y = "Percentage of Features",
       param = "Statistic") +
  theme_bw() +
  theme(
    axis.title = element_text(size = 20),
    axis.text = element_text(size = 15),
    legend.position = "inside",
    legend.position.inside = c(.90, .08),
    legend.title = element_blank(),
    legend.text = element_text(size = 15)
  )
```

Here we can see that roughly `r round(cv_thresh_percent, 0)`% of features were below the CV threshold meaning `r 100 - round(cv_thresh_percent, 0)`% were removed at a CV threshold of 0.2.

#### Remove insource fragment ions

Some mass species can be fragmented during ionization in tandem MS/MS, creating insource ions. This can result in ions from one compound being represented more than once in the feature table. If you would like to remove insource ions fragments, you can do so with mpactr's `filter_insource_ions()`. `filter_insource_ions()` conducts ion deconvolution via retention time correlation matrices within MS1 scans. Highly correlated ion groups are determined by the `cluster_threshold` parameter and filtered to remove the low mass features. The highest mass feature is identified as the likely precursor ion and retained in the feature table. 

```{r}
data_insource <- filter_insource_ions(data,
  cluster_threshold = 0.95,
  copy_object = TRUE
)
```

`r length(filter_summary(data_insource, "insource")$failed_ions)` ions were identified and removed during deconvolution of this dataset, leaving `r (length(filter_summary(data_insource, "insource")$failed_ions) + length(filter_summary(data_insource, "insource")$passed_ions)) - length(filter_summary(data_insource, "insource")$failed_ions)` ions in the feature table. 

#### Chaining filters together

Filters can be chained in a customizable workflow, shown below. While filters can be chained in any order, we recommend filtering mispicked ions, then solvent blanks, prior to filtering non-reproducible or insource ions. This will allow for incorrectly picked peaks to be merged and any contamination/carryover removed prior to identifying non-reproducible and insource fragment ions. Here we also demonstrate the removal of media blank components with the `filter_group()` function after identification of high-quality ions.

```{r}
data <- import_data(peak_table = example_path("PTY087I2_dataset.csv"),
  metadata = metadata,
  format = "Progenesis"
)

data_filtered <- filter_mispicked_ions(data, merge_method = "sum") |>
  filter_group(group_to_remove = "Blanks") |>
  filter_cv(cv_threshold = 0.2) |>
  filter_group(group_to_remove = "Media")
```

## Summary

mpactr offers multiple ways to view a summary of data filtering. 

#### View passing and failed ions for a single filter

If you are interested in viewing the passing and failing ions for a single filter, use the `filter_summary()` function.  You must specify which filter you are interested in, either "mispicked", "group", "replicability", or "insource".

```{r}
mispicked_summary <- filter_summary(data_filtered, filter = "mispicked")
```

Failed ions: 
```{r}
head(mispicked_summary$failed_ions, 100)
```

Passing ions:
```{r}
head(mispicked_summary$passed_ions, 100)
```

If you set `filter` to a filter name that you did not apply to your data, an error message will be returned.

```{r, error=TRUE}
filter_summary(data_filtered, filter = "insource")
```

If you want to retrieve the filter summary for the group filter, you must also supply the group name with the `group` argument:

```{r, eval = FALSE}
filter_summary(data_filtered, filter = "group", group = "Blanks")
```

`group` must always be supplied for this filter, even if only one group filter has been applied.

#### View passing and failed ions for all input ions

You can view the filtering status of all input ions with the `qc_summary()` function. A data.table reporting the compound id (`compounds`) and if it failed or passed filtering. If the ion failed filtering, its status will report the name of the filter it failed.

```{r}
head(qc_summary(data_filtered)[order(compounds), ])
```

#### Visualize filtering QC with tree map plot

You can visualize filtering results with a tree map using the filtering summary obtained from `qc_summary()` and the packages `ggplot2` and `treemapify`.

First, we need to determine the number of ions for each ion status in the summary table. You can report the count; however, we need to calculate the percent of ions in each group. We have done this in the code chunk below, where we have used `data.table` syntax as the `qc_summary()` returns a `data.table` object. If you are not familiar with the package data.table, check out their resources on [gitlab](https://rdatatable.gitlab.io/data.table/).

```{r}
library(ggplot2)
library(treemapify)

ion_counts <- qc_summary(data_filtered)[, .(count = .N), by = status][
  , percent := (count / sum(count) * 100)
]
```

Finally, we plot the treemap:

```{r}
tm <- ggplot(ion_counts) +
  aes(area = percent, fill = status) +
  geom_treemap() +
  geom_treemap_text(aes(
    label = paste(status, count, paste0("(", round(percent, 2), "%)"),
                  sep = "\n"),
    fontface = c("bold")
  ))

tm
```

This plot can be customized with ggplot2, for example we only want to display the percentage:

```{r}
tm <- ggplot(ion_counts) +
  aes(area = percent, fill = status) +
  geom_treemap() +
  geom_treemap_text(aes(
    label = paste(status, paste0("(", round(percent, 2), "%)"), sep = "\n"),
    fontface = c("bold")
  ))

tm
```

Or you no longer need the legend and maybe we want custom colors:

```{r}
tm +
  scale_fill_brewer(palette = "Greens") +
  theme(legend.position = "none")
```

If you want a fast visualization of the treemap, you can pass the mpactr object to the function `plot_qc_tree()`.

```{r}
plot_qc_tree(data_filtered)
```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mpactr-accessors.R
\name{get_metadata}
\alias{get_metadata}
\title{Return the meta_data data.table from the mpactr object.}
\usage{
get_metadata(mpactr_object)
}
\arguments{
\item{mpactr_object}{The mpactr object that is created by calling
the import_data() function.}
}
\value{
a \code{data.table}.
}
\description{
\code{get_metadata()} a wrapper function to return the meta data object
of the given mpactr object.
}
\examples{
 \dontshow{
limit_cores()
}
data <- import_data(
  example_path("coculture_peak_table.csv"),
  example_path("metadata.csv"),
  format = "Progenesis"
)

meta_data <- get_metadata(data)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/quality_control.R
\name{qc_summary}
\alias{qc_summary}
\title{Summary of Filtering}
\usage{
qc_summary(mpactr_object)
}
\arguments{
\item{mpactr_object}{an \code{mpactr_object}.}
}
\value{
a \code{data.table} reporting the number of high quality ions
("Passed") or the filter in which they were removed.
}
\description{
Parses an mpactr object and extracts a summary of all applied filters.
Specifically, the fate of each input ion is reported as ion status. Status
options are: Passed, mispicked, group, replicability, and insouce. A status
of Passed ions is returned for ions that passed all applied filters and
therefore are expected to be high quality ions. Ions tagged as group,
mispicked, replicability, or ionsource were removed during the corresponding
filter.
}
\examples{
 \dontshow{
limit_cores()
}
data <- import_data(
  example_path("coculture_peak_table.csv"),
  example_path("metadata.csv"),
  format = "Progenesis"
)

data_filter <- filter_mispicked_ions(data,
  ringwin = 0.5,
  isowin = 0.01,
  trwin = 0.005,
  max_iso_shift = 3,
  merge_peaks = TRUE
)

summary <- qc_summary(data_filter)
summary
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/quality_control.R
\name{plot_qc_tree}
\alias{plot_qc_tree}
\title{Visualize Filtering Summary as Tree Map}
\usage{
plot_qc_tree(mpactr_object)
}
\arguments{
\item{mpactr_object}{an \code{mpactr_object}.}
}
\value{
a tree map plot of class \code{ggplot}.
}
\description{
\code{plot_qc_tree()} visualizes the filtering summary as a treemap. Ion
status (see \code{\link[=qc_summary]{qc_summary()}}) is reported here as percentage of all
pre-filtered ions.
}
\examples{
data <- import_data(
  example_path("coculture_peak_table.csv"),
  example_path("metadata.csv"),
  format = "Progenesis"
)

data_filter <- filter_mispicked_ions(data,
  ringwin = 0.5,
  isowin = 0.01,
  trwin = 0.005,
  max_iso_shift = 3,
  merge_peaks = TRUE
)

plot_qc_tree(data_filter)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/filters.R
\name{filter_mispicked_ions}
\alias{filter_mispicked_ions}
\title{Mispicked ions filter}
\usage{
filter_mispicked_ions(
  mpactr_object,
  ringwin = 0.5,
  isowin = 0.01,
  trwin = 0.005,
  max_iso_shift = 3,
  merge_peaks = TRUE,
  merge_method = "sum",
  copy_object = FALSE
)
}
\arguments{
\item{mpactr_object}{An \code{mpactr_object}. See \code{\link[=import_data]{import_data()}}.}

\item{ringwin}{Ringing mass window or detector saturation mass window.
Default = 0.5 atomic mass units (AMU).}

\item{isowin}{Isotopic mass window. Default = 0.01 AMU.}

\item{trwin}{A \code{numeric} denoting the retention time threshold for assessing
if ions should be merged. Default = 0.005.}

\item{max_iso_shift}{A \code{numeric}. Default = 3.}

\item{merge_peaks}{A \code{boolean} parameter to determine if peaks found to
belong to the same ion should be merged in the feature table.}

\item{merge_method}{If merge_peaks is TRUE, a method for how similar peaks
should be merged. Can be one of "sum".}

\item{copy_object}{A \code{boolean} parameter that allows users to return a copied
object instead of modifying the object.}
}
\value{
an \code{mpactr_object}.
}
\description{
\code{filter_mispicked_ions()} identifies ions that were incorrectly split into
separate features during preprocessing. This filter checks the feature table
for similar ions in terms of mass and retention time. Peaks found to be
similar are merged into a single feature given \code{merge_peaks} is \code{TRUE}.

The parameter \code{ringwin} is the detector saturation mass window, specific for
some instruments, such as Waters Synapse G2-Si-Q-ToF, to account for high
concentration samples.

Parameter \code{isowin} is the isotopic mass window, which accounts for isotopic
peaks of the same precursor mass that were incorrectly assigned during
preprocessing.

\code{copy_object}: mpactr is built on an R6 class-system, meaning it operates on
reference semantics in which data is updated \emph{in-place}. Compared to a
shallow copy, where only data pointers are copied, or a deep copy, where
the entire data object is copied in memory, any changes to the original
data object, regardless if they are assigned to a new object, result in
changes to the original data object. We recommend using the default
\code{copy_object = FALSE} as this makes for an extremely fast and
memory-efficient way to chain mpactr filters together; however, if you
would like to run the filters individually with traditional R style objects,
you can set \code{copy_object} to \code{TRUE} as shown in the filter examples.
}
\examples{
 \dontshow{
limit_cores()
}
data <- import_data(
  example_path("coculture_peak_table.csv"),
  example_path("metadata.csv"),
  format = "Progenesis"
)

data_filter <- filter_mispicked_ions(data,
  ringwin = 0.5,
  isowin = 0.01,
  trwin = 0.005,
  max_iso_shift = 3,
  merge_peaks = TRUE,
  merge_method = "sum"
)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/filters.R
\name{filter_insource_ions}
\alias{filter_insource_ions}
\title{Filter Insource ions}
\usage{
filter_insource_ions(
  mpactr_object,
  cluster_threshold = 0.95,
  copy_object = FALSE
)
}
\arguments{
\item{mpactr_object}{An \code{mpactr_object}. See \code{\link[=import_data]{import_data()}}.}

\item{cluster_threshold}{Cluster threshold for ion deconvolution.
Default = 0.95.}

\item{copy_object}{A \code{boolean} parameter that allows users to return
a copied object instead of modifying the object.}
}
\value{
an \code{mpactr_object}
}
\description{
\code{filter_insource_ions()} identifies and removes in-source ion clusters based
on a Pearson correlation threshold. Groups of co-eluting features with
identical retention time are identified and used to generate Pearson
correlation matrices. Clusters with self-similarity greater than the
user-defined \code{cluster_threshold} within these matrices are identified as
likely belonging to a single precursor ion and is associated insource ion.
Highly correlated ions are identified and removed.

\code{copy_object}: mpactr is built on an R6 class-system, meaning it operates on
reference semantics in which data is updated \emph{in-place}. Compared to a
shallow copy, where only data pointers are copied, or a deep copy, where
the entire data object is copied in memory, any changes to the original
data object, regardless if they are assigned to a new object, result in
changes to the original data object. We recommend using the default
\code{copy_object = FALSE} as this makes for an extremely fast and
memory-efficient way to chain mpactr filters together; however, if you
would like to run the filters individually with traditional R style objects,
you can set \code{copy_object} to \code{TRUE} as shown in the filter examples.
}
\examples{
 \dontshow{
limit_cores()
}
data <- import_data(
  example_path("coculture_peak_table.csv"),
  example_path("metadata.csv"),
  format = "Progenesis"
)

data_filter <- filter_insource_ions(data,
  cluster_threshold = 0.95
)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/filters.R
\name{filter_cv}
\alias{filter_cv}
\title{Filter Non-reproducible ions}
\usage{
filter_cv(mpactr_object, cv_threshold = NULL, copy_object = FALSE)
}
\arguments{
\item{mpactr_object}{An \code{mpactr_object}. See \code{\link[=import_data]{import_data()}}.}

\item{cv_threshold}{Coefficient of variation threshold.
A lower cv_threshold will result in more stringent filtering and higher
reproducibility. Recommended values between 0.2 - 0.5.}

\item{copy_object}{A \code{boolean} parameter that allows users to return a copied
object instead of modifying the object.}
}
\value{
an \code{mpactr_object}.
}
\description{
\code{filter_cv()} removes feature ions that are found to be non-reproducible
between technical injection replicates. Reproducibility is assessed via mean
coefficient of variation (CV) between technical replicates. As
such, this filter is expecting an input dataset with at least two replicate
injections per sample.

\code{copy_object}: mpactr is built on an R6 class-system, meaning it operates on
reference semantics in which data is updated \emph{in-place}. Compared to a
shallow copy, where only data pointers are copied, or a deep copy, where
the entire data object is copied in memory, any changes to the original
data object, regardless if they are assigned to a new object, result in
changes to the original data object. We recommend using the default
\code{copy_object = FALSE} as this makes for an extremely fast and
memory-efficient way to chain mpactr filters together; however, if you
would like to run the filters individually with traditional R style objects,
you can set \code{copy_object} to \code{TRUE} as shown in the filter examples.
}
\examples{
 \dontshow{
limit_cores()
}
data <- import_data(
  example_path("coculture_peak_table.csv"),
  example_path("metadata.csv"),
  format = "Progenesis"
)

data_filter <- filter_cv(data,
  cv_threshold = 0.01,
  copy_object = TRUE
)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{cultures_data}
\alias{cultures_data}
\title{LC-MS/MS sample data}
\format{
\subsection{\code{culture_data}}{

A \code{mpactr} with 2 attributes:
\describe{
\item{peak_table}{A feature table of class \code{data.table}}
\item{meta_data}{A \code{data.table} with associated sample metadata}
}
}
}
\usage{
cultures_data
}
\value{
An \code{mpactr} R6 class object.
}
\description{
A \code{mpactr} R6 class object containing a feature table and associated sample
metadata.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mpactR-package.R
\docType{package}
\name{mpactr-package}
\alias{mpactr}
\alias{mpactr-package}
\title{mpactr: Correction of Preprocessed MS Data}
\description{
\if{html}{\figure{logo.png}{options: style='float: right' alt='logo' width='120'}}

An 'R' implementation of the 'python' program Metabolomics Peak Analysis Computational Tool ('MPACT') (Robert M. Samples, Sara P. Puckett, and Marcy J. Balunas (2023) \doi{10.1021/acs.analchem.2c04632}). Filters in the package serve to address common errors in tandem mass spectrometry preprocessing, including: (1) isotopic patterns that are incorrectly split during preprocessing, (2) features present in solvent blanks due to carryover between samples, (3) features whose abundance is greater than user-defined abundance threshold in a specific group of samples, for example media blanks, (4) ions that are inconsistent between technical replicates, and (5) in-source fragment ions created during ionization before fragmentation in the tandem mass spectrometry workflow.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://www.mums2.org/mpactr/}
  \item \url{https://github.com/mums2/mpactr}
  \item Report bugs at \url{https://github.com/mums2/mpactr/issues}
}

}
\author{
\strong{Maintainer}: Patrick Schloss \email{pschloss@umich.edu} (\href{https://orcid.org/0000-0002-6935-4275}{ORCID}) [copyright holder]

Authors:
\itemize{
  \item Allison Mason \email{masonar@umich.edu} (\href{https://orcid.org/0000-0003-1339-1592}{ORCID})
  \item Gregory Johnson \email{grejoh@umich.edu} (\href{https://orcid.org/0009-0008-3890-0297}{ORCID})
}

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/filter_pactr-accessors.R
\name{get_cv_data}
\alias{get_cv_data}
\title{Get CV values.}
\usage{
get_cv_data(mpactr_object)
}
\arguments{
\item{mpactr_object}{The mpactr object that is created by calling
the import_data() function.}
}
\value{
a \code{data.table} reporting the mean and median coefficient
of variation for each input ion.
}
\description{
\code{get_cv_data()} is a wrapper function to return cv (coefficient of
variation) calculated with \code{\link[=filter_cv]{filter_cv()}}.
}
\examples{
 \dontshow{
limit_cores()
}
data <- import_data(
  example_path("coculture_peak_table.csv"),
  example_path("metadata.csv"),
  format = "Progenesis"
)

data_filter <- filter_cv(data,
  cv_threshold = 0.01,
)

cv <- get_cv_data(data_filter)
head(cv)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mpactr-accessors.R
\name{get_raw_data}
\alias{get_raw_data}
\title{Return the input peak table from mpactr object.}
\usage{
get_raw_data(mpactr_object)
}
\arguments{
\item{mpactr_object}{The mpactr object that is created by calling the
import_data() function.}
}
\value{
a \code{data.table}.
}
\description{
\code{get_raw_data} a wrapper function to return the meta data object of the
given mpactr object.
}
\examples{
 \dontshow{
limit_cores()
}
data <- import_data(
  example_path("coculture_peak_table.csv"),
  example_path("metadata.csv"),
  format = "Progenesis"
)

raw_data <- get_raw_data(data)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\name{example_path}
\alias{example_path}
\title{Get file paths for examples}
\usage{
example_path(file = NULL)
}
\arguments{
\item{file}{Name of a file. If \code{NULL}, all examples files will be listed.}
}
\value{
A file path to example data stored in the \code{inst/extdata} directory
of the package.
}
\description{
mpactr contains a number of example files in the \code{inst/extdata} directory.
This function makes them accessible in documentation that shows how file
paths are used in function examples.
}
\examples{
example_path()

example_path("metadata.csv")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/filter_pactr-accessors.R
\name{filter_summary}
\alias{filter_summary}
\title{Return the summary for a single mpactr filter.}
\usage{
filter_summary(mpactr_object, filter, group = NULL)
}
\arguments{
\item{mpactr_object}{The mpactr object that is created by calling
the import_data() function.}

\item{filter}{The name of a filter whose summary is to be extracted.
Must be one of: "mispicked", "group", "replicability", or "insource".}

\item{group}{If filter = "group", the name of the Biological_Group
used to filter.}
}
\value{
a \code{list} reporting 1) compound ids for compounds which failed
the filter and 2) compound ids for compounds which passed the filter.
}
\description{
\code{filter_summary()} is a wrapper function to return the summary
from a single filter within the given mpactr object.
}
\examples{
 \dontshow{
limit_cores()
}
data <- import_data(example_path("coculture_peak_table.csv"),
  example_path("metadata.csv"),
  format = "Progenesis"
)

data_filter <- filter_mispicked_ions(data)

mispicked_summary <- filter_summary(data_filter, filter = "mispicked")
mispicked_summary

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/filters.R
\name{filter_group}
\alias{filter_group}
\title{Filter Ions by Group}
\usage{
filter_group(
  mpactr_object,
  group_threshold = 0.01,
  group_to_remove,
  remove_ions = TRUE,
  copy_object = FALSE
)
}
\arguments{
\item{mpactr_object}{An \code{mpactr_object}. See \code{\link[=import_data]{import_data()}}.}

\item{group_threshold}{Relative abundance threshold at which to remove ions.
Default = 0.01.}

\item{group_to_remove}{Biological group name to remove ions from.}

\item{remove_ions}{A \code{boolean} parameter. If \code{TRUE} failing ions will be
removed from the peak table. Default = TRUE.}

\item{copy_object}{A \code{boolean} parameter that allows users to return a copied
object instead of modifying the object.}
}
\value{
an \code{mpactr_object}.
}
\description{
Filter Ions by Group
}
\details{
\code{filter_group()} removes feature ions that are present in a user-defined
group based on a relative abundance threshold. This could be particularly
useful to filter out features found present in solvent blank samples.
Further, this filter can be utilized to remove features in media blank
sample for experiments on microbial cultures.
The presence or absence of features in a group of samples is determined by
first averaging injection replicates and then averaging biological
replicates within each biological treatment group. A feature is present in
a group if its abundance is greater than the user-defined \code{group_threshold}.
The default is 0.01, meaning a feature is removed if its abundance is 1\% of
that in the sample group in which it is most abundant. For example, blank
filtering can remove features whose mean abundance in solvent blank
injections is greater than 1\% of their maximum mean abundance in experimental
samples.

If you would like to remove features found in media blank
samples, we recommend testing the \code{group_threshold} parameter.

\code{copy_object}: mpactr is built on an R6 class-system, meaning it operates on
reference semantics in which data is updated \emph{in-place}. Compared to a
shallow copy, where only data pointers are copied, or a deep copy, where
the entire data object is copied in memory, any changes to the original
data object, regardless if they are assigned to a new object, result in
changes to the original data object. We recommend using the default
\code{copy_object = FALSE} as this makes for an extremely fast and
memory-efficient way to chain mpactr filters together; however, if you
would like to run the filters individually with traditional R style objects,
you can set \code{copy_object} to \code{TRUE} as shown in the filter examples.
}
\examples{
 \dontshow{
limit_cores()
}
data <- import_data(
  example_path("coculture_peak_table.csv"),
  example_path("metadata.csv"),
  format = "Progenesis"
)

data_filter <- filter_group(data,
  group_threshold = 0.01,
  group_to_remove = "Blanks",
  remove_ions = TRUE
)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/filter_pactr-accessors.R
\name{get_group_averages}
\alias{get_group_averages}
\title{Get groups averages.}
\usage{
get_group_averages(mpactr_object)
}
\arguments{
\item{mpactr_object}{The mpactr object that is created by calling the
import_data() function.}
}
\value{
a \code{data.table} reporting the average and relative standard
deviation across biological groups and technical replicates within
each group.
}
\description{
\code{get_group_averages()} is a wrapper function to return group averages
for the filtered peak table.
}
\examples{
 \dontshow{
limit_cores()
}
data <- import_data(
  example_path("coculture_peak_table.csv"),
  example_path("metadata.csv"),
  format = "Progenesis"
)

data_filter <- filter_group(data, group_to_remove = "Blanks")

group_averages <- get_group_averages(data_filter)
head(group_averages)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/import_data.R
\name{import_data}
\alias{import_data}
\title{Import data into an mpactr object.}
\usage{
import_data(peak_table, meta_data, format = "none")
}
\arguments{
\item{peak_table}{The file path or valid \code{https} url to your feature table
file.}

\item{meta_data}{The file path to your meta_data file or \code{data.frame}.}

\item{format}{The expected exported type of your peak table, can be
one of "Progenesis", "Metaboscape", "None".}
}
\value{
an \code{mpactr_object}.
}
\description{
\code{import_data()} takes two file paths, one for the pre-processed feature
table and one for sample metadata. Both files should be .csv.
}
\details{
mpactr requires a peak table and meta data as input. Files are expected to be
comma separated files (\emph{.csv}).
\enumerate{
\item \code{peak_table}: a peak table where rows are expected to be compounds. mpactr
supports import of feature table files from multiple tools through the
\code{format} argument. Currently supported value for \code{format} are "Progenesis",
"Metaboscape", or "None".
}

\code{format} = "Progenesis." allows users to provide a feature table exported by
Progenesis. To export a compatible peak table in Progenesis, navigate to the
\emph{Review Compounds} tab then File -> Export Compound Measurements. Select
the following properties: Compound, m/z, Retention time (min), and Raw
abundance and click ok.

\code{format} = "Metaboscape" allows users to provide a feature table exported by
Metaboscape with default settings. The import function will save the raw peak
table in the \code{mpactr_object} and store a formatted peak table for filtering.
Reformatting includes selecting "FEATURE_ID", "RT", "PEPMASS", and sample
columns. Sample columns are determined from the "Injection" column in
\code{meta_data} (see below). "PEPMASS" is converted to m/z using the "ADDUCT"
column and compound metadata columns are renamed for mpactr.

\code{format} = "None" allows users to provide a feature table file in the
expected format. This can be useful if you have a file from another tool and
want to manually format it in R. The table rows are expected to be individual
features, while columns are compound metadata and samples. The feature table
must have the compound metadata columns "Compound", "mz", and "rt". Where
"Compound" is the compound id, and can be \code{numeric} or \code{character}. "mz" is
the compound m/z, and should be \code{numeric}. "rt" is the retention time, in
minutes, and should be \code{numeric}. The remaining columns should be samples,
and match the names in the "Injection" column of the \code{meta_data} file.
2. \code{meta_data}: a table with sample information. Either a file path or
\code{data.frame} can be supplied. At minimum the following columns are expected:
"Injection", "Sample_Code", and "Biological_Group". "Injection" is the sample
name and is expected to match sample column names in the \code{peak_table}.
"Sample_Code" is the id for technical replicate groups. "Biological_Group"
is the id for biological replicate groups. Other sample metadata can be
added, and is encouraged for downstream analysis following filtering with
mpactr.
}
\examples{
 \dontshow{
limit_cores()
}
data <- import_data(
  example_path("coculture_peak_table.csv"),
  example_path("metadata.csv"),
  format = "Progenesis"
)

meta_data <- read.csv(example_path("metadata.csv"))
data <- import_data(example_path("coculture_peak_table.csv"),
  meta_data,
  format = "Progenesis"
)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{limit_cores}
\alias{limit_cores}
\title{Limit Cores}
\usage{
limit_cores()
}
\value{
does not return anything.
}
\description{
Limits the amount of cores used to two. Mostly used in test
and examples so we can pass cran checks.
}
\examples{
limit_cores()

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/filter_pactr-accessors.R
\name{get_similar_ions}
\alias{get_similar_ions}
\title{Get similar ion groups.}
\usage{
get_similar_ions(mpactr_object)
}
\arguments{
\item{mpactr_object}{The mpactr object that is created by calling the
import_data() function.}
}
\value{
a \code{data.table} reporting the main ion and those found to be
similar with \code{\link[=filter_mispicked_ions]{filter_mispicked_ions()}}.
}
\description{
\code{get_similar_ions()} is a wrapper function to return similar ion groups
determined with the \code{\link[=filter_mispicked_ions]{filter_mispicked_ions()}}.
}
\examples{
 \dontshow{
limit_cores()
}
data <- import_data(
  example_path("coculture_peak_table.csv"),
  example_path("metadata.csv"),
  format = "Progenesis"
)

data_filter <- filter_mispicked_ions(data)

mispicked_ion_groups <- get_similar_ions(data_filter)
mispicked_ion_groups

}
